package com.c503.ca.cadv.common.util;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * 字节工具类
 *
 * @author liaow
 */
@SuppressWarnings({"unused"})
public class ByteUtils {

    public final static short UNSIGNED_MAX_VALUE = (Byte.MAX_VALUE * 2) + 1;

    private ByteUtils() {
    }

    /**
     * int整数转换为4字节的byte数组
     *
     * @param i 整数
     * @return byte数组
     */
    public static byte[] intToBytes(int i) {
        byte[] targets = new byte[4];
        targets[3] = (byte) (i & 0xFF);
        targets[2] = (byte) (i >> 8 & 0xFF);
        targets[1] = (byte) (i >> 16 & 0xFF);
        targets[0] = (byte) (i >> 24 & 0xFF);
        return targets;
    }

    /**
     * long整数转换为8字节的byte数组
     *
     * @param lo long整数
     * @return byte数组
     */
    public static byte[] longToBytes(long lo) {
        byte[] targets = new byte[8];
        for (int i = 0; i < 8; i++) {
            int offset = (targets.length - 1 - i) * 8;
            targets[i] = (byte) ((lo >>> offset) & 0xFF);
        }
        return targets;
    }

    /**
     * short整数转换为2字节的byte数组
     *
     * @param s short整数
     * @return byte数组
     */
    public static byte[] unsignedShortToBytes(int s) {
        byte[] targets = new byte[2];
        targets[0] = (byte) (s >>> 8 & 0xFF);
        targets[1] = (byte) (s & 0xFF);
        return targets;
    }

    public static byte[] unsignedIntTobytes(long value) {
        byte[] targets = new byte[4];
        for (int i = 0; i < 4; i++) {
            int offset = (targets.length - 1 - i) * 8;
            targets[i] = (byte) ((value >>> offset) & 0xff);
        }

        return targets;
    }

    /**
     * byte数组转换为无符号short整数
     *
     * @param bytes byte数组
     * @return short整数
     */
    public static int bytesToUnsignedShort(byte[] bytes) {
        return bytesToUnsignedShort(bytes, 0);
    }

    /**
     * byte数组转换为无符号short整数
     *
     * @param bytes byte数组
     * @param off   开始位置
     * @return short整数
     */
    public static int bytesToUnsignedShort(byte[] bytes, int off) {
        int high = bytes[off];
        int low = bytes[off + 1];
        return (high << 8 & 0xFF00) | (low & 0xFF);
    }

    /**
     * byte数组转换为int整数
     *
     * @param bytes byte数组
     * @param off   开始位置
     * @return int整数
     */
    public static int byte4ToInt(byte[] bytes, int off) {
        int b0 = bytes[off] & 0xFF;
        int b1 = bytes[off + 1] & 0xFF;
        int b2 = bytes[off + 2] & 0xFF;
        int b3 = bytes[off + 3] & 0xFF;
        return (b0 << 24) | (b1 << 16) | (b2 << 8) | b3;
    }

    /**
     * @deprecated prefer unsignedPromote(...)
     */
    public static short toUnsigned(byte b) {
        return (short) (b < 0 ? (UNSIGNED_MAX_VALUE + 1) + b : b);
    }

    public static int unsignedPromote(byte b) {
        return b & 0xff;
    }

    public static String toHexAscii(byte b) {
        StringWriter sw = new StringWriter(2);
        addHexAscii(b, sw);
        return sw.toString();
    }

    public static String toLowercaseHexAscii(byte b) {
        StringWriter sw = new StringWriter(2);
        addLowercaseHexAscii(b, sw);
        return sw.toString();
    }

    public static String toHexAscii(byte[] bytes) {
        int len = bytes.length;
        StringWriter sw = new StringWriter(len * 2);
        for (byte aByte : bytes) {
            addHexAscii(aByte, sw);
        }
        return sw.toString();
    }

    public static String toLowercaseHexAscii(byte[] bytes) {
        int len = bytes.length;
        StringWriter sw = new StringWriter(len * 2);
        for (byte aByte : bytes) {
            addLowercaseHexAscii(aByte, sw);
        }
        return sw.toString();
    }

    public static byte[] fromHexAscii(String s) throws NumberFormatException {
        try {
            int len = s.length();
            if ((len % 2) != 0) {
                throw new NumberFormatException("Hex ascii must be exactly two digits per byte.");
            }

            int outLen = len / 2;
            byte[] out = new byte[outLen];
            int i = 0;
            StringReader sr = new StringReader(s);
            while (i < outLen) {
                int val = (16 * fromHexDigit(sr.read())) + fromHexDigit(sr.read());
                out[i++] = (byte) val;
            }
            return out;
        } catch (IOException e) {
            throw new InternalError("IOException reading from StringReader?!?!");
        }
    }

    static void addHexAscii(byte b, StringWriter sw) {
        int ub = unsignedPromote(b);
        int h1 = ub / 16;
        int h2 = ub % 16;
        sw.write(toHexDigit(h1));
        sw.write(toHexDigit(h2));
    }

    static void addLowercaseHexAscii(byte b, StringWriter sw) {
        int ub = unsignedPromote(b);
        int h1 = ub / 16;
        int h2 = ub % 16;
        sw.write(toLowercaseHexDigit(h1));
        sw.write(toLowercaseHexDigit(h2));
    }

    private static int fromHexDigit(int c) throws NumberFormatException {
        if (c >= 0x30 && c < 0x3A) {
            return c - 0x30;
        } else if (c >= 0x41 && c < 0x47) {
            return c - 0x37;
        } else if (c >= 0x61 && c < 0x67) {
            return c - 0x57;
        } else {
            throw new NumberFormatException('\'' + c + "' is not a valid hexadecimal digit.");
        }
    }

    /**
     * note: we do no arg. checking, because
     * we only ever call this from addHexAscii()
     * above, and we are sure the args are okay
     **/
    private static char toHexDigit(int h) {
        char out;
        if (h <= 9) {
            out = (char) (h + 0x30);
        } else {
            out = (char) (h + 0x37);
        }
        return out;
    }


    /**
     * note: we do no arg. checking, because we only ever call this from addHexAscii()
     * above, and we are sure the args are okay
     **/
    private static char toLowercaseHexDigit(int h) {
        char out;
        if (h <= 9) {
            out = (char) (h + 0x30);
        } else {
            out = (char) (h + 0x57);
        }
        return out;
    }

    public static String bytesToHexString(byte[] src) {
        StringBuilder sb = new StringBuilder();
        if (src == null || src.length <= 0) {
            return null;
        }
        for (byte b : src) {
            int v = b & 0xFF;
            String hv = Integer.toHexString(v).toUpperCase();
            if (hv.length() < 2) {
                sb.append(0);
            }
            sb.append(hv);
        }
        return sb.toString();
    }

    public static byte[] hex2bytes(String inHex) {
        int hexlen = inHex.length();
        byte[] result;
        if (hexlen % 2 == 1) {
            //奇数
            hexlen++;
            result = new byte[(hexlen / 2)];
            inHex = "0" + inHex;
        } else {
            //偶数
            result = new byte[(hexlen / 2)];
        }
        int j = 0;
        for (int i = 0; i < hexlen; i += 2) {
            result[j] = hexToByte(inHex.substring(i, i + 2));
            j++;
        }
        return result;
    }

    public static byte[] charToBytes(char c) {
        byte[] b = new byte[2];
        b[0] = (byte) ((c & 0xFF00) >> 8);
        b[1] = (byte) (c & 0xFF);
        return b;
    }

    public static byte hexToByte(String inHex) {
        return (byte) Integer.parseInt(inHex, 16);
    }

    /**
     * 从byte数组的index处的连续4个字节获得一个int
     */
    public static short bytesToShort(byte[] arr) {
        return ByteBuffer.wrap(arr).getShort();
    }

    public static byte[] shortToBytesLE(Short value) {
        byte[] byteArr = new byte[2];
        ByteBuffer.wrap(byteArr).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer().put(value);
        return byteArr;
    }

    public static byte[] shortToBytes(Short value) {
        byte[] byteArr = new byte[2];
        ByteBuffer.wrap(byteArr).asShortBuffer().put(value);
        return byteArr;
    }

    /**
     * 从byte数组的index处的连续4个字节获得一个int
     */
    public static int bytesToInt(byte[] arr) {
        return bytesToInt(arr, 0);
    }

    /**
     * 从byte数组的index处的连续4个字节获得一个int
     */
    public static int bytesToInt(byte[] arr, int index) {
        return ByteBuffer.wrap(arr).getInt(index);
    }


    public static byte[] floatToBytes(float f) {
        //将float里面的二进制串解释为int整数
        int intbits = Float.floatToIntBits(f);
        return intToBytes(intbits);
    }

    public static byte[] doubleToBytes(double d) {
        //将float里面的二进制串解释为int整数
        long intbits = Double.doubleToLongBits(d);
        return longToBytes(intbits);
    }

    /**
     * 从byte数组的0处的连续4个字节获得一个float
     */
    public static float bytesToFloat(byte[] arr) {
        return ByteBuffer.wrap(arr).getFloat();
        //return Float.intBitsToFloat(bytesToInt(arr, 0));
    }

    /**
     * 从byte数组的从byte数组的index处的连续4个字节获得一个float处的连续4个字节获得一个float
     */
    public static float bytesToFloat(byte[] arr, int index) {
        return ByteBuffer.wrap(arr).getFloat(index);
        //return Float.intBitsToFloat(bytesToInt(arr, index));
    }

    public static char bytesToChar(byte[] bytes) {
        return ByteBuffer.wrap(bytes).getChar();
    }

    public static String bytesToString(byte[] bytes) {
        return ByteBuffer.wrap(bytes).toString();
    }

    public static long bytesToLong(byte[] bytes) {
        return ByteBuffer.wrap(bytes).getLong();
    }

    public static double bytesToDouble(byte[] bytes) {
        return ByteBuffer.wrap(bytes).getDouble();
    }

    /**
     * 字节和校验
     */
    public static Short checkSum(byte[] packBytes) {
        int checkSum = 0;
        for (byte packByte : packBytes) {
            checkSum += packByte;
        }
        //取低八位
        checkSum &= 0xff;
        return (short) checkSum;
    }

    /**
     * 16 位 CRC 计算
     */
    public static short crc16(byte[] bytes) {
        int crc = 0x0000ffff;
        int polynomial = 0x0000a001;

        int i, j;
        for (i = 0; i < bytes.length; i++) {
            crc ^= (int) bytes[i] & 0x000000ff;
            for (j = 0; j < 8; j++) {
                if ((crc & 0x00000001) == 1) {
                    crc >>= 1;
                    crc ^= polynomial;
                } else {
                    crc >>= 1;
                }
            }
        }
        //高低位转换，看情况使用
        // CRC = ((CRC & 0x0000FF00) >> 8) | ((CRC & 0x000000FF) << 8);
        return (short) crc;
    }

    public static Integer getBitValue(Integer value, Integer start, Integer end) {
        if (value == null) {
            return null;
        }
        start = start == null ? 0 : start;
        end = end == null ? 31 : end;

        int bitValue;

        int bigIndex = start > end ? start : end;
        int smallIndex = start > end ? end : start;

        int andValue = (0xFFFFFFFF >>> (31 - bigIndex)) & (0xFFFFFFFF << smallIndex);

        bitValue = andValue & value;
        bitValue = bitValue >>> start;
        return bitValue;
    }


    /**
     * 根据位，设置int值
     *
     * @param bytes 所有参数内容字节数组
     */
    public static void setBitValue32(byte[] bytes, Integer targetValue, int attrLength, int offset, Integer startBit, Integer endBit) {
        if (targetValue == null) {
            return;
        }
        byte[] oriBytes = new byte[attrLength];
        System.arraycopy(bytes, offset, oriBytes, 0, attrLength);
        int oriValue = 0;

        int andValue = (0xFFFFFFFF >>> (31 - endBit)) & (0xFFFFFFFF << startBit);
        targetValue = targetValue << startBit;
        targetValue &= andValue;

        if (attrLength == 1) {
            oriValue = oriBytes[0];
        } else if (attrLength == 2) {
            oriValue = bytesToShort(oriBytes);
        } else if (attrLength == 4) {
            oriValue = bytesToInt(oriBytes);
        }

        int xorValue = ~andValue;
        xorValue &= oriValue;

        int newValue = xorValue | targetValue;
        oriBytes = intToBytes(newValue);
        System.arraycopy(oriBytes, 4 - attrLength, bytes, offset, attrLength);
    }

    /**
     * 根据位，设置int值
     */
    public static Integer setBitValue32(Integer oriValue, Integer targetValue, Integer startBit, Integer endBit) {
        if (targetValue == null) {
            return oriValue;
        }

        int andValue = (0xFFFFFFFF >>> (31 - endBit)) & (0xFFFFFFFF << startBit);
        targetValue = targetValue << startBit;
        targetValue &= andValue;

        int xorValue = ~andValue;
        xorValue &= oriValue;

        return xorValue | targetValue;
    }
}
