package com.c503.ca.cadv.common.util;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateFormatUtils;

import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

/**
 * 时间工具类
 */
@Slf4j
@SuppressWarnings({"unused"})
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {

    public static final String FILE_HOUR_PATTERN = "yyyy-MM-dd-HH";
    public static final String DAY_PATTERN = "yyyy-MM-dd";
    public static final String[] parsePatterns = {"yyyy-MM-dd HH:mm:ss.SSS", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd", "yyyy-MM-dd HH:mm", "yyyy-MM",
        "yyyyMMdd HHmmss", "yyyyMMddHHmmss", "yyyyMMdd HH:mm:ss", "yyyyMMdd", "yyyyMM",
        "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM/dd", "yyyy/MM",
        "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM.dd", "yyyy.MM", "yyyy"};

    /**
     * 得到当前日期字符串 格式（yyyy-MM-dd）
     */
    public static String getDate() {
        return getDate("yyyy-MM-dd");
    }

    /**
     * 得到当前日期字符串 格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
     */
    public static String getDate(String pattern) {
        return DateFormatUtils.format(new Date(), pattern);
    }

    public static String getDate(String pattern, TimeZone timeZone) {
        return DateFormatUtils.format(new Date(), pattern, timeZone);
    }

    /**
     * 得到日期字符串 默认格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
     */
    public static String formatDate(Date date, Object... pattern) {
        return formatDate(date, SystemProperties.getInstance().getBusinessTimeZone(), pattern);
    }

    public static String formatDateToCN(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int year = calendar.get(Calendar.YEAR);
        int month = (calendar.get(Calendar.MONTH) + 1);
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        int hour = calendar.get(Calendar.HOUR);
        int minute = calendar.get(Calendar.MINUTE);
        return year + "年" + (month >= 10 ? month : "0" + month) + "月" + day + "日" + hour + "时" + minute + "分";
    }

    public static String formatDate(Date date, TimeZone timeZone, Object... pattern) {
        String formatDate;
        if (pattern != null && pattern.length > 0) {
            formatDate = DateFormatUtils.format(date, pattern[0].toString(), timeZone);
        } else {
            formatDate = DateFormatUtils.format(date, "yyyy-MM-dd", timeZone);
        }
        return formatDate;
    }

    /**
     * 得到年月时间字符串，转换格式（yyyy-MM）
     */
    public static String getYearAndMonth(Date date) {
        return formatDate(date, "yyyy-MM");
    }

    /**
     * 得到日期时间字符串，转换格式（yyyy-MM-dd HH:mm:ss）
     */
    public static String formatDateTime(Date date) {
        return formatDate(date, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 得到当前时间字符串 格式（HH:mm:ss）
     */
    public static String getTime() {
        return formatDate(new Date(), "HH:mm:ss");
    }

    /**
     * 得到当前日期和时间字符串 格式（yyyy-MM-dd HH:mm:ss）
     */
    public static String getDateTime() {
        return formatDate(new Date(), "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 得到当前年份字符串 格式（yyyy）
     */
    public static String getYear() {
        return formatDate(new Date(), "yyyy");
    }

    /**
     * 得到当前月份字符串 格式（MM）
     */
    public static String getMonth() {
        return formatDate(new Date(), "MM");
    }

    /**
     * 得到当天字符串 格式（dd）
     */
    public static String getDay() {
        return formatDate(new Date(), "dd");
    }

    /**
     * 得到当天字符串 格式（HH）
     */
    public static String getHours(Date date, TimeZone timeZone) {
        return formatDate(date, timeZone, "HH");
    }

    /**
     * 得到当前星期字符串 格式（E）星期几
     */
    public static String getWeek() {
        return formatDate(new Date(), "E");
    }

    /**
     * 日期型字符串转化为日期
     */
    public static Date parseDate(String str) {
        return parseDate(str, SystemProperties.getInstance().getBusinessTimeZone());
    }

    /**
     * 字符串转日期
     * <p>时间格式模糊匹配
     */
    public static Date parseDate(String strVal, TimeZone timeZone) {
        Date date = null;
        if (StringUtils.isNotEmpty(strVal)) {
            if (strVal.contains("T")) {
                //iso8601，时区参数已经在字符串里面，无需外部参数
                DateTimeFormatter timeFormatter = DateTimeFormatter.ISO_DATE_TIME;
                OffsetDateTime offsetDateTime = OffsetDateTime.parse(strVal, timeFormatter);
                date = Date.from(Instant.from(offsetDateTime));
            } else {
                Locale locale = Locale.getDefault();
                final Calendar calendar = Calendar.getInstance(timeZone, locale);
                calendar.setLenient(false);
                final ParsePosition pos = new ParsePosition(0);
                for (final String parsePattern : parsePatterns) {
                    FastDateParser fdp = new FastDateParser(parsePattern, timeZone, locale);
                    calendar.clear();
                    try {
                        if (fdp.parse(strVal, pos, calendar) && pos.getIndex() == strVal.length()) {
                            return calendar.getTime();
                        }
                    } catch (IllegalArgumentException ignore) {
                        log.warn("IllegalArgumentException: {}", ignore.getMessage());
                    }
                    pos.setIndex(0);
                }
            }
        }
        return date;
    }

    /**
     * 通过格式，时区转换时间
     */
    public static Date parseDate(Object str, String pattern, final TimeZone timeZone) {
        if (str == null) {
            return null;
        }
        final SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        sdf.setTimeZone(timeZone);
        try {
            return sdf.parse(str.toString());
        } catch (ParseException e) {
            log.warn("时间转换失败: {}", e.getMessage());
            return null;
        }
    }

    /**
     * 将"yyyyMMddHHmmss"转换为"yyyy.MM.dd HH:mm:ss"
     */
    public static String parseString(String str) {
        if (str == null) {
            return null;
        }
        try {
            char[] strArray = str.toCharArray();
            StringBuilder newStr = new StringBuilder();
            for (int i = 0; i < strArray.length; i++) {
                if (i == 4 || i == 6) {
                    newStr.append(".");
                } else if (i == 8) {
                    newStr.append(" ");
                } else if (i == 10 || i == 12) {
                    newStr.append(":");
                }
                newStr.append(strArray[i]);
            }
            return newStr.toString();
        } catch (IndexOutOfBoundsException e) {
            return null;
        }

    }

    /**
     * 获取过去的天数
     *
     * @param date 日期对象
     * @return 天数
     */
    public static long pastDays(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (24 * 60 * 60 * 1000);
    }

    /**
     * 获取过去的小时
     *
     * @param date 日期对象
     * @return 小时
     */
    public static long pastHour(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (60 * 60 * 1000);
    }

    /**
     * 获取过去的分钟
     *
     * @param date 日期对象
     * @return 分钟
     */
    public static long pastMinutes(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (60 * 1000);
    }

    /**
     * 转换为时间（天,时:分:秒.毫秒）
     *
     * @param timeMillis 毫秒
     * @return 时间字符串
     */
    public static String formatDateTime(long timeMillis) {
        long day = timeMillis / (24 * 60 * 60 * 1000);
        long hour = (timeMillis / (60 * 60 * 1000) - day * 24);
        long min = ((timeMillis / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (timeMillis / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        long sss = (timeMillis - day * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000 - s * 1000);
        return (day > 0 ? day + "," : "") + hour + ":" + min + ":" + s + "." + sss;
    }

    /**
     * 获取两个日期之间的天数
     *
     * @param before 开始时间
     * @param after  结束时间
     * @return 天数
     */
    public static double getDistanceOfTwoDate(Date before, Date after) {
        long beforeTime = before.getTime();
        long afterTime = after.getTime();
        return (double) (afterTime - beforeTime) / (1000 * 60 * 60 * 24);
    }

    // 01. java.util.Date --> java.time.LocalDateTime
    public static LocalDateTime convertDateToLocalDateTime(Date date) {
        Instant instant = date.toInstant();
        ZoneId zone = ZoneId.systemDefault();
        return LocalDateTime.ofInstant(instant, zone);
    }

    /**
     * 取得当月天数
     */
    public static int getCurrentMonthLastDay() {
        Calendar a = Calendar.getInstance();
        a.set(Calendar.DATE, 1);//把日期设置为当月第一天
        a.roll(Calendar.DATE, -1);//日期回滚一天，也就是最后一天
        return a.get(Calendar.DATE);
    }

    /**
     * 获取指定日期当月的第一天
     *
     * @param dateStr 时间字符串
     * @param format  格式
     * @return 日期对象
     */
    public static Date getFirstDayOfGivenMonth(String dateStr, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        try {
            Date date = sdf.parse(dateStr);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.add(Calendar.MONTH, 0);
            return calendar.getTime();
        } catch (ParseException e) {
            log.warn("时间转换错误: {}", e.getMessage());
            throw new RuntimeErrorException("时间转换错误");
        }
    }

    /**
     * 获取指定日期当月的第一天的开始时间
     *
     * @param date 日期对象
     * @return 当月的第一天的开始时间
     */
    public static Date getFirstDayOfGivenMonth(Date date) {
        try {
            Calendar calendar = startTimeOfMonth(date, null);
            calendar.add(Calendar.MONTH, 0);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定日期当月的第一天的开始时间错误: {}", e.getMessage());
        }
        return null;
    }

    private static Calendar startTimeOfMonth(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar;
    }

    /**
     * 获取指定日期下个月的第一天
     *
     * @param date 日期对象
     * @return 下个月的第一天
     */
    public static Date getFirstDayOfNextMonth(Date date) {
        try {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.add(Calendar.MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期的前一小时点时间
     *
     * @return 前一小时点时间
     */
    public static Date getLastHoursStartTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.HOUR, -1);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    public static Date getLastHoursEndTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.add(Calendar.MILLISECOND, -1);
        return calendar.getTime();
    }

    /**
     * 获取指定日期的前一天0点时间
     *
     * @return 前一天0点时间
     */
    public static Date getLastDayStartTime(Date date) {
        return getLastDayStartTime(date, null);
    }

    public static Date getLastDayStartTime(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    public static Date getGivenHourStartTime(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 获取指定日期的当天0点时间
     *
     * @return 当天0点时间
     */
    public static Date getGivenDayStartTime(Date date) {
        return getGivenDayStartTime(date, null);
    }

    public static Date getGivenDayStartTime(Date date, TimeZone timeZone) {
        return startTimeOfDay(date, timeZone);
    }

    private static Date startTimeOfDay(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    public static Date getGivenDayEndTime(Date date) {
        return getGivenDayEndTime(date, null);
    }

    public static Date getGivenDayEndTime(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.add(Calendar.MILLISECOND, -1);
        return calendar.getTime();
    }

    /**
     * 周一作为第一天
     */
    public static Date getGivenWeekStartTime(Date date) {
        return getGivenWeekStartTime(date, null);
    }

    public static Date getGivenWeekStartTime(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    /**
     * 周日作为最后一天
     */
    public static Date getGivenWeekEndTime(Date date) {
        return getGivenWeekEndTime(date, null);
    }

    public static Date getGivenWeekEndTime(Date date, TimeZone timeZone) {
        Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
        calendar.setTime(date);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        calendar.add(Calendar.DAY_OF_MONTH, 1);
        calendar.add(Calendar.WEEK_OF_MONTH, 1);
        calendar.add(Calendar.MILLISECOND, -1);
        return calendar.getTime();
    }

    /**
     * 获取指定日期当月开始时间
     *
     * @return 当月开始时间
     */
    public static Date getGivenMonthStartTime(Date date) {
        try {
            return startTimeOfMonth(date, null).getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    public static Date getGivenMonthStartTime(Date date, TimeZone timeZone) {
        try {
            return startTimeOfMonth(date, timeZone).getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期当月最后一天24点时间
     *
     * @return 当月最后一天24点时间
     */
    public static Date getGivenMonthEndTime(Date date) {
        try {
            Calendar calendar = startTimeOfMonth(date, null);
            calendar.add(Calendar.MONTH, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    public static Date getGivenMonthEndTime(Date date, TimeZone timeZone) {
        try {
            Calendar calendar = startTimeOfMonth(date, timeZone);
            calendar.add(Calendar.MONTH, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期当季度开始时间
     *
     * @return 当季度开始时间
     */
    public static Date getGivenQuarterStartTime(Date date) {
        try {
            return getGivenQuarterStartTime(date, null);
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    public static Date getGivenQuarterStartTime(Date date, TimeZone timeZone) {
        try {
            Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
            calendar.setTime(date);

            int quarterIndex = calendar.get(Calendar.MONTH) / 3;
            calendar.set(Calendar.MONTH, quarterIndex * 3);
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期当季度最后一天24点时间
     *
     * @return 当季度最后一天24点时间
     */
    public static Date getGivenQuarterEndTime(Date date) {
        try {
            return getGivenQuarterEndTime(date, null);
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    public static Date getGivenQuarterEndTime(Date date, TimeZone timeZone) {
        try {
            Calendar calendar = timeZone == null ? Calendar.getInstance() : Calendar.getInstance(timeZone);
            calendar.setTime(date);

            int quarterIndex = calendar.get(Calendar.MONTH) / 3;
            calendar.set(Calendar.MONTH, quarterIndex * 3 + 2);
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);

            calendar.add(Calendar.MONTH, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期当年第一天0点时间
     *
     * @return 当年第一天0点时间
     */
    public static Date getGivenYearStartTime(Date date) {
        try {
            Calendar calendar = startTimeOfYear(date, null);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    private static Calendar startTimeOfYear(Date date, TimeZone timeZone) {
        Calendar calendar = startTimeOfMonth(date, timeZone);
        calendar.set(Calendar.MONTH, Calendar.JANUARY);
        return calendar;
    }

    public static Date getGivenYearStartTime(Date date, TimeZone timeZone) {
        try {
            Calendar calendar = startTimeOfYear(date, timeZone);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 获取指定日期当年最后一天24点时间
     *
     * @return 当年最后一天24点时间
     */
    public static Date getGivenYearEndTime(Date date) {
        try {
            Calendar calendar = startTimeOfYear(date, null);
            calendar.add(Calendar.YEAR, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    public static Date getGivenYearEndTime(Date date, TimeZone timeZone) {
        try {
            Calendar calendar = startTimeOfYear(date, timeZone);
            calendar.add(Calendar.YEAR, 1);
            calendar.add(Calendar.MILLISECOND, -1);
            return calendar.getTime();
        } catch (Exception e) {
            log.warn("获取指定时间错误: {}", e.getMessage());
        }
        return null;
    }

    /**
     * 2000年1月1日到现在积日
     */
    public static Short accumulateDaysOf2000() {
        Calendar calendar = Calendar.getInstance(TimeZoneEnum.CN.value());

        calendar.set(Calendar.YEAR, 2000);
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        long milliseconds = System.currentTimeMillis() - calendar.getTimeInMillis();

        return (short) (milliseconds / (1000 * 60 * 60 * 24));
    }


    /**
     * 当日积秒
     * 当日零时到当前时间的毫秒数
     */
    public static long accumulateMillisecondsOfDay() {
        Calendar calendar = Calendar.getInstance(TimeZoneEnum.CN.value());
        calendar.setTime(new Date());

        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return System.currentTimeMillis() - calendar.getTimeInMillis();
    }

    protected static class FastDateParser extends org.apache.commons.lang3.time.FastDateParser {
        public FastDateParser(String pattern, TimeZone timeZone, Locale locale) {
            super(pattern, timeZone, locale);
        }
    }

}



