package com.c503.ca.cadv.common.util;

import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.charset.StandardCharsets;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.security.MessageDigest;
import java.util.*;

/**
 * 文件操作工具类 实现文件的创建、删除、复制、压缩、解压以及目录的创建、删除、复制、压缩解压等功能
 */
@SuppressWarnings({"unused"})
public class FileUtils extends org.apache.commons.io.FileUtils {

    private static final int BUFFER_SIZE = 2 * 1024;

    public static final String TMP_SUFFIX = ".tmp";
    /**
     * 路径分隔符
     */
    public static final String SEPARATOR = File.separator;
    /**
     * Point
     */
    public static final String POINT = ".";
    /**
     * ContentType
     */
    public static final Map<String, String> EXT_MAPS = new HashMap<>();
    private static final Logger log = LoggerFactory.getLogger(FileUtils.class);

    static {
        // image
        EXT_MAPS.put("png", "image/png");
        EXT_MAPS.put("gif", "image/gif");
        EXT_MAPS.put("bmp", "image/bmp");
        EXT_MAPS.put("ico", "image/x-ico");
        EXT_MAPS.put("jpeg", "image/jpeg");
        EXT_MAPS.put("jpg", "image/jpeg");
        // 压缩文件
        EXT_MAPS.put("zip", "application/zip");
        EXT_MAPS.put("rar", "application/x-rar");
        // doc
        EXT_MAPS.put("pdf", "application/pdf");
        EXT_MAPS.put("ppt", "application/vnd.ms-powerpoint");
        EXT_MAPS.put("xls", "application/vnd.ms-excel");
        EXT_MAPS.put("xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        EXT_MAPS.put("pptx", "application/vnd.openxmlformats-officedocument.presentationml.presentation");
        EXT_MAPS.put("doc", "application/msword");
        EXT_MAPS.put("docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        EXT_MAPS.put("txt", "text/plain");
        // 音频
        EXT_MAPS.put("mp4", "video/mp4");
        EXT_MAPS.put("flv", "video/x-flv");
    }

    public static boolean validateFilePath(String filePath) {
        return StringUtils.isNotEmpty(filePath) && !filePath.contains("../") && !filePath.contains("./");
    }

    /**
     * 获取文件名称的后缀
     *
     * @param filename 文件名 或 文件路径
     * @return 文件后缀
     */
    public static String getFilenameSuffix(String filename) {
        String suffix = null;
        String originalFilename = filename;
        if (StringUtils.isNotBlank(filename)) {
            if (filename.contains(SEPARATOR)) {
                filename = filename.substring(filename.lastIndexOf(SEPARATOR) + 1);
            }
            if (filename.contains(POINT)) {
                suffix = filename.substring(filename.lastIndexOf(POINT) + 1);
            } else {
                if (log.isErrorEnabled()) {
                    log.error("filename error without suffix : {}", originalFilename);
                }
            }
        }
        return suffix;
    }

    /**
     * 复制单个文件，如果目标文件存在，则不覆盖
     *
     * @param srcFileName  待复制的文件名
     * @param descFileName 目标文件名
     * @return 如果复制成功，则返回true，否则返回false
     */
    public static boolean copyFile(String srcFileName, String descFileName) {
        return FileUtils.copyFileCover(srcFileName, descFileName, false);
    }

    /**
     * 复制单个文件
     *
     * @param srcFileName  待复制的文件名
     * @param descFileName 目标文件名
     * @param coverlay     如果目标文件已存在，是否覆盖
     * @return 如果复制成功，则返回true，否则返回false
     */
    public static boolean copyFileCover(String srcFileName, String descFileName, boolean coverlay) {
        File srcFile = new File(srcFileName);
        // 判断源文件是否存在
        if (!srcFile.exists()) {
            log.debug("复制文件失败，源文件 " + srcFileName + " 不存在!");
            return false;
        }
        // 判断源文件是否是合法的文件
        else if (!srcFile.isFile()) {
            log.debug("复制文件失败，" + srcFileName + " 不是一个文件!");
            return false;
        }
        File descFile = new File(descFileName);
        // 判断目标文件是否存在
        if (descFile.exists()) {
            // 如果目标文件存在，并且允许覆盖
            if (coverlay) {
                log.debug("目标文件已存在，准备删除!");
                if (!FileUtils.delFile(descFileName)) {
                    log.debug("删除目标文件 " + descFileName + " 失败!");
                    return false;
                }
            } else {
                log.debug("复制文件失败，目标文件 " + descFileName + " 已存在!");
                return false;
            }
        } else {
            if (!descFile.getParentFile().exists()) {
                // 如果目标文件所在的目录不存在，则创建目录
                log.debug("目标文件所在的目录不存在，创建目录!");
                // 创建目标文件所在的目录
                if (!descFile.getParentFile().mkdirs()) {
                    log.debug("创建目标文件所在的目录失败!");
                    return false;
                }
            }
        }

        // 准备复制文件
        // 读取的位数
        int readByte;
        InputStream ins = null;
        OutputStream outs = null;
        try {
            // 打开源文件
            ins = new FileInputStream(srcFile);
            // 打开目标文件的输出流
            outs = new FileOutputStream(descFile);
            byte[] buf = new byte[1024];
            // 一次读取1024个字节，当readByte为-1时表示文件已经读取完毕
            while ((readByte = ins.read(buf)) != -1) {
                // 将读取的字节流写入到输出流
                outs.write(buf, 0, readByte);
            }
            log.debug("复制单个文件 " + srcFileName + " 到" + descFileName + "成功!");
            return true;
        } catch (Exception e) {
            log.debug("复制文件失败：" + e.getMessage());
            return false;
        } finally {
            // 关闭输入输出流，首先关闭输出流，然后再关闭输入流
            if (outs != null) {
                try {
                    outs.close();
                } catch (IOException oute) {
                    oute.printStackTrace();
                }
            }
            if (ins != null) {
                try {
                    ins.close();
                } catch (IOException ine) {
                    ine.printStackTrace();
                }
            }
        }
    }

    /**
     * 复制整个目录的内容，如果目标目录存在，则不覆盖
     *
     * @param srcDirName  源目录名
     * @param descDirName 目标目录名
     * @return 如果复制成功返回true，否则返回false
     */
    public static boolean copyDirectory(String srcDirName, String descDirName) {
        return FileUtils.copyDirectoryCover(srcDirName, descDirName, false);
    }

    /**
     * 复制整个目录的内容
     *
     * @param srcDirName  源目录名
     * @param descDirName 目标目录名
     * @param coverlay    如果目标目录存在，是否覆盖
     * @return 如果复制成功返回true，否则返回false
     */
    public static boolean copyDirectoryCover(String srcDirName, String descDirName, boolean coverlay) {
        File srcDir = new File(srcDirName);
        // 判断源目录是否存在
        if (!srcDir.exists()) {
            log.debug("复制目录失败，源目录 " + srcDirName + " 不存在!");
            return false;
        }
        // 判断源目录是否是目录
        else if (!srcDir.isDirectory()) {
            log.debug("复制目录失败，" + srcDirName + " 不是一个目录!");
            return false;
        }
        // 如果目标文件夹名不以文件分隔符结尾，自动添加文件分隔符
        String descDirNames = descDirName;
        if (!descDirNames.endsWith(File.separator)) {
            descDirNames = descDirNames + File.separator;
        }
        File descDir = new File(descDirNames);
        // 如果目标文件夹存在
        if (descDir.exists()) {
            if (coverlay) {
                // 允许覆盖目标目录
                log.debug("目标目录已存在，准备删除!");
                if (!FileUtils.delFile(descDirNames)) {
                    log.debug("删除目录 " + descDirNames + " 失败!");
                    return false;
                }
            } else {
                log.debug("目标目录复制失败，目标目录 " + descDirNames + " 已存在!");
                return false;
            }
        } else {
            // 创建目标目录
            log.debug("目标目录不存在，准备创建!");
            if (!descDir.mkdirs()) {
                log.debug("创建目标目录失败!");
                return false;
            }

        }

        boolean flag = true;
        // 列出源目录下的所有文件名和子目录名
        File[] files = srcDir.listFiles();
        if (files != null) {
            for (File file : files) {
                // 如果是一个单个文件，则直接复制
                if (file.isFile()) {
                    flag = FileUtils.copyFile(file.getAbsolutePath(), descDirName + file.getName());
                    // 如果拷贝文件失败，则退出循环
                    if (!flag) {
                        break;
                    }
                }
                // 如果是子目录，则继续复制目录
                if (file.isDirectory()) {
                    flag = FileUtils.copyDirectory(file.getAbsolutePath(), descDirName + file.getName());
                    // 如果拷贝目录失败，则退出循环
                    if (!flag) {
                        break;
                    }
                }
            }
        }

        if (!flag) {
            log.debug("复制目录 " + srcDirName + " 到 " + descDirName + " 失败!");
        } else {
            log.debug("复制目录 " + srcDirName + " 到 " + descDirName + " 成功!");
        }
        return flag;

    }

    /**
     * 删除文件，可以删除单个文件或文件夹
     *
     * @param fileName 被删除的文件名
     * @return 如果删除成功，则返回true，否是返回false
     */
    public static boolean delFile(String fileName) {
        File file = new File(fileName);
        if (!file.exists()) {
            log.debug(fileName + " 文件不存在!");
            return true;
        } else {
            if (file.isFile()) {
                return FileUtils.deleteFile(fileName);
            } else {
                return FileUtils.deleteDirectory(fileName);
            }
        }
    }

    /**
     * 删除单个文件
     *
     * @param fileName 被删除的文件名
     * @return 如果删除成功，则返回true，否则返回false
     */
    public static boolean deleteFile(String fileName) {
        File file = new File(fileName);
        if (file.exists() && file.isFile()) {
            if (file.delete()) {
                //log.debug("删除单个文件 " + fileName + " 成功!");
                return true;
            } else {
                log.debug("删除单个文件 " + fileName + " 失败!");
                return false;
            }
        } else {
            log.debug(fileName + " 文件不存在!");
            return true;
        }
    }

    /**
     * 删除目录及目录下的文件
     *
     * @param dirName 被删除的目录所在的文件路径
     * @return 如果目录删除成功，则返回true，否则返回false
     */
    public static boolean deleteDirectory(String dirName) {
        String dirNames = dirName;
        if (!dirNames.endsWith(File.separator)) {
            dirNames = dirNames + File.separator;
        }
        File dirFile = new File(dirNames);
        if (!dirFile.exists() || !dirFile.isDirectory()) {
            log.debug(dirNames + " 目录不存在!");
            return true;
        }
        boolean flag = true;
        // 列出全部文件及子目录
        File[] files = dirFile.listFiles();
        if (files != null) {
            for (File file : files) {
                // 删除子文件
                if (file.isFile()) {
                    flag = FileUtils.deleteFile(file.getAbsolutePath());
                    // 如果删除文件失败，则退出循环
                    if (!flag) {
                        break;
                    }
                }
                // 删除子目录
                else if (file.isDirectory()) {
                    flag = FileUtils.deleteDirectory(file.getAbsolutePath());
                    // 如果删除子目录失败，则退出循环
                    if (!flag) {
                        break;
                    }
                }
            }
        }

        if (!flag) {
            log.debug("删除目录失败!");
            return false;
        }
        // 删除当前目录
        if (dirFile.delete()) {
            log.debug("删除目录 " + dirName + " 成功!");
            return true;
        } else {
            log.debug("删除目录 " + dirName + " 失败!");
            return false;
        }

    }

    /**
     * 创建单个文件
     *
     * @param descFileName 文件名，包含路径
     * @return 如果创建成功，则返回true，否则返回false
     */
    public static boolean createFile(String descFileName) {
        File file = new File(descFileName);
        if (file.exists()) {
            log.debug("文件 " + descFileName + " 已存在!");
            return false;
        }
        if (descFileName.endsWith(File.separator)) {
            log.debug(descFileName + " 为目录，不能创建目录!");
            return false;
        }
        if (!file.getParentFile().exists()) {
            // 如果文件所在的目录不存在，则创建目录
            if (!file.getParentFile().mkdirs()) {
                log.debug("创建文件所在的目录失败!");
                return false;
            }
        }

        // 创建文件
        try {
            if (file.createNewFile()) {
                log.debug(descFileName + " 文件创建成功!");
                return true;
            } else {
                log.debug(descFileName + " 文件创建失败!");
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.debug(descFileName + " 文件创建失败!");
            return false;
        }

    }

    /**
     * 创建目录
     *
     * @param descDirName 目录名,包含路径
     * @return 如果创建成功，则返回true，否则返回false
     */
    public static boolean createDirectory(String descDirName) {
        String descDirNames = descDirName;
        if (!descDirNames.endsWith(File.separator)) {
            descDirNames = descDirNames + File.separator;
        }
        File descDir = new File(descDirNames);
        if (descDir.exists()) {
            log.debug("目录 " + descDirNames + " 已存在!");
            return false;
        }
        // 创建目录
        if (descDir.mkdirs()) {
            log.debug("目录 " + descDirNames + " 创建成功!");
            return true;
        } else {
            log.debug("目录 " + descDirNames + " 创建失败!");
            return false;
        }

    }

    /**
     * 获取待压缩文件在ZIP文件中entry的名字，即相对于跟目录的相对路径名
     *
     * @param dirPath 目录名
     * @param file    entry文件名
     */
    private static String getEntryName(String dirPath, File file) {
        String dirPaths = dirPath;
        if (!dirPaths.endsWith(File.separator)) {
            dirPaths = dirPaths + File.separator;
        }
        String filePath = file.getAbsolutePath();
        // 对于目录，必须在entry名字后面加上"/"，表示它将以目录项存储
        if (file.isDirectory()) {
            filePath += "/";
        }
        int index = filePath.indexOf(dirPaths);

        return filePath.substring(index + dirPaths.length());
    }

    public static String getFileMD5(File file) {
        if (!file.isFile()) {
            return null;
        }
        MessageDigest digest;
        byte[] buffer = new byte[1024];
        int len;
        try (FileInputStream in = new FileInputStream(file)) {
            digest = MessageDigest.getInstance("MD5");
            while ((len = in.read(buffer, 0, 1024)) != -1) {
                digest.update(buffer, 0, len);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        BigInteger bigInt = new BigInteger(1, digest.digest());
        return bigInt.toString(16);
    }

    /**
     * 获取以年月日为文件夹字符串，规则<br>
     *
     * @return 文件夹
     */
    public static String getFolderByYearMonth() {
        int y, m, d;
        Calendar cal = Calendar.getInstance();
        y = cal.get(Calendar.YEAR);
        m = (cal.get(Calendar.MONTH) + 1);
        d = cal.get(Calendar.DAY_OF_MONTH);
        return StringUtils.toAppendStr("/", String.valueOf(y), "/", String.valueOf(m), "/", String.valueOf(d), "/");
    }

    /**
     * 随机获取文件名（uuid）
     *
     * @param fileName 文件名
     * @return 随机文件名
     */
    public static String getRandomFileName(String fileName) {
        String fileNewName = "";
        if (StringUtils.isNotEmpty(fileName)) {
            String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
            fileNewName = StringUtils.toAppendStr(UUID.randomUUID().toString(), ".", extension);
        }
        return fileNewName;
    }

    /**
     * 写入文件
     */
    public static void writeToFile(String fileName, String content, boolean append) {
        try {
            FileUtils.write(new File(fileName), content, "utf-8", append);
            log.debug("文件 " + fileName + " 写入成功!");
        } catch (IOException e) {
            log.debug("文件 " + fileName + " 写入失败! " + e.getMessage());
        }
    }

    /**
     * 写入文件
     */
    public static void writeToFile(String fileName, String content, String encoding, boolean append) {
        try {
            FileUtils.write(new File(fileName), content, encoding, append);
            log.debug("文件 " + fileName + " 写入成功!");
        } catch (IOException e) {
            log.debug("文件 " + fileName + " 写入失败! " + e.getMessage());
        }
    }

    public static long sizeOfDirectory(String path) {
        File folder = new File(path);
        return FileUtils.sizeOfDirectory(folder);
    }


    /**
     * 前端上传文件, 默认保存相同文件名
     *
     * @param file       前端文件
     * @param targetPath 保存路径
     */
    public static void upload(MultipartFile file, String targetPath) throws Exception {
        upload(file, targetPath, file.getOriginalFilename());
    }

    /**
     * 前端上传文件
     *
     * @param file           前端文件
     * @param targetPath     目标路径, 以分隔符结尾
     * @param targetFileName 目标文件名
     */
    public static void upload(MultipartFile file, String targetPath, String targetFileName) throws Exception {
        File targetFile = new File(targetPath + targetFileName);
        upload(file, targetFile);
    }

    /**
     * 前端上传文件
     *
     * @param file       前端文件
     * @param targetFile 目标文件
     */
    public static void upload(MultipartFile file, File targetFile) throws Exception {
        file.transferTo(new File(targetFile.getAbsolutePath()));
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param filePath 文件路径
     * @param response HTTP响应对象
     */
    @SneakyThrows
    public static void downloadFile(HttpServletResponse response, String fileName, String filePath) {
        // 文件编码 处理文件名中的 '+'、' ' 特殊字符
        String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");

        String contentType = EXT_MAPS.get(getFilenameSuffix(encoderName));
        contentType = StringUtils.isEmpty(contentType) ? "application/octet-stream" : contentType;
        response.setHeader("Content-Disposition", "attachment;filename=\"" + encoderName + "\"");
        response.setContentType(contentType + ";charset=UTF-8");
        response.setHeader("Accept-Ranges", "bytes");
        writeFileToResponse(response, filePath);
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param response HTTP响应对象
     */
    @SneakyThrows
    public static void downloadFile(HttpServletResponse response, byte[] bytes, String fileName) {
        // 文件编码 处理文件名中的 '+'、' ' 特殊字符
        String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");

        String contentType = EXT_MAPS.get(getFilenameSuffix(encoderName));
        contentType = StringUtils.isEmpty(contentType) ? "application/octet-stream" : contentType;
        response.setHeader("Content-Disposition", "attachment;filename=\"" + encoderName + "\"");
        response.setContentType(contentType + ";charset=UTF-8");
        response.setHeader("Accept-Ranges", "bytes");
        OutputStream os =response.getOutputStream();
        os.write(bytes);
        os.flush();
        os.close();
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param response HTTP响应对象
     */
    @SneakyThrows
    public static void downloadPDF(HttpServletResponse response, byte[] bytes, String fileName) {
        // 文件编码 处理文件名中的 '+'、' ' 特殊字符
        String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");
        response.setContentType("application/pdf");
        response.setHeader("Content-Disposition", "inline;filename=\"" + encoderName + "\"");
        OutputStream os =response.getOutputStream();
        os.write(bytes);
        os.flush();
        os.close();
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param content  文件内容
     * @param response HTTP响应对象
     */
    @SneakyThrows
    public static void downloadStrToFile(HttpServletResponse response, String content, String fileName) {
        // 文件编码 处理文件名中的 '+'、' ' 特殊字符
        String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");

        String contentType = EXT_MAPS.get(getFilenameSuffix(encoderName));
        contentType = StringUtils.isEmpty(contentType) ? "application/octet-stream" : contentType;
        response.setHeader("Content-Disposition", "attachment;filename=\"" + encoderName + "\"");
        response.setContentType(contentType + ";charset=UTF-8");
        response.setHeader("Accept-Ranges", "bytes");
        writeStrToResponse(response, content);
    }

    /**
     * 图片下载
     *
     * @param fileName 文件名称
     * @param filePath 文件路径
     * @param response HTTP响应对象
     */
    @SneakyThrows
    public static void downloadImg(HttpServletResponse response, String fileName, String filePath) {
        String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");

        String contentType = EXT_MAPS.get(getFilenameSuffix(encoderName));
        contentType = StringUtils.isEmpty(contentType) ? "application/octet-stream" : contentType;
        response.setHeader("Content-Disposition", "filename=\"" + encoderName + "\"");
        response.setContentType(contentType);
        response.setHeader("Accept-Ranges", "bytes");
        //缓存过期时间设置
        response.setDateHeader("expires", DateUtils.addWeeks(new Date(), 1).getTime());
        response.setHeader("Pragma", "Pragma");

        writeFileToResponse(response, filePath);
    }

    private static void writeFileToResponse(HttpServletResponse response, String filePath) throws IOException {
        File targetFile = new File(filePath);
        OutputStream out = response.getOutputStream();
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(targetFile));
        try {
            //读取数据文件
            byte[] buf = new byte[1024];
            int readByte;
            while ((readByte = bis.read(buf)) != -1) {
                out.write(buf, 0, readByte);
            }
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    log.debug("bis关闭对象失败: " + e.toString());
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    log.debug("out关闭对象失败: " + e.toString());
                }
            }
        }
    }

    private static void writeStrToResponse(HttpServletResponse response, String content) throws IOException {
        OutputStream out = response.getOutputStream();
        out.write(content.getBytes(StandardCharsets.UTF_8));
        out.flush();
        out.close();
    }

    /**
     * 创建文件夹，支持多级
     *
     * @param file 文件对象
     */
    public static void mkdirs(File file) {
        if (!file.mkdirs()) {
            throw new RuntimeException("mkdir failed");
        }
    }

    /**
     * 创建文件夹，支持多级
     *
     * @param path 路径
     */
    public static void mkdirs(String path) {
        mkdirs(new File(path));
    }

    public static void initLocalDir(String localDir) {
        if (!new File(localDir).exists()) {
            FileUtils.mkdirs(localDir);
        }
    }

    /**
     * 字符串写入文件
     */
    public static void appendToFile(String filePath, String... msg) throws IOException {
        String content = StringUtils.join(msg, "");
        appendToFile(filePath, content.getBytes());
    }

    public static void appendToFile(String filePath, byte[] bytes) throws IOException {
        FileChannel fileChannel = null;
        FileLock fileLock = null;
        try {
            fileChannel = FileChannel.open(Paths.get(filePath), StandardOpenOption.APPEND);
            fileLock = fileChannel.lock();
            fileChannel.write(ByteBuffer.wrap(bytes));
        } finally {
            if (fileChannel != null) {
                try {
                    fileChannel.force(true);
                } catch (IOException e) {
                    log.warn("fileChannel.force释放时抛异常:" + e.toString());
                }
                if (fileLock != null) {
                    try {
                        fileLock.release();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                try {
                    fileChannel.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 获取文件及文件下的所有文件，不包含文件夹
     *
     * @param files 文件集合
     * @param file  文件/文件夹
     */
    public static void innerListFiles(Collection<File> files, File file) {
        if (!file.exists()) {
            return;
        }
        if (file.isDirectory()) {
            File[] listFiles = file.listFiles();
            if (listFiles != null && listFiles.length > 0) {
                for (File listFile : listFiles) {
                    innerListFiles(files, listFile);
                }
            }
        } else if (file.isFile()) {
            files.add(file);
        }
    }

    /**
     * 写入文件
     *
     * @param fileName 文件名称
     * @param content  文件内容
     */
    public static void writeFile(String fileName, String content) {
        writeFile(fileName, content, "utf-8");
    }

    /**
     * 指定字符集，写入文件。
     *
     * @param fileName
     * @param content
     * @param charset
     */
    public static void writeFile(String fileName, String content, String charset) {
        Writer out = null;
        try {
            createFolder(fileName, true);
            out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fileName), charset));
            out.write(content);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 创建文件夹
     *
     * @param path
     * @param isFile
     */
    public static void createFolder(String path, boolean isFile) {
        if (isFile) {
            path = new File(path).getParentFile().getPath();
        }
        File file = new File(path);
        if (!file.exists())
            file.mkdirs();
    }

    /**
     * 下载文件。
     *
     * @param response
     * @param fullPath 文件的全路径
     * @param fileName 文件名称。
     * @throws IOException
     */
    public static void downLoadFile(HttpServletRequest request, HttpServletResponse response, String fullPath, String fileName) throws IOException {
        OutputStream outp = response.getOutputStream();
        File file = new File(fullPath.replace("\\", File.separator));
        if (file.exists()) {
            response.setContentType("APPLICATION/OCTET-STREAM");
            String filedisplay = fileName;
            String agent = request.getHeader("USER-AGENT");
            //firefox，谷歌            Trident是标识是ie浏览器 特别处理ie11 的问题
            if (agent != null && !agent.contains("MSIE") && !agent.contains("Trident")) {
                String encoderName = URLEncoder.encode(fileName, "UTF-8").replace("+", "%20").replace("%2B", "+");
                response.setHeader("Content-Disposition", "attachment; filename=" + encoderName);
            } else {
                //ie
                filedisplay = URLEncoder.encode(filedisplay, "utf-8");
                response.addHeader("Content-Disposition", "attachment;filename=" + filedisplay);
            }
            FileInputStream in = null;
            try {
                outp = response.getOutputStream();
                in = new FileInputStream(fullPath);
                byte[] b = new byte[BUFFER_SIZE];
                int i = 0;
                while ((i = in.read(b)) > 0) {
                    outp.write(b, 0, i);
                }
                outp.flush();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (in != null) {
                    try {
                        in.close();
                        in = null;
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                if (outp != null) {
                    try {
                        outp.close();
                        outp = null;
                        response.flushBuffer();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            outp.write("文件不存在!".getBytes(StandardCharsets.UTF_8));
        }
    }

    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        return dir.delete();
    }
}
