package com.c503.ca.cadv.common.util;

import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipFile;

import java.io.*;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class ZipUtils {

    private static final int BUFFER_SIZE = 2 * 1024;

    /**
     * 压缩至文件当前目录
     *
     * @param filePath 文件路径
     * @return 压缩后文件
     */
    public static File compressFile(String filePath) {
        return compressFile(filePath, null);
    }

    public static File compressFile(String filePath, String targetPath) {
        File originFile = new File(filePath);
        if (!originFile.exists()) {
            throw new RuntimeErrorException("目标文件不存在，无法压缩：" + originFile.getAbsolutePath());
        }
        if (StringUtils.isEmpty(targetPath)) {
            targetPath = originFile.getAbsolutePath() + ".zip";
        } else if (!targetPath.endsWith(".zip")) {
            targetPath += File.separator + originFile.getName() + ".zip";
        }

        File archive = new File(targetPath);
        try (ZipArchiveOutputStream outputStream = new ZipArchiveOutputStream(archive);
             InputStream inputStream = new FileInputStream(originFile)) {
            ZipArchiveEntry entry = new ZipArchiveEntry(originFile.getName());
            // 可以设置压缩等级
            outputStream.setLevel(5);
            // 可以设置压缩算法，当前支持ZipEntry.DEFLATED和ZipEntry.STORED两种
            outputStream.setMethod(ZipEntry.DEFLATED);
            // 在zip中创建一个文件
            outputStream.putArchiveEntry(entry);
            byte[] buffer = new byte[1024 * 4];
            int length;
            while ((length = inputStream.read(buffer)) != -1) {
                //把缓冲区的字节写入到 ZipArchiveEntry
                outputStream.write(buffer, 0, length);
            }
            // 完成一个文件的写入
            outputStream.closeArchiveEntry();
            outputStream.finish();
        } catch (Exception e) {
            throw new RuntimeErrorException("压缩失败：" + e.getMessage());
        }
        return archive;
    }

    /**
     * 解压文件至文件当前目录
     *
     * @param filePath 压缩文件路径
     */
    public static void decompressFile(String filePath) throws IOException {
        decompressFile(filePath, null);
    }

    /**
     * 解压文件
     *
     * @param filePath   压缩文件路径
     * @param targetPath 解压目标路径
     */
    public static void decompressFile(String filePath, String targetPath) throws IOException {
        File originFile = new File(filePath);
        if (!originFile.exists()) {
            throw new RuntimeErrorException("目标文件不存在，无法解压：" + originFile.getAbsolutePath());
        }
        if (StringUtils.isEmpty(targetPath)) {
            targetPath = originFile.getParent();
        } else if (!new File(targetPath).isDirectory()) {
            throw new RuntimeErrorException("解压目标路径请填写一个文件夹路径");
        }
        if (!new File(targetPath).exists()) {
            FileUtils.mkdirs(targetPath);
        }
        ZipFile zipFile = new ZipFile(originFile);
        ZipArchiveEntry entry;
        // 获取全部文件的迭代器
        Enumeration<ZipArchiveEntry> entries = zipFile.getEntries();
        while (entries.hasMoreElements()) {
            entry = entries.nextElement();
            if (entry.isDirectory()) {
                continue;
            }
            File outputFile = new File(targetPath + File.separator + entry.getName());

            byte[] buffer = new byte[4096];
            int length;
            try (InputStream inputStream = zipFile.getInputStream(entry); FileOutputStream fos = new FileOutputStream(outputFile)) {
                while ((length = inputStream.read(buffer)) != -1) {
                    fos.write(buffer, 0, length);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 递归压缩文件
     *
     * @param sourceFile       要压缩的文件/文件夹
     * @param zos              输出流
     * @param name             名称
     * @param keepDirStructure keepDirStructure
     * @throws Exception 异常
     */
    private static void compress(File sourceFile, ZipOutputStream zos, String name, boolean keepDirStructure) throws Exception {
        byte[] buf = new byte[BUFFER_SIZE];
        if (sourceFile.isFile()) {
            zos.putNextEntry(new ZipEntry(name));
            int len;
            FileInputStream in = null;
            try {
                in = new FileInputStream(sourceFile);
                while ((len = in.read(buf)) != -1) {
                    zos.write(buf, 0, len);
                }
                zos.closeEntry();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } else {
            File[] listFile = sourceFile.listFiles();
            if (listFile == null || listFile.length == 0) {
                if (keepDirStructure) {
                    zos.putNextEntry(new ZipEntry(name + "/"));
                    zos.closeEntry();
                }
            } else {
                for (File file : listFile) {
                    if (keepDirStructure) {
                        compress(file, zos, name + "/" + file.getName(), keepDirStructure);
                    } else {
                        compress(file, zos, file.getName(), keepDirStructure);
                    }
                }
            }
        }
    }

    /**
     * 压缩当前文件夹
     *
     * @param srcDir           路径
     * @param zipFileName      输出文件
     * @param targetSrc        目标目录
     * @param keepDirStructure keepDirStructure
     */
    public static void toZip(String srcDir, String zipFileName, String targetSrc, boolean keepDirStructure) {
        ZipOutputStream zos = null;
        try {
            zos = new ZipOutputStream(new FileOutputStream(targetSrc + "/" + zipFileName));
            File sourceFile = new File(srcDir);
            compress(sourceFile, zos, sourceFile.getName(), keepDirStructure);
        } catch (Exception e) {
            throw new RuntimeException("zip error from zipUtils", e);
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public  static File compressFolder(String srcDir){
        return  compressFolder(srcDir, true);
    }

    public static File compressFolder(String srcDir, boolean deletedSrcFile) {
        File sourceFile = new File(srcDir);
        String targetSrc = sourceFile.getParentFile().getPath();
        String zipFileName = sourceFile.getName() + ".zip";
        ZipOutputStream zos = null;
        File zipFile = new File(targetSrc + "/" + zipFileName);
        try {
            zos = new ZipOutputStream(new FileOutputStream(zipFile));
            compress(sourceFile, zos, sourceFile.getName(), false);
            zos.flush();
            zos.close();
            return zipFile;
        } catch (Exception e) {
            if (zos != null) {
                try {
                    zos.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            throw new RuntimeException("zip error trow zipUtils", e);
        } finally {
            if (deletedSrcFile) {
                FileUtils.deleteDir(sourceFile);
            }
        }
    }
}
