package com.c503.ca.cadv.common.util.domain;

import com.c503.ca.cadv.common.constant.SystemConstants;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.collect.Lists;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.text.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.convert.converter.Converter;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.util.Assert;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@ApiModel(description = "分页")
@Data
@SuppressWarnings({"unchecked", "rawtypes", "unused"})
public class PageModel<T> implements Pageable, Serializable {

    private static final Logger log = LoggerFactory.getLogger(PageModel.class);

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "Page number of the requested page", example = "1")
    private int page = 1;

    @ApiModelProperty(value = "Size of a page", example = "10")
    private int size = 10;

    @ApiModelProperty(value = "排序方式：如多排序条件,分隔符为英文半角逗号：name asc,date desc")
    @JsonIgnore
    private String sortName;

    @ApiModelProperty(hidden = true, example = "1")
    @JsonProperty("total")
    private long recordsTotal;

    @ApiModelProperty(value = "当前页数据", accessMode = ApiModelProperty.AccessMode.READ_ONLY)
    private List<T> data;

    @ApiModelProperty(accessMode = ApiModelProperty.AccessMode.READ_ONLY)
    @JsonIgnore
    private Sort sort;

    /**
     * 查询条件json
     */
    @JsonIgnore
    @ApiModelProperty(value = "查询条件:" +
        "\n* [{\"fieldName\":\"name\",\"operate\":\"eq\",\"value\":\"g\"},{},...,{}]\r\n" +
        "<br></br>" +
        "\n* 等于:eq，不等于:ne" +
        "\n* 大于:gt，小于:lt，大于等于:ge，小于等于:le" +
        "\n* 类似:like，不类似:notLike" +
        "\n* 区间:between，包含:in，不包含:notIn" +
        "\n* 为Null:isNull，不为Null:isNotNull")
    private String queryConditionJson;

    @JsonIgnore
    private List<QueryCondition> queryConditions = com.google.common.collect.Lists.newArrayList();
    private List<Sort.Order> orders = com.google.common.collect.Lists.newArrayList();

    public PageModel() {

    }

    /**
     * Creates a new {@link PageRequest}. Pages are zero indexed, thus providing 0 for {@code page}
     * will return the first page.
     *
     * @param page zero-based page index.
     * @param size the size of the page to be returned.
     */
    public PageModel(int page, int size) {
        this(page, size, null);
    }

    public PageModel(int page, int size, List<T> dataList, long recordsTotal) {
        this(page, size, null);
        setData(dataList);
        setRecordsTotal(recordsTotal);
    }

    /**
     * Creates a new {@link PageRequest} with sort parameters applied.
     *
     * @param page zero-based page index.
     * @param size the size of the page to be returned.
     * @param sort can be {@literal null}.
     */
    public PageModel(int page, int size, Sort sort) {
        this.page = page;
        this.size = size;
        this.setSort(sort);
    }

    public PageModel(int page, int size, Sort sort, List<QueryCondition> conditions) {
        this(page, size, sort);
        setQueryConditions(conditions);
    }

    /**
     * 拷贝构造器
     * <p>用于转换不同泛型对象，因此不拷贝data属性
     */
    public PageModel(PageModel source) {
        this(source.getPage(), source.getPageSize(), source.getSort(), source.getQueryConditions());
        this.queryConditionJson = source.getQueryConditionJson();
    }

    public void setQueryConditionJson(String queryConditionJson) {
        this.queryConditionJson = StringEscapeUtils.unescapeHtml4(queryConditionJson);
        if (StringUtils.isNotBlank(queryConditionJson)) {
            try {
                this.queryConditions.addAll(JacksonUtils.deserializeList(this.queryConditionJson, QueryCondition.class));
            } catch (Exception e) {
                log.warn("queryCondition[{}] is not json or other error: {}", queryConditionJson, e.getMessage());
            }
        }
    }

    public List<QueryCondition> getQueryConditions() {
        return this.queryConditions;
    }

    public void setSortName(String sort) {
        if (sort == null) {
            return;
        }
        orders = com.google.common.collect.Lists.newArrayList();
        if (sort.contains(SystemConstants.STRING_SPACE)) {
            List<Sort.Order> orders = Lists.newArrayList();
            String[] orderStrList = sort.split(SystemConstants.STRING_SPLIT);
            for (String orderStr : orderStrList) {
                if (sort.contains(SystemConstants.STRING_SPACE)) {
                    String[] sts = orderStr.split(SystemConstants.STRING_SPACE);
                    this.orders.add(new Sort.Order(Direction.fromString(sts[1]), sts[0]));
                    orders.add(new Sort.Order(Direction.fromString(sts[1]), sts[0]));
                } else {
                    throw new IllegalArgumentException("sortName格式有误，请检查");
                }
            }
            this.sort = Sort.by(orders);
        } else {
            this.orders.add(new Sort.Order(Direction.ASC, sort));
            this.sort = Sort.by(this.orders);
        }
    }

    public void setSortDefaultName(Direction direction, String... sorts) {
        if (sort == null) {
            this.sort = new Sort.Order(direction, sorts[0]).withProperties(sorts);
        }
    }

    @Override
    @JsonIgnore
    @ApiModelProperty(hidden = true)
    public int getPageSize() {
        return size;
    }

    @Override
    @JsonIgnore
    @ApiModelProperty(hidden = true)
    public int getPageNumber() {
        return page;
    }

    @Override
    @JsonIgnore
    @ApiModelProperty(hidden = true)
    public long getOffset() {
        return (long) (page - 1) * size;
    }

    @Override
    @JsonIgnore
    public boolean isPaged() {
        return true;
    }

    @Override
    @JsonIgnore
    public boolean isUnpaged() {
        return !isPaged();
    }

    @Override
    public boolean hasPrevious() {
        return page > 0;
    }

    @Override
    public Pageable previousOrFirst() {
        return hasPrevious() ? previous() : first();
    }

    @Override
    public Pageable next() {
        return PageRequest.of(getPageNumber() + 1, getPageSize(), getSort());
    }

    public PageModel previous() {
        return getPageNumber() == 0 ? this
            : new PageModel(getPageNumber() - 1, getPageSize(), getSort());
    }

    @Override
    public Pageable first() {
        return PageRequest.of(0, getPageSize(), getSort());
    }

    public <S> PageModel<S> map(Converter<? super T, ? extends S> converter) {
        return new PageModel(this.getPage(), this.getSize(),
            this.getConvertedContent(converter), this.getRecordsTotal());
    }

    protected <S> List<S> getConvertedContent(Converter<? super T, ? extends S> converter) {
        Assert.notNull(converter, "Converter must not be null!");
        List<S> result = new ArrayList(this.data.size());

        for (T element : this.data) {
            result.add(converter.convert(element));
        }

        return result;
    }

    @ApiModelProperty(hidden = true)
    public int getFirstResult() {
        int firstResult = (getPage() - 1) * getPageSize();
        if (firstResult >= getRecordsTotal()) {
            firstResult = 0;
        }
        return firstResult;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;

        result = prime * result + page;
        result = prime * result + size;

        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }

        PageModel other = (PageModel) obj;
        return page == other.page && this.size == other.size;
    }
}


