package com.c503.ca.cadv.common.util.domain;

import org.apache.commons.lang3.ArrayUtils;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 结果对象构造器
 * <p>统一化接口返回对象格式
 *
 * @author liaow
 */
@SuppressWarnings({"unused"})
public class ResultBuilder {
    public static <T> ResponseEntity<CustomMessage<T>> build(CustomMessage<T> customMessage) {
        return new ResponseEntity<>(customMessage, customMessage.getHttpStatus() == null ? HttpStatus.OK : customMessage.getHttpStatus());
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildOk(String... messages) {
        CustomMessage<T> customMessage = CustomMessage.createSuccess(messages);
        customMessage.setHttpStatus(HttpStatus.OK);
        return new ResponseEntity<>(CustomMessage.createSuccess(messages), HttpStatus.OK);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildOk(T data, String... messages) {
        CustomMessage<T> customMessage = CustomMessage.createSuccessData(data, messages);
        customMessage.setHttpStatus(HttpStatus.OK);
        return new ResponseEntity<>(customMessage, HttpStatus.OK);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildFailed(String... messages) {
        return buildFailed(null, messages);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildFailed(T data, HttpStatus httpStatus, String... messages) {
        if (ArrayUtils.isEmpty(messages)) {
            messages = new String[]{"failed"};
        }
        CustomMessage<T> warn = CustomMessage.createWarn(data, messages);
        warn.setHttpStatus(httpStatus);

        return new ResponseEntity<>(warn, httpStatus != null ? httpStatus : HttpStatus.OK);

    }

    public static <T> ResponseEntity<CustomMessage<T>> buildError(T data, String... messages) {
        return buildError(data, null, messages);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildError(HttpStatus httpStatus, String... messages) {
        return buildError(null, httpStatus, messages);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildError(T data, HttpStatus httpStatus, String... messages) {
        if (ArrayUtils.isEmpty(messages)) {
            messages = new String[]{"error"};
        }
        CustomMessage<T> error = CustomMessage.createError(data, messages);
        error.setHttpStatus(httpStatus);

        return new ResponseEntity<>(error, httpStatus != null ? httpStatus : HttpStatus.INTERNAL_SERVER_ERROR);

    }

    public static <T> ResponseEntity<CustomMessage<T>> buildFailed(HttpStatus httpStatus, String... messages) {

        return buildFailed(null, httpStatus, messages);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildFailed(T data, String... messages) {

        return buildFailed(data, HttpStatus.OK, messages);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildDataOk(T data) {
        String[] msg;
        if (data instanceof BindingResult) {
            BindingResult bindingResult = (BindingResult) data;
            List<String> errorsList = bindingResult.getAllErrors().stream().map(DefaultMessageSourceResolvable::getDefaultMessage).collect(Collectors.toList());
            data = null;
            msg = new String[errorsList.size()];
            msg = errorsList.toArray(msg);
        } else {
            msg = new String[]{"ok"};
        }
        return buildOk(data, msg);
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildObject(T data) {
        return buildOk(data);
    }

    public static <T> ResponseEntity<CustomMessage<T>> wrapOrNotFound(Optional<T> maybeResponse) {
        return wrapOrNotFound(maybeResponse, null);
    }

    public static <T> ResponseEntity<CustomMessage<T>> wrapOrNotFound(Optional<T> maybeResponse, HttpHeaders header) {
        return maybeResponse.map((response) -> ResponseEntity.ok().headers(header).body(CustomMessage.createSuccessData(response))).orElse(new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildMessage(boolean success, String... messages) {
        if (success) {
            return buildOk(messages);
        } else {
            return buildFailed(messages);
        }
    }

    public static <T> ResponseEntity<CustomMessage<T>> buildDataMessage(boolean success, T data, String... messages) {
        if (success) {
            return buildOk(data, messages);
        } else {
            return buildFailed(data, messages);
        }
    }

}
