package com.c503.ca.cadv.common.mvc;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.c503.ca.cadv.common.mvc.util.QueryWrapperUtil;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.QueryCondition;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Sort;

import java.io.Serializable;
import java.util.*;

/**
 * 查询条件构造器
 *
 * @author liaow
 */
@SuppressWarnings({"unused"})
public class QueryDetail<T> implements Serializable {

    private static final long serialVersionUID = 1L;
    private static final String SELF_ENTITY_ALIAS = "a";
    protected static Logger logger = LoggerFactory.getLogger(QueryDetail.class);
    private final String mainTableAlias;
    private final Map<String, Map<String, ColumnCache>> entityColumnMap = Maps.newHashMap();
    public Class<T> entityClass;
    private QueryWrapper<T> queryWrapper = new QueryWrapper<>();

    /**
     * and条件
     */
    private List<QueryCondition> andQueryConditions = Lists.newArrayList();

    /**
     * 排序属性
     */
    private List<Sort.Order> orders = Lists.newArrayList();

    public QueryDetail(Class<T> entityClass) {
        this(entityClass, null, null);
    }

    public QueryDetail(Class<T> entityClass, String mainTableAlias) {
        this(entityClass, mainTableAlias, null);
    }

    public QueryDetail(Class<T> entityClass, String mainTableAlias, Wrapper<T> wrapper) {
        this.entityClass = entityClass;
        this.mainTableAlias = mainTableAlias;
        if (wrapper != null) {
            this.queryWrapper = (QueryWrapper<T>) wrapper;
            this.queryWrapper.setEntityClass(this.entityClass);
        }
        checkEntityClass();
        initColumnMap();
    }

    protected void checkEntityClass() {
        Assert.notNull(this.entityClass,
            "entityClass must not null,please set entity before use this method!");
    }

    private void initColumnMap() {
        addAliasJoinEntity(SELF_ENTITY_ALIAS, this.entityClass);
        if (StringUtils.isNotEmpty(this.mainTableAlias) && !SELF_ENTITY_ALIAS.equals(this.mainTableAlias)) {
            addAliasJoinEntity(this.mainTableAlias, this.entityClass);
        }
    }

    public void addAliasJoinEntity(String alias, Class<?> joinEntity) {
        if (joinEntity != null) {
            Map<String, ColumnCache> columnMap = LambdaUtils.getColumnMap(joinEntity);
            if (columnMap != null && !columnMap.isEmpty()) {
                entityColumnMap.put(alias, columnMap);
            }
        }
    }

    /**
     * 根据属性名称获取对应的sql字段
     */
    public String getSqlColumnByProperty(String propertyName) {
        if (StringUtils.isEmpty(propertyName)) {
            throw new RuntimeErrorException("exception.common.query.paramNameNull");
        }

        String[] a = propertyName.split("\\.");
        if (a.length == 1) {
            return getSqlColumnByProperty(SELF_ENTITY_ALIAS, propertyName);
        } else if (a.length == 2) {
            return getSqlColumnByProperty(a[0], a[1]);
        } else {
            throw new RuntimeErrorException("exception.common.query.paramNameFormatError");
        }
    }

    private String getSqlColumnByProperty(String alias, String propertyName) {
        if (this.entityColumnMap.isEmpty()) {
            return propertyName;
        }

        Map<String, ColumnCache> cacheMap = this.entityColumnMap.get(alias);
        if (cacheMap == null) {
            return alias + "." + propertyName;
        }
        ColumnCache columnCache = cacheMap.get(propertyName.toUpperCase(Locale.ENGLISH));
        if (columnCache != null) {
            if (SELF_ENTITY_ALIAS.equals(alias) && mainTableAlias == null) {
                return columnCache.getColumn();
            } else {
                return alias + "." + columnCache.getColumn();
            }
        }
        return alias + "." + propertyName;
    }

    public List<Sort.Order> getOrders() {
        return orders;
    }

    public QueryDetail<T> setOrders(List<Sort.Order> orders) {
        this.orders = orders;
        return this;
    }

    public List<QueryCondition> getAndQueryConditions() {
        return andQueryConditions;
    }

    public void setAndQueryConditions(List<QueryCondition> andQueryConditions) {
        this.andQueryConditions = andQueryConditions;
    }

    /**
     * 添加一个and条件
     *
     * @param condition 该条件
     * @return 链式调用
     */
    public QueryDetail<T> and(QueryCondition condition) {
        this.andQueryConditions.add(condition);
        return this;
    }

    public QueryDetail<T> andAll(String queryConditionJson) {
        List<QueryCondition> list = null;
        if (StringUtils.isNotEmpty(queryConditionJson)) {
            try {
                list = JacksonUtils.deserializeList(queryConditionJson, QueryCondition.class);
            } catch (Exception e) {
                logger.warn(StringUtils.toAppendStr("queryCondition[", queryConditionJson,
                    "] is not json or other error", e.getMessage()));
            }
        }
        if (list == null) {
            list = Lists.newArrayList();
        }
        List<QueryCondition> rsList = Lists.newArrayList(list);
        this.andQueryConditions.addAll(rsList);
        return this;
    }

    /**
     * 添加多个and条件
     *
     * @param condition 该条件
     * @return 链式调用
     */
    public QueryDetail<T> and(QueryCondition... condition) {
        this.andQueryConditions.addAll(Arrays.asList(condition));
        return this;
    }

    public QueryDetail<T> andAll(Collection<QueryCondition> conditions) {
        this.andQueryConditions.addAll(conditions);
        return this;
    }

    /**
     * 升序字段
     *
     * @param property 该字段对应变量名
     * @return 链式调用
     */
    public QueryDetail<T> orderAsc(String... property) {
        if (ArrayUtils.isNotEmpty(property)) {
            for (String s : property) {
                this.orders.add(Sort.Order.asc(s));
            }
        }
        return this;
    }

    /**
     * 降序字段
     *
     * @param property 该字段对应变量名
     * @return 链式调用
     */
    public QueryDetail<T> orderDest(String... property) {
        if (ArrayUtils.isNotEmpty(property)) {
            for (String s : property) {
                this.orders.add(Sort.Order.desc(s));
            }
        }
        return this;
    }

    public QueryDetail<T> convertPageModel(PageModel<T> pm) {
        this.andAll(pm.getQueryConditions());
        this.setOrders(pm.getOrders());
        pm.setOrders(null);
        return this;
    }

    /**
     * 清除所有条件
     *
     * @return 该实例
     */
    public QueryDetail<T> clearAll() {
        if (!this.andQueryConditions.isEmpty()) {
            this.andQueryConditions.clear();
        }
        if (!this.orders.isEmpty()) {
            this.orders.clear();
        }
        return this;
    }

    /**
     * 清除and条件
     *
     * @return 该实例
     */
    public QueryDetail<T> clearAnd() {
        if (!this.andQueryConditions.isEmpty()) {
            this.andQueryConditions.clear();
        }
        return this;
    }

    /**
     * 清除order条件
     *
     * @return 该实例
     */
    public QueryDetail<T> clearOrder() {
        if (!this.orders.isEmpty()) {
            this.orders.clear();
        }
        return this;
    }

    public LambdaQueryWrapper<T> lambda() {
        //转换已有QueryCondition条件
        this.applyWrapper();
        return this.queryWrapper.lambda();
    }

    public QueryWrapper<T> getRealQueryWrapper() {
        return this.queryWrapper;
    }

    public QueryWrapper<T> getQueryWrapper() {
        if (this.andQueryConditions.size() > 0 || this.getOrders().size() > 0) {
            this.applyWrapper();
        }
        return this.queryWrapper;
    }

    public Wrapper<T> getWrapper() {
        return getQueryWrapper();
    }

    public void applyWrapper() {
        //转换已有QueryCondition条件
        QueryWrapperUtil.convertQueryDetail(this);
    }

    /**
     * 连接条件
     */
    public enum Condition {
        /**
         * 与
         */
        AND,
        /**
         * 或
         */
        OR
    }
}
