package com.c503.ca.cadv.common.mvc.service;

import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.c503.ca.cadv.common.mvc.PageQuery;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.repository.BaseRepository;
import com.c503.ca.cadv.common.mvc.util.Querys;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.QueryCondition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * Service基类
 */
@Transactional(rollbackFor = Exception.class)
@SuppressWarnings({"unused"})
public abstract class BaseService<Repository extends BaseRepository<T>, T> extends ServiceImpl<Repository, T> {

    public final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(getClass());

    @Autowired
    public Repository repository;

    private Class<T> entityClass;

    public BaseService() {
        Class<?> c = getClass();
        Type type = c.getGenericSuperclass();
        if (type instanceof ParameterizedType) {
            Type[] parameterizedType = ((ParameterizedType) type).getActualTypeArguments();
            entityClass = (Class<T>) parameterizedType[1];
        }
    }

    @Override
    public Class<T> getEntityClass() {
        return entityClass;
    }

    /**
     * 调用自定义条件构造器
     *
     * @return 条件对象
     */
    public QueryDetail<T> createQuery() {
        return Querys.createQuery(getEntityClass());
    }

    public QueryDetail<T> createQuery(String mainTableAlias) {
        return Querys.createQuery(getEntityClass(), mainTableAlias);
    }

    public QueryDetail<T> createQuery(List<QueryCondition> queryConditionList) {
        return Querys.createQuery(getEntityClass(), queryConditionList);
    }

    public QueryDetail<T> createQuery(QueryCondition... queryConditions) {
        return Querys.createQuery(getEntityClass(), queryConditions);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public T findOne(Serializable id) {
        return repository.selectById(id);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public Optional<T> findOptionalOne(Serializable id) {
        T result = findOne(id);
        return Optional.ofNullable(result);
    }

    public T findOne(Map<String, Object> paramsMap) {
        List<T> ts = repository.selectByMap(paramsMap);
        return CollectionUtils.isNotEmpty(ts) ? ts.get(0) : null;
    }

    public T findOne(Wrapper<T> wrapper) {
        List<T> ts = repository.selectList(wrapper);
        return CollectionUtils.isNotEmpty(ts) ? ts.get(0) : null;
    }

    public T findOne(QueryDetail<T> queryDetail) {
        if (queryDetail == null) {
            return null;
        }
        return findOne(queryDetail.getWrapper());
    }

    /**
     * 动态集合查询
     *
     * @param queryDetail 动态条件对象
     * @return 数量
     */
    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public long count(QueryDetail<T> queryDetail) {
        try {
            return super.count(queryDetail.getQueryWrapper());
        } catch (Exception e) {
            log.error(e.getMessage());
            throw e;
        }
    }

    public List<T> findAll() {
        return repository.selectList(null);
    }

    /**
     * 动态集合查询
     *
     * @param queryDetail 动态条件构造对象
     * @return 记录集合
     */
    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<T> findAll(QueryDetail<T> queryDetail) {
        return findAll(queryDetail.getWrapper());
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<T> findAll(Wrapper<T> wrapper) {
        return super.list(wrapper);
    }

    public PageModel<T> findPage(PageModel<T> pm) {
        return findPageQuery(pm, null);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public PageModel<T> findPage(PageModel<T> pm, List<QueryCondition> queryConditions) {
        return findPageQuery(pm, queryConditions);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public PageModel<T> findPageQuery(PageModel<T> pm, List<QueryCondition> queryConditions) {
        //构造外来条件
        QueryDetail<T> queryDetail = createQuery();
        queryDetail = queryDetail.convertPageModel(pm);

        if (CollectionUtils.isNotEmpty(queryConditions)) {
            queryDetail.andAll(queryConditions);
        }

        return findPageWrapper(pm, queryDetail.getWrapper());
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public PageModel<T> findPageQuery(PageModel<T> pm, QueryDetail<T> queryDetail, List<QueryCondition> queryConditions) {
        //转换前端条件
        queryDetail.convertPageModel(pm);

        if (CollectionUtils.isNotEmpty(queryConditions)) {
            queryDetail.andAll(queryConditions);
        }
        PageQuery<T> page = new PageQuery<>(getEntityClass(), pm, false);

        IPage<T> pageResult = page(page, queryDetail.getWrapper());
        pm.setData(pageResult.getRecords());
        pm.setRecordsTotal(pageResult.getTotal());
        return pm;
    }

    /**
     * 动态分页查询
     *
     * @param pm      分页对象
     * @param wrapper 动态条件对象
     * @return 分页对象
     */
    private PageModel<T> findPageWrapper(PageModel<T> pm, Wrapper<T> wrapper) {
        try {
            IPage<T> page = page(new PageQuery<>(getEntityClass(), pm), wrapper);
            pm.setData(page.getRecords());
            pm.setRecordsTotal(page.getTotal());
            return pm;
        } catch (Exception e) {
            log.error("error: {}", e.toString());
            throw e;
        }
    }

    /**
     * 获取实体的表名
     *
     * @return 表名
     */
    public String findEntityTableName() {
        TableName table = getEntityClass().getAnnotation(TableName.class);
        return table.value();
    }

    /**
     * 物理删除
     */
    public boolean physicalRemoveByIds(Collection<? extends Serializable> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return false;
        }
        return SqlHelper.retBool(repository.physicalDeleteBatchIds(findEntityTableName(), idList));
    }

    /**
     * 物理删除
     */
    public boolean physicalRemoveAll() {
        return SqlHelper.retBool(repository.physicalDeleteAll(findEntityTableName()));
    }

    /**
     * 物理删除
     */
    public boolean physicalRemoveById(Serializable id) {
        return SqlHelper.retBool(repository.physicalDelete(findEntityTableName(), id));
    }
}
