package com.c503.ca.cadv.common.mvc.service;

import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.TreeEntity;
import com.c503.ca.cadv.common.mvc.domain.query.TreeQuery;
import com.c503.ca.cadv.common.mvc.domain.query.TreeResult;
import com.c503.ca.cadv.common.mvc.repository.TreeRepository;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.base.Assert;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.List;

@Transactional
@Slf4j
public abstract class TreeService<Repository extends TreeRepository<T>, T extends TreeEntity> extends BaseService<Repository, T> {

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public T findTreeOne(Serializable id) {
        List<T> treeList = super.list(createQuery().lambda().eq(T::getId, id));
        return SqlHelper.getObject(null, treeList);
    }

    public Integer countByParentId(String parentId) {
        return super.count(
            createQuery().lambda().eq(T::getParentId, parentId)
        );
    }

    public List<T> findAllByParentIdsLike(String parentIds) {
        return super.list(
            createQuery().lambda().like(T::getParentIds, parentIds)
        );
    }

    public List<T> findAllByParentId(String parentId) {
        QueryDetail<T> queryDetail = createQuery();
        queryDetail.lambda().like(T::getParentId, parentId);
        return super.list(queryDetail.getWrapper());
    }

    public List<T> findByParentIdOrderBySortDesc(String parentId) {
        QueryDetail<T> queryDetail = createQuery();
        queryDetail.lambda().eq(T::getParentId, parentId).orderByDesc(T::getSort);
        return super.list(queryDetail.getWrapper());
    }

    public List<T> findAllOrderBySort() {
        return super.list(
            createQuery().lambda().orderByAsc(TreeEntity::getSort)
        );
    }

    public List<T> findAllByIdOrParentIdsLike(Serializable id, String likeParentIds) {
        QueryDetail<T> queryDetail = createQuery();
        queryDetail.lambda().likeRight(T::getParentIds, likeParentIds).or().eq(T::getId, id);
        return super.list(queryDetail.getWrapper());
    }

    /**
     * 逻辑删除
     */
    public int deleteById(Serializable id, String likeParentIds, String lastModifiedBy) {
        Assert.assertNotNull(id, "ids 信息为空，操作失败");
        Assert.assertNotNull(likeParentIds, "likeParentIds 信息为空，操作失败");
        Assert.assertNotNull(lastModifiedBy, "lastModifiedBy 信息为空，操作失败");

        List<T> entityList = findAllByIdOrParentIdsLike(id, likeParentIds);

        List<Serializable> idList = Lists.newArrayList();
        entityList.forEach(entity -> idList.add(entity.getId()));

        T dbEntity = findOne(id);
        String parentId = dbEntity.getParentId();

       // super.removeByIds(idList);
        super.physicalRemoveByIds(idList);

        updateParentIsLeaf(parentId);

        return entityList.size();
    }

    private boolean updateParentIsLeaf(String parentId) {
        boolean success = false;

        T parentEntity = findOne(parentId);
        if (parentEntity != null) {
            Integer count = countByParentId((String) parentEntity.getId());
            if (count == 0) {
                parentEntity.setLeaf(true);
                success = super.updateById(parentEntity);
            }
        }

        return success;
    }

    @Override
    public boolean saveOrUpdate(T entity) {
        // 获取修改前的parentIds，用于更新子节点的parentIds
        String oldParentIds = "";
        String oldParentId = null;
        T originEntity = null;

        //设置是否叶子节点
        if (entity.getId() != null && StringUtils.isNotEmpty(entity.getId().toString())) {
            originEntity = super.getById(entity.getId());
            if (originEntity != null) {
                Integer count = countByParentId((String) entity.getId());
                entity.setLeaf(count == null || count == 0);
                oldParentIds = originEntity.getParentIds();
                oldParentId = originEntity.getParentId();
            }
        } else {
            entity.setLeaf(true);
        }

        //设置所有父节点id
        if (entity.getParentId() != null) {
            T parent = super.getById(entity.getParentId());
            if (parent != null && parent.getId() != null && StringUtils.isNotEmpty(parent.getId().toString())) {
                if (parent.isLeaf()) {
                    parent.setLeaf(false);
                    super.updateById(parent);
                }
                String newParentIds = StringUtils.toAppendStr(parent.getParentIds(), parent.getId().toString(), ",");
                entity.setParentIds(newParentIds);
            }
        }

//        checkSave(entity);
        boolean success = super.saveOrUpdate(entity);

        // 更新子节点 parentIds
        if (success) {
            if (StringUtils.isNotEmpty(oldParentIds) && !oldParentIds.equals(entity.getParentIds())) {
                List<T> list = findAllByParentIdsLike((String) entity.getId());
                for (T e : list) {
                    if (StringUtils.isNotEmpty(e.getParentIds())) {
                        e.setParentIds(e.getParentIds().replace(oldParentIds, entity.getParentIds()));
                    }
                }
                if (list != null && !list.isEmpty()) {
                    saveOrUpdateBatch(list);
                }
                log.debug("Save Information for T: {}", entity);
            }
            updateParentIsLeaf(oldParentId);
        }

        return success;
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<TreeResult> findTreeData(TreeQuery query) {
        String extId = query != null ? query.getExtId() : null;
        List<TreeResult> mapList = Lists.newArrayList();
        List<T> list = findAll();
        TreeResult treeResult = null;
        for (T e : list) {
            if ((StringUtils.isEmpty(extId)
                || StringUtils.isEmpty(e.getParentIds()) || (StringUtils.isNotEmpty(extId) && !extId
                .equals(e.getId()) && e.getParentIds() != null
                && !e.getParentIds().contains("," + extId + ",")))) {
                treeResult = new TreeResult();
                treeResult.setId((String) e.getId());
                treeResult.setPid(StringUtils.isEmpty(e.getParentId()) ? "0" : e.getParentId());
                treeResult.setLabel(e.getName());
                treeResult.setKey(e.getName());
                treeResult.setValue((String) e.getId());
                mapList.add(treeResult);
            }
        }
        return mapList;
    }

    /**
     * 逻辑删除，更新子节点
     */
    @Transactional
    public void deleteByParentIds(List<? extends Serializable> ids, String lastModifiedBy) {
        Assert.assertNotNull(ids, "ids 信息为空，操作失败");
        Assert.assertNotNull(lastModifiedBy, "lastModifiedBy 信息为空，操作失败");
        ids.forEach(id -> deleteByParentIds(id, lastModifiedBy));
    }

    /**
     * 逻辑删除，更新子节点
     */
    public void deleteByParentIds(Serializable id, String lastModifiedBy) {
        Assert.assertNotNull(id, "id 信息为空，操作失败");
        Assert.assertNotNull(lastModifiedBy, "lastModifiedBy 信息为空，操作失败");
        T entity = super.findOne(id);
        if (entity != null) {
            deleteById(id, StringUtils.toAppendStr(entity.getParentIds(), entity.getId().toString(), ","), lastModifiedBy);
        } else {
            return;
        }
    }

}
