package com.c503.ca.cadv.common.rest;

import com.c503.ca.cadv.common.util.BeanValidators;
import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.c503.ca.cadv.common.util.LocaleMessageUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.common.util.domain.StatusEmun;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.common.util.exception.RuntimeInfoException;
import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import org.mybatis.spring.MyBatisSystemException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.AntPathMatcher;
import org.springframework.validation.BindException;
import org.springframework.web.HttpMediaTypeNotAcceptableException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.support.MissingServletRequestPartException;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.io.IOException;
import java.lang.reflect.UndeclaredThrowableException;
import java.nio.file.AccessDeniedException;
import java.sql.SQLSyntaxErrorException;
import java.util.List;
import java.util.Set;

/**
 * 异常捕获，统一返回
 */
@ControllerAdvice
@SuppressWarnings({"rawtypes", "unchecked"})
public class ExceptionTranslator {

    protected Logger log = LoggerFactory.getLogger(getClass());

    private final AntPathMatcher pathMatcher = new AntPathMatcher();

    /**
     * 参数绑定异常
     */
    @ExceptionHandler
    @ResponseBody
    public ResponseEntity bindException(Exception e, HttpServletRequest request) {
        // 跳过ws请求报错的统一处理
        if(pathMatcher.match("**/ws/**/**/xhr_streaming", request.getRequestURL().toString())) {
            return null;
        }
        log.error("请求链接:{} 操作异常:{}", request.getRequestURI(), e.toString());
        CustomMessage message = new CustomMessage();
        message.setHttpStatus(HttpStatus.INTERNAL_SERVER_ERROR);
        message.setStatusEmun(StatusEmun.MSG_TYPE_WARNING);
        if (e instanceof RuntimeErrorException) {
            RuntimeErrorException msg = (RuntimeErrorException) e;
            message.setStatusEmun(StatusEmun.MSG_TYPE_ERROR);
            if (msg.getCode() != null) {
                message.setHttpStatus(msg.getCode());
            }
            message.setData(msg.getData());
            message.addMessage(msg.getMessage());
        } else if (e instanceof RuntimeInfoException) {
            RuntimeInfoException msg = (RuntimeInfoException) e;
            message.setStatusEmun(StatusEmun.MSG_TYPE_INFO);
            message.setHttpStatus(HttpStatus.OK);
            if (msg.getCode() != null) {
                message.setHttpStatus(msg.getCode());
            }
            message.setData(msg.getData());
            message.addMessage(msg.getMessage());
        } else if (e instanceof RuntimeWarnException) {
            RuntimeWarnException msg = (RuntimeWarnException) e;
            message.setStatusEmun(StatusEmun.MSG_TYPE_WARNING);
            message.setHttpStatus(HttpStatus.OK);
            if (msg.getCode() != null) {
                message.setHttpStatus(msg.getCode());
            }
            message.setData(msg.getData());
            message.addMessage(msg.getMessage());
        } else if (e instanceof IOException) {
            message.setMessage("");
        } else if (e instanceof MethodArgumentNotValidException) {
            message.setHttpStatus(HttpStatus.BAD_REQUEST);
            MethodArgumentNotValidException methodArgumentNotValidException = (MethodArgumentNotValidException) e;
            message.setData(BeanValidators.extractPropertyAndMessage(methodArgumentNotValidException));
            List<String> list = BeanValidators
                    .extractPropertyAndMessageAsList(methodArgumentNotValidException, ": ");
            message.addMessage(LocaleMessageUtils.get("exception.common.argumentNotValid", "参数校验错误："));
            message.addMessage(CollectionsUtils.convertToString(list, ""));
        } else if (e instanceof BindException ||
                e instanceof MissingServletRequestPartException ||
                e instanceof MissingServletRequestParameterException) {
            message.setHttpStatus(HttpStatus.BAD_REQUEST);
            message.addMessage(LocaleMessageUtils.get("exception.common.invalidParameter", "您提交的参数，服务器无法处理"));
        } else if (e instanceof AccessDeniedException) {
            message.setHttpStatus(HttpStatus.FORBIDDEN);
            message.addMessage(LocaleMessageUtils.get("exception.common.forbidden", "权限不足"));
        } else if (e instanceof ConstraintViolationException) {
            ConstraintViolationException constraintViolationException = (ConstraintViolationException) e;
            Set<? extends ConstraintViolation> constraintViolations = constraintViolationException.getConstraintViolations();
            message.setHttpStatus(HttpStatus.BAD_REQUEST);
            message.setData(constraintViolations);
            List<String> list = BeanValidators
                    .extractPropertyAndMessageAsList((ConstraintViolationException) e, ": ");
            message.addMessage(LocaleMessageUtils.get("exception.common.constraintViolation", "数据验证失败："));
            message.addMessage(CollectionsUtils.convertToString(list, ""));
        } else if (e instanceof MyBatisSystemException || e instanceof SQLSyntaxErrorException) {
            message.setStatusEmun(StatusEmun.MSG_TYPE_ERROR);
            message.addMessage(LocaleMessageUtils.get("exception.common.mybatisError", "数据库操作错误"));
        } else if (e instanceof UndeclaredThrowableException) {
            message.setStatusEmun(StatusEmun.MSG_TYPE_ERROR);
            message.addMessage(LocaleMessageUtils.get("exception.common.internalServerError", "操作异常:"));
            message.addMessage(((UndeclaredThrowableException) e).getUndeclaredThrowable().getMessage());
        } else if (e instanceof HttpMediaTypeNotAcceptableException) {
            log.trace("消息无法被接收：{}", e.getMessage());
            return null;
        } else if (e.getMessage().contains("未能够识别出目标")) {
            message.setStatusEmun(StatusEmun.MSG_TYPE_ERROR);
            message.addMessage(LocaleMessageUtils.get("登录失效，3秒后跳转至登录界面"));

        }else {
            log.error("{}", e.toString());
            message.setStatusEmun(StatusEmun.MSG_TYPE_ERROR);
            message.addMessage(LocaleMessageUtils.get("exception.common.internalServerError", "操作异常:"));
            message.addMessage(e.getMessage());
        }
        return ResultBuilder.build(message);
    }

}
