package com.c503.ca.cadv.common.rest.base;

import com.c503.ca.cadv.common.util.*;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import org.apache.commons.text.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.annotation.Resource;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import javax.validation.Validator;
import java.beans.PropertyEditorSupport;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * 基础控制器支持类
 */
@SuppressWarnings({"unused"})
public abstract class BaseResource {

    protected static final String RESPONSE_JSON = "application/json;charset=UTF-8";
    /**
     * ThreadLocal确保高并发下每个请求的request，response都是独立的
     */
    private static final ThreadLocal<ServletRequest> currentRequest = new ThreadLocal<ServletRequest>();
    private static final ThreadLocal<ServletResponse> currentResponse = new ThreadLocal<ServletResponse>();
    /**
     * 日志对象
     */
    protected static Logger log = LoggerFactory.getLogger(BaseResource.class);
    /**
     * 验证Bean实例对象
     */
    @Resource
    protected Validator validator;

    public static void writeStringHttpResponse(String str, HttpServletResponse response) {
        if (str == null) {
            throw new RuntimeErrorException("WRITE_STRING_RESPONSE_NULL");
        }
        response.reset();
        response.setContentType("text/html;charset=UTF-8");
        response.setCharacterEncoding("UTF-8");
        try {
            response.getWriter().write(str);
        } catch (IOException e) {
            log.warn("{}", e.getMessage());
            throw new RuntimeException(e);
        }
    }

    public static void writeJsonHttpResponse(Object object, HttpServletResponse response) {
        if (object == null) {
            throw new RuntimeErrorException("WRITE_JSON_RESPONSE_NULL");
        }
        try {
            response.setContentType(RESPONSE_JSON);
            response.setCharacterEncoding("UTF-8");
            String str = object instanceof String ? (String) object : JacksonUtils.toJsonString(object);
            log.info("write {}", str);
            assert str != null;
            response.getWriter().write(str);
        } catch (IOException e) {
            log.warn("{}", e.getMessage());
            throw new RuntimeException(e);
        }
    }

    /**
     * 服务端参数有效性验证
     *
     * @param object 验证的实体对象
     * @param groups 验证组
     * @return 验证成功：返回true；验证失败：将错误信息添加到 message 中
     */
    protected boolean beanValidator(Object object, Class<?>... groups) {
        try {
            BeanValidators.validateWithException(validator, object, groups);
        } catch (ConstraintViolationException ex) {
            List<String> list = BeanValidators.extractPropertyAndMessageAsList(ex, ": ");
            list.add(0, LocaleMessageUtils.get("exception.common.constraintViolation"));
            String[] msg = list.toArray(new String[]{});
            throw new RuntimeErrorException(StringUtils.toAppendStr(msg));
        }
        return true;
    }

    /**
     * 线程安全初始化reque，respose对象
     */
    @ModelAttribute
    public void initReqAndRep(HttpServletRequest request, HttpServletResponse response) {
        currentRequest.set(request);
        currentResponse.set(response);
    }

    /**
     * 线程安全
     *
     * @return 当前请求request
     */
    public HttpServletRequest request() {
        return (HttpServletRequest) currentRequest.get();
    }

    /**
     * 线程安全
     *
     * @return 当前请求的response
     */
    public HttpServletResponse response() {
        return (HttpServletResponse) currentResponse.get();
    }

    /**
     * 初始化数据绑定 1. 将所有传递进来的String进行HTML编码，防止XSS攻击 2. 将字段中Date类型转换为String类型
     */
    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        // String类型转换，将所有传递进来的String进行HTML编码，防止XSS攻击
        binder.registerCustomEditor(String.class, new PropertyEditorSupport() {

            @Override
            public String getAsText() {
                Object value = getValue();
                return value != null ? value.toString() : "";
            }

            @Override
            public void setAsText(String text) {
                setValue(text == null ? null : StringEscapeUtils.escapeHtml4(text.trim()));
            }
        });
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
            @Override
            public void setAsText(String text) {
                setValue(DateUtils.parseDate(text));
            }
        });
    }


}
