package com.c503.ca.cadv.common.util.xml;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.TimeZoneEnum;
import com.c503.ca.cadv.common.util.xml.converter.CustomDateConverter;
import com.c503.ca.cadv.common.util.xml.converter.CustomFieldConverter;
import com.google.common.collect.Maps;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.mapper.CachingMapper;
import com.thoughtworks.xstream.mapper.MapperWrapper;
import lombok.extern.slf4j.Slf4j;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Map;
import java.util.TimeZone;

/**
 * Xml 转换工具
 *
 * @author liaow
 */
@Slf4j
@SuppressWarnings({"unused"})
public class XmlUtils {

    public static final String INTERFACE_FILE_PATTERN = "yyyy-MM-dd'T'HH:mm:ss";
    public static final String INTERFACE_FILE_PATTERN_MILLI = "yyyy-MM-dd'T'HH:mm:ss.SSS";

    public static Map<String, XStream> xStreamMap = Maps.newConcurrentMap();

    //序列化用
    private volatile static XStream normalToXmlXStream;
    //反序列化用
    private volatile static XStream normalFromXmlXStream;
    //反序列化用
    private volatile static XStream ignoreFieldFromXmlXStream;

    public static String toIgnoreFieldXML(Object object) {
        String defaultXmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        return toIgnoreFieldXML(object, defaultXmlHeader);
    }

    public static String toIgnoreFieldXML(Object object, String xmlHeader) {
        XStream xStream = getIgnoreFieldFromXmlXStream();
        String xmlStr = xStream.toXML(object);
        return xmlHeader + xmlStr;
    }

    public static String toNormalXML(Object object, String dateFormat, TimeZone timeZone) {
        XStream xStream = getNormalToXmlXStream();
        CustomDateConverter dateConverter = new CustomDateConverter(dateFormat, null, timeZone);
        xStream.registerConverter(dateConverter);
        return xStream.toXML(object);
    }

    public static String toNormalXML(Object object) {
        String defaultXmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        return toNormalXML(object, defaultXmlHeader);
    }

    public static String excelToNormalXML(Object object) {
        String xmlHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        XStream xStream = new XStream(new DomDriver("UTF-8", new NoNameCoder()));
        xStream.autodetectAnnotations(true);
        CustomDateConverter dateConverter = new CustomDateConverter(INTERFACE_FILE_PATTERN, new String[]{INTERFACE_FILE_PATTERN_MILLI}, TimeZoneEnum.CN.value());
        xStream.registerConverter(dateConverter);
        xStream.registerConverter(new CustomFieldConverter());
        String xmlStr = xStream.toXML(object);
        return xmlHeader + xmlStr;
    }

    public static String toNormalXML(Object object, String xmlHeader) {
        XStream xStream = getNormalToXmlXStream();
        String xmlStr = xStream.toXML(object);
        return xmlHeader + xmlStr;
    }

    public static String toString(File file) {
        SAXReader reader = new SAXReader();
        try {
            Document document = reader.read(file);
            return document.asXML();
        } catch (DocumentException e) {
            log.warn(e.getMessage());
        }
        return null;
    }

    public static Object fromIgnoreXML(File file, Class targetClass) throws Exception {
        return fromIgnoreXML(new FileInputStream(file), targetClass);
    }

    public static Object fromIgnoreXML(InputStream in, Class targetClass) throws Exception {
        XStream xStream = xStreamMap.get(targetClass.getName());
        if (xStream == null) {
            xStream = new XStream();
            xStreamMap.put(targetClass.getName(), xStream);
        }
//        XStream xStream = getIgnoreFieldFromXmlXStream();
        xStream.ignoreUnknownElements();
        xStream.processAnnotations(targetClass);
        Object obj = xStream.fromXML(in);
        in.close();
        return obj;
    }

    public static Object fromNormalXML(File xmlFile, Class targetClass) {
        return fromNormalXML(xmlFile, targetClass, INTERFACE_FILE_PATTERN, new String[]{"yyyyMMddHHmmss"}, SystemProperties.getInstance().getBusinessTimeZone());
    }

    public static Object fromNormalXML(File xmlFile, Class targetClass, String defaultFormat, String[] acceptableFormats, TimeZone timeZone) {
        XStream xStream = getNormalFromXmlXStream();
        if (timeZone == null) {
            timeZone = SystemProperties.getInstance().getBusinessTimeZone();
        }
        if (StringUtils.isNotEmpty(defaultFormat)) {
            CustomDateConverter dateConverter = new CustomDateConverter(defaultFormat, acceptableFormats, timeZone);
            xStream.registerConverter(dateConverter);
        }
        xStream.processAnnotations(targetClass);
        return xStream.fromXML(xmlFile);
    }

    public static String readFileAsString(File xmlFile) {
        String xmlFileContent = "";
        try {
            Document document = new SAXReader().read(xmlFile);
            xmlFileContent = document.asXML();
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        return xmlFileContent;
    }

    /* *********   */

    private static XStream createNormalXStream() {
        XStream normalXStream = new XStream();
        normalXStream.autodetectAnnotations(true);
        CustomDateConverter dateConverter = new CustomDateConverter(INTERFACE_FILE_PATTERN, new String[]{INTERFACE_FILE_PATTERN_MILLI}, TimeZoneEnum.CN.value());
        normalXStream.registerConverter(dateConverter);

        return normalXStream;
    }

    private static XStream createIgnoreFieldXStream() {
        XStream xstream = new XStream() {
            @Override
            public Object unmarshal(HierarchicalStreamReader reader, Object root) {
                Object obj = super.unmarshal(reader, root, null);
                if (getMapper() != null && getMapper() instanceof CachingMapper) {
                    ((CachingMapper) getMapper()).flushCache();
                }
                return obj;
            }

            @Override
            protected MapperWrapper wrapMapper(MapperWrapper next) {
                return new MapperWrapper(next) {
                    @Override
                    public boolean shouldSerializeMember(Class definedIn, String fieldName) {
                        if (definedIn == Object.class) {
                            return false;
                        }
                        return super.shouldSerializeMember(definedIn, fieldName);
                    }
                };
            }
        };
        xstream.autodetectAnnotations(true);
        CustomDateConverter dateConverter = new CustomDateConverter(INTERFACE_FILE_PATTERN, new String[]{INTERFACE_FILE_PATTERN_MILLI}, TimeZoneEnum.CN.value());
        xstream.registerConverter(dateConverter);

        return xstream;
    }

    private static XStream getNormalFromXmlXStream() {
        if (normalFromXmlXStream == null) {
            synchronized (XmlUtils.class) {
                if (normalFromXmlXStream == null) {
                    normalFromXmlXStream = createNormalXStream();
                }
            }
        }
        return normalFromXmlXStream;
    }

    private static XStream getIgnoreFieldFromXmlXStream() {
        if (ignoreFieldFromXmlXStream == null) {
            synchronized (XmlUtils.class) {
                if (ignoreFieldFromXmlXStream == null) {
                    ignoreFieldFromXmlXStream = createIgnoreFieldXStream();
                }
            }
        }
        return ignoreFieldFromXmlXStream;
    }


    private static XStream getNormalToXmlXStream() {
        if (normalToXmlXStream == null) {
            synchronized (XmlUtils.class) {
                if (normalToXmlXStream == null) {
                    normalToXmlXStream = createNormalXStream();
                    normalToXmlXStream.registerConverter(new CustomFieldConverter());
                }
            }
        }
        return normalToXmlXStream;
    }

    public static Object fromIgnoreXML(String xml, Class targetClass) {
        XStream xStream = xStreamMap.get(targetClass.getName());
        if (xStream == null) {
            xStream = new XStream();
            xStreamMap.put(targetClass.getName(), xStream);
        }
        xStream.autodetectAnnotations(true);
        CustomDateConverter dateConverter = new CustomDateConverter(INTERFACE_FILE_PATTERN, new String[]{INTERFACE_FILE_PATTERN}, TimeZoneEnum.CN.value());
        xStream.registerConverter(dateConverter);
        xStream.ignoreUnknownElements();
        xStream.processAnnotations(targetClass);
        Object obj = xStream.fromXML(xml);
        return obj;
    }
}
