package com.c503.ca.cadv.common.util.xml;

import com.c503.ca.cadv.common.util.DateUtils;
import com.c503.ca.cadv.common.util.TimeZoneEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.springframework.util.ResourceUtils;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import java.io.*;
import java.util.Date;
import java.util.HashMap;

/**
 * XSD校验工具类
 *
 * @author lijie
 */
@Slf4j
public class XsdUtil {

    public static HashMap<Boolean, String> validate(String xsd, File file) {
        HashMap<Boolean, String> resultMap = new HashMap<>(2);
        File xsdFile = null;
        try {
            xsdFile = ResourceUtils.getFile("classpath:xsd/" + xsd + ".xsd");
        } catch (FileNotFoundException e) {
            log.debug("Xsd 文件不存在: " + "xsd/" + xsd + ".xsd");
            resultMap.put(true, null);
            return resultMap;
        }

        FileInputStream inputStream = null;
        //xsd文件的存储位置
        try {
            inputStream = new FileInputStream(file);
            Document document = new SAXReader().read(inputStream);
            SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = factory.newSchema(xsdFile);
            Validator validator = schema.newValidator();

            //进行缓存机制
            InputSource inputSource = new InputSource(new StringReader(document.asXML()));
            Source source = new SAXSource(inputSource);
            validator.validate(source);

            resultMap.put(true, null);
            return resultMap;
        } catch (SAXParseException e) {
            String reason = "【行号:" + e.getLineNumber() + "】【元素:" + getElement(readNumLine(e.getLineNumber(), file)).replaceAll("/", "") + "】【错误:" + messageFormater(e.getMessage()) + "】";
            log.error(reason + ": " + e);
            resultMap.put(false, reason);
            return resultMap;
        } catch (Exception e) {
            String errorMessage = e.getMessage();
            errorMessage = errorMessage.replace("Error on line 1 of document  : 文件提前结束", "文件内容为空");
            log.error("未知错误: " + e);
            resultMap.put(false, "未知错误: " + errorMessage);
            return resultMap;
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                    System.gc();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static Boolean validate(String xsd, String content) {
        //xsd文件的存储位置
        try {
            SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = factory.newSchema(ResourceUtils.getFile("classpath:xsd/" + xsd + ".xsd"));
            Validator validator = schema.newValidator();

            //进行缓存机制
            InputSource inputSource = new InputSource(new StringReader(content));
            Source source = new SAXSource(inputSource);
            validator.validate(source);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static String createTime() {
        return DateUtils.formatDate(new Date(), TimeZoneEnum.CN.value(), XmlUtils.INTERFACE_FILE_PATTERN);
    }

    public static String parseTime(Date date) {
        return DateUtils.formatDate(date, TimeZoneEnum.CN.value(), XmlUtils.INTERFACE_FILE_PATTERN);
    }

    /**
     * isNull
     *
     * @param parentElement
     * @param elementName
     * @return
     * @描述: 验证xml文件元素是否为空
     * @作者: lijie
     * @创建时间: 2016-11-24下午04:09:17
     */
    public static String isNull(Element parentElement, String elementName) {
        String elementText = null;
        if (null != parentElement.element(elementName)) {
            elementText = parentElement.element(elementName).getText();
        }
        return elementText;
    }

    /**
     * 获取文件制定行号的内容
     *
     * @param lineNumber
     * @param f
     * @return
     */
    public static String readNumLine(long lineNumber, File f) {
        //文件读者
        if (f.exists()) {
            FileInputStream fin = null;
            BufferedReader bReader = null;
            try {
                fin = new FileInputStream(f);
                bReader = new BufferedReader(new InputStreamReader(fin));
                LineNumberReader reader = new LineNumberReader(bReader);
                String str;
                try {
                    str = reader.readLine();
                    while (null != str) {
                        //遇到注释，行号加一
                        if (str.startsWith("<!--") && str.endsWith("-->")) {
                            lineNumber++;
                        }
                        if (reader.getLineNumber() == lineNumber) {
                            return str;
                        } else {
                            str = reader.readLine();
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            } finally {
                if (fin != null) {
                    try {
                        fin.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                if (bReader != null) {
                    try {
                        bReader.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        return null;
    }

    /**
     * 提取元素名
     *
     * @param element
     * @return
     */
    public static String getElement(String element) {
        if (StringUtils.isEmpty(element)) {
            return "";
        }
        int start = 0, end = 0;
        for (char c : element.toCharArray()) {
            if (c == '<') {
                start = element.indexOf(c);
            }
            if (c == '>') {
                end = element.indexOf(c);
                break;
            }
        }
        return element.substring(start + 1, end);
    }

    /**
     * 错误消息格式化
     *
     * @param message
     * @return
     */
    public static String messageFormater(String message) {
        message = message.split(":")[1];
        message = message.replaceAll(" ", "");
        message = message.replaceAll("'", "");
        message = message.replaceAll("\\{", "");
        message = message.replaceAll("}", "");
        message = message.replaceAll("不具有面有效性", "无效");
        message = message.replaceAll("#AnonType_", "");
        message = message.replaceAll("之一开头", "开头");
        message = message.replaceAll("。", "");
        message = message.replaceAll("//", "");
        return message;
    }
}
