package com.c503.ca.cadv.common.util.xml.converter;

import ch.qos.logback.classic.Logger;
import com.c503.ca.cadv.common.util.DateUtils;
import com.c503.ca.cadv.common.util.TimeZoneEnum;
import com.c503.ca.cadv.common.util.xml.XmlUtils;
import com.google.common.collect.Lists;
import com.google.common.primitives.Primitives;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.internal.bytebuddy.implementation.bind.annotation.FieldValue;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

/**
 * 自定义字段转换器
 */
@Slf4j
public class CustomFieldConverter implements Converter {

    @Override
    public boolean canConvert(Class type) {
        return true;
    }

    @Override
    public void marshal(Object source, HierarchicalStreamWriter writer, MarshallingContext context) {
        try {
            handleSource(source, writer);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    private void handleSource(Object source, HierarchicalStreamWriter writer) {
        try {
            Class<?> aClass = source.getClass();
            List<Field> allFields = Lists.newArrayList();
            Class currentClass = aClass.getSuperclass();
            while (currentClass != null) {
                allFields.addAll(Arrays.asList(currentClass.getDeclaredFields()));
                currentClass = currentClass.getSuperclass();
            }
            allFields.addAll(Arrays.asList(aClass.getDeclaredFields()));
            for (Field field : allFields) {
                field.setAccessible(true);
                boolean isIgnore = field.isAnnotationPresent(XStreamOmitField.class);
                if (isIgnore) {
                    continue;
                }
                boolean isAttr = field.isAnnotationPresent(XStreamAsAttribute.class);
                boolean isXstreamAlias = field.isAnnotationPresent(XStreamAlias.class);
                boolean isFiledValue = field.isAnnotationPresent(FieldValue.class);
                Object currentFieldValue = field.get(source);
                String xmlAlias;
                if (isXstreamAlias) {
                    xmlAlias = field.getAnnotation(XStreamAlias.class).value();
                } else {
                    xmlAlias = field.getName();
                }
                if (isAttr) {
                    writer.addAttribute(xmlAlias, null == currentFieldValue ? "" : String.valueOf(currentFieldValue));
                } else if (isFiledValue) {
                    writer.setValue(String.valueOf(currentFieldValue));
                } else {
                    if (currentFieldValue instanceof Logger) {
                        continue;
                    }
                    boolean isBasicType = isBasicType(currentFieldValue);
                    if (currentFieldValue instanceof List) {
                        List<Object> list = (List<Object>) currentFieldValue;
                        for (Object curr : list) {
                            writer.startNode(xmlAlias);
                            isBasicType = isBasicType(curr);
                            if (curr == null || isBasicType) {
                                writer.setValue(null == curr ? "" : String.valueOf(curr));
                            } else {
                                handleSource(curr, writer);
                            }
                            writer.endNode();
                        }
                    } else {
                        writer.startNode(xmlAlias);
                        if (currentFieldValue == null || isBasicType) {
                            writer.setValue(null == currentFieldValue ? "" : String.valueOf(currentFieldValue));
                        } else if (field.getType().equals(java.util.Date.class)) {
                            writer.setValue(DateUtils.formatDate((java.util.Date) currentFieldValue, TimeZoneEnum.CN.value(), XmlUtils.INTERFACE_FILE_PATTERN));
                        } else {
                            handleSource(currentFieldValue, writer);
                        }
                        writer.endNode();
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    private boolean isBasicType(Object object) {
        if (object == null) {
            return true;
        } else if (Primitives.allWrapperTypes().contains(object.getClass())) {
            return true;
        } else {
            return object.getClass().equals(String.class) || object.getClass().equals(BigDecimal.class);
        }
    }

    @Override
    public Object unmarshal(HierarchicalStreamReader hierarchicalStreamReader, UnmarshallingContext unmarshallingContext) {
        throw new UnsupportedOperationException("CustomFieldConverter 不支持反序列化");
    }
}
