package com.c503.ca.cadv.enhance.ftp;

import com.c503.ca.cadv.common.transfer.TransferProgress;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.enhance.ftp.config.FtpProperties;
import com.c503.ca.cadv.enhance.ftp.pool.FTPClientPool;
import com.c503.ca.cadv.enhance.ftp.service.FtpConfigService;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPFileFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * ftp 工具
 *
 * @author liaow
 */
@SuppressWarnings({"unused", "UnusedReturnValue"})
public class FtpUtils {

    private static final Logger logger = LoggerFactory.getLogger(FtpUtils.class);

    /**
     * FTP协议里面，规定文件名编码为iso-8859-1
     */
    private static final String FTP_SERVER_CHARSET = "ISO-8859-1";
    private static volatile FtpConfigService ftpConfigService;

    private volatile static FTPClientPool ftpClientPool;
    private volatile static FtpProperties ftpProperties;

    public static FtpConfigService getFtpConfigService() {
        if (ftpConfigService == null) {
            synchronized (FtpUtils.class) {
                if (ftpConfigService == null) {
                    ftpConfigService = SpringContextHolder.getBean(FtpConfigService.class);
                }
            }
        }
        return ftpConfigService;
    }

    /**
     * 发送文件到FTP上，支持失败自动重试
     *
     * @param ftpId      ftp配置的id
     * @param localFile  待发送的本地文件
     * @param retryTimes 失败重试次数
     */
    public static boolean storeFileWithRetry(String ftpId, File localFile, Integer retryTimes) {
        boolean uploadResult = false;
        try {
            uploadResult = FtpUtils.store(ftpId, localFile);
        } catch (Exception e) {
            if (retryTimes < 1) {
                logger.debug("文件发送FTP失败: " + e.getMessage());
                throw new RuntimeWarnException(e.getMessage());
            } else {
                retryTimes -= 1;
                //logger.debug("重试发送, 剩余次数: " + retryTimes);
                try {
                    Thread.sleep(500);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
                storeFileWithRetry(ftpId, localFile, retryTimes);
            }
        }
        return uploadResult;
    }

    /**
     * 在FTP的工作目录下创建多层目录
     *
     * @param path (路径分隔符使用"/"，且以"/"开头，例如"/data/photo/2018")
     * @return (耗时多少毫秒)
     * @throws Exception (异常)
     */
    public static boolean mkdirs(String ftpId, String path) throws Exception {
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            return mkdirs(client, path);
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    public static boolean mkdirs(FTPClient client, String path) throws Exception {
        checkPath(path);
        String workDirectory = client.printWorkingDirectory();
        String targetPath = (workDirectory + convertCharset(path)).replaceAll("//", "/");
        boolean success = true;

        File f = new File(targetPath);
        List<String> names = new LinkedList<>();
        while (f != null && f.toString().length() > 0) {
            names.add(0, f.toString().replaceAll("\\\\", "/"));
            f = f.getParentFile();
        }
        for (int i = 0; i < names.size(); i++) {
            String name = names.get(i);
            if (!client.makeDirectory(convertCharset(name)) && i == names.size() - 1) {
                success = false;
            }
        }
        client.changeWorkingDirectory(workDirectory);
        return success;
    }

    /**
     * 上传文件到FTP工作目录
     *
     * @param localFile (上传的文件)
     * @param path      (在工作目录中的路径，示例"/data/2018")
     * @param filename  (文件名，示例"default.jpg")
     * @return (耗时多少毫秒)
     * @throws Exception (异常)
     */
    public static boolean store(String ftpId, File localFile, String path, String filename) throws Exception {
        InputStream in = new FileInputStream(localFile);
        return store(ftpId, in, path, filename);
    }

    public static boolean store(String ftpId, File file) throws Exception {
        FTPClient client = null;
        boolean success;
        try (InputStream in = new FileInputStream(file)) {
            client = getFtpClientPool().borrowObject(ftpId);
            success = client.storeFile(convertCharset(file.getName()), in);
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
        return success;
    }

    /**
     * 上传文件到FTP工作目录，path示例"/data/2018"，filename示例"default.jpg"
     *
     * @param in       (要上传的输入流)
     * @param path     (在工作目录中的路径，示例"/data/2018")
     * @param filename (文件名，示例"default.jpg")
     * @return (耗时多少毫秒)
     * @throws Exception (异常)
     */
    public static boolean store(String ftpId, InputStream in, String path, String filename) throws Exception {
        FTPClient client = null;
        String workDirectory;
        try {
            if (path.charAt(path.length() - 1) != '/') {
                path += "/";
            }
            client = getFtpClientPool().borrowObject(ftpId);
            workDirectory = client.printWorkingDirectory();
            mkdirs(client, path);
            client.changeWorkingDirectory(path);
            client.setFileType(FTP.BINARY_FILE_TYPE);
            boolean result = client.storeFile(convertCharset(filename), in);
            if (result) {
                logger.debug("文件上传FTP成功：[{}]", filename);
            }
            client.changeWorkingDirectory(workDirectory);
            return result;
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    /**
     * FTP发送，支持断点续传
     *
     * @param ftpId     ftp ID
     * @param ftpPath   目标FTP路径
     * @param localFile 本地待发送文件
     * @return 发送结果
     */
    public static boolean storeWithResume(String ftpId, File localFile, String ftpPath, String filename) throws Exception {
        return storeWithResume(ftpId, localFile, ftpPath, filename, null);
    }

    /**
     * 发送文件到FTP上，支持失败自动重试
     *
     * @param ftpId      ftp配置的id
     * @param ftpPath    目标FTP路径
     * @param localFile  本地待发送文件
     * @param retryTimes 失败重试次数
     */
    public static boolean storeWithResumeAndRetry(String ftpId, File localFile, String ftpPath, String filename, Integer retryTimes) {
        return storeWithResumeAndRetry(ftpId, localFile, ftpPath, filename, retryTimes, null);
    }

    /**
     * 发送文件到FTP上，支持失败自动重试
     *
     * @param ftpId      ftp配置的id
     * @param ftpPath    目标FTP路径
     * @param localFile  本地待发送文件
     * @param retryTimes 失败重试次数
     */
    public static boolean storeWithResumeAndRetry(String ftpId, File localFile, String ftpPath, String filename, Integer retryTimes, TransferProgress transferProgress) {
        boolean uploadResult = false;
        try {
            uploadResult = FtpUtils.storeWithResume(ftpId, localFile, ftpPath, filename, transferProgress);
        } catch (Exception e) {
            if (retryTimes < 1) {
                logger.debug("文件发送FTP失败。" + e.getMessage());
                throw new RuntimeWarnException(e.getMessage());
            } else {
                retryTimes -= 1;
                //logger.debug("重试发送, 剩余次数: " + retryTimes);
                try {
                    Thread.sleep(500);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
                storeWithResumeAndRetry(ftpId, localFile, ftpPath, filename, retryTimes, transferProgress);
            }
        }
        return uploadResult;
    }

    /**
     * FTP发送，支持断点续传
     *
     * @param ftpId     ftp记录 Id
     * @param ftpPath   目标FTP路径
     * @param localFile 本地待发送文件
     * @param progress  发送进度对象
     * @return 发送结果
     */
    public static boolean storeWithResume(String ftpId, File localFile, String ftpPath, String filename, TransferProgress progress) throws Exception {
        FTPClient client = null;
        InputStream source = null;
        OutputStream dest = null;
        long localFileSize = localFile.length();
        long total = 0;
        //初始化进度信息
        if (progress != null) {
            progress.setSize(localFileSize);
            progress.setStatus(TransferProgress.STATUS_ING);
        }
        try {
            source = new FileInputStream(localFile);
            //远程路径准备
            if (ftpPath.charAt(ftpPath.length() - 1) != '/') {
                ftpPath += "/";
            }
            client = getFtpClientPool().borrowObject(ftpId);
            String workDirectory = client.printWorkingDirectory();
            mkdirs(client, ftpPath);
            client.changeWorkingDirectory(convertCharset(ftpPath));
            client.setFileType(FTP.BINARY_FILE_TYPE);
            //检查远程文件是否存在，并获取输出流
            String fileRemote = ftpPath + filename;
            String fileRemoteEncode = convertCharset(fileRemote);
            if (isFileExist(client, fileRemote)) {
                FTPFile[] ftpFiles = client.listFiles(fileRemoteEncode);
                if (ftpFiles.length > 0) {
                    long remoteSize = ftpFiles[0].getSize();
                    if (remoteSize == localFileSize) {
                        Optional.ofNullable(progress).ifPresent(p -> p.setOperatedSize(remoteSize));
                        return true;
                    } else if (remoteSize < localFileSize) {
                        if (source.skip(remoteSize) == remoteSize) {
                            dest = client.appendFileStream(fileRemoteEncode);
                            total += remoteSize;
                            if (progress != null) {
                                progress.setOperatedSize(total);
                            }
                        }
                    }
                }
            }
            if (dest == null) {
                dest = client.storeFileStream(fileRemoteEncode);
            }
            if (dest == null) {
                logger.warn("FTP[@{}]数据传输连接建立失败，请检查连接配置", ftpId);
                return false;
            }
            //发送
            int numBytes;
            byte[] buffer = new byte[2048];
            while ((numBytes = source.read(buffer)) != -1) {
                dest.write(buffer, 0, numBytes);
                if (progress != null) {
                    progress.setOperatedSize(total += numBytes);
                }
            }
            //完成
            source.close();
            dest.close();
            boolean ok = client.completePendingCommand();
            client.changeWorkingDirectory(workDirectory);
            if (progress != null) {
                progress.setStatus(ok ? TransferProgress.STATUS_OK : TransferProgress.STATUS_FAIL);
            }
            if (ok) {
                logger.debug("文件[{}]发送FTP成功", filename);
            }
            return ok;
        } catch (Exception e) {
            Optional.ofNullable(progress).ifPresent(p -> p.setStatus(TransferProgress.STATUS_FAIL));
            throw e;
        } finally {
            if (dest != null) {
                try {
                    dest.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (source != null) {
                try {
                    source.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    /**
     * 删除FTP工作目录中的指定文件
     *
     * @param pathname (文件路径，示例"/data/2018/default.jpg")
     * @return (删除成功返回true ， 删除失败返回false)
     * @throws Exception (IO异常)
     */
    public static boolean delete(String ftpId, String pathname) throws Exception {
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            return client.deleteFile(convertCharset(pathname));
        } catch (Exception e) {
            logger.error("删除文件失败:{}", e.getMessage());
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    public static boolean deleteByFolder(String ftpId, String ftpFolder) throws Exception {
        boolean success;
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            success = deleteByFolder(client, ftpFolder);
        } catch (Exception e) {
            logger.error("删除失败:{}", e.getMessage());
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return success;
    }

    /**
     * 删除FTP上指定文件夹下文件及其子文件方法，添加了对中文目录的支持
     */
    private static boolean deleteByFolder(FTPClient client, String ftpFolder) throws IOException {
        boolean success;
        client.changeWorkingDirectory(convertCharset(ftpFolder));
        client.enterLocalPassiveMode();
        FTPFile[] files = client.listFiles();
        for (FTPFile file : files) {
            //判断为文件则删除
            if (file.isFile()) {
                client.deleteFile(convertCharset(file.getName()));
            }
            //判断是文件夹
            if (file.isDirectory()) {
                String childPath = ftpFolder + File.separator + file.getName();
                //递归删除子文件夹
                deleteByFolder(client, childPath);
            }
        }
        //循环完成后删除文件夹
        success = client.removeDirectory(convertCharset(ftpFolder));
        if (success) {
            logger.info(ftpFolder + "文件夹删除成功");
        } else {
            logger.error(ftpFolder + "文件夹删除失败");
        }
        return success;
    }

    public static String[] listPathFileNames(String ftpId, String scanPath) throws Exception {
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            return client.listNames(convertCharset(scanPath));
        } catch (Exception e) {
            logger.error("获取文件列表失败:{}", e.getMessage());
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    public static FTPFile[] listPathFiles(String ftpId, String scanPath) throws Exception {
        return listPathFiles(ftpId, scanPath, null);
    }

    public static FTPFile[] listPathFiles(String ftpId, String scanPath, FTPFileFilter ftpFileFilter) throws Exception {
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            if (ftpFileFilter == null) {
                return client.listFiles(convertCharset(scanPath));
            }
            return client.listFiles(convertCharset(scanPath), ftpFileFilter);
        } catch (Exception e) {
            logger.error("获取文件列表失败:{}", e.getMessage());
            e.printStackTrace();
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    public static Map<String, Date> listPathFileDates(String ftpId, String scanPath, String[] filenames) {
        FTPClient client = null;
        Map<String, Date> map = new HashMap<>();
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            if (filenames != null && filenames.length > 0) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
                sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
                for (int i = 0 ; i < filenames.length; i++) {
                    String modificationTime = client.getModificationTime(convertCharset(scanPath + filenames[i]));
                    if (StringUtils.isNotEmpty(modificationTime)) {
                        map.put(String.valueOf(i), sdf.parse(modificationTime));
                    }
                }
            }
        } catch (Exception e) {
            logger.error("获取文件列表失败:{}", e.getMessage());
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
        return map;
    }

    /**
     * 从FTP工作目录下载remote文件
     *
     * @param remote (FTP文件路径，示例"/data/2018/default.jpg")
     * @param local  (保存到本地的位置)
     * @return (耗时多少毫秒)
     * @throws Exception (异常)
     */
    public static boolean retrieve(String ftpId, String remote, File local) throws Exception {
        return retrieve(ftpId, remote, new FileOutputStream(local));
    }

    /**
     * 从FTP工作目录下载remote文件
     *
     * @param remote (文件路径，示例"/data/2018/default.jpg")
     * @param out    (输出流)
     * @return (耗时多少毫秒)
     * @throws Exception (异常)
     */
    public static boolean retrieve(String ftpId, String remote, OutputStream out) throws Exception {
        InputStream in = null;
        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            in = client.retrieveFileStream(convertCharset(remote));
            if (in != null) {
                byte[] buffer = new byte[1024];
                for (int len; (len = in.read(buffer)) != -1; ) {
                    out.write(buffer, 0, len);
                }
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            logger.error("获取FTP下载流异常:{}", e.getMessage());
            throw e;
        } finally {
            if (in != null) {
                try {
                    in.close();
                    client.completePendingCommand();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            
            getFtpClientPool().returnObject(ftpId, client);
        }
    }

    public static boolean retrieve(String ftpId, String path, String fileName, File local) throws Exception {
        FTPClient client = null;
        boolean result = false;
        OutputStream out = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            client.changeWorkingDirectory(convertCharset(path));
            if (local.exists()) {
                out = new FileOutputStream(local);
                result = client.retrieveFile(convertCharset(fileName), out);
            }
        } catch (Exception e) {
            logger.error("FTP下载文件异常:{}", e.getMessage());
            throw e;
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            getFtpClientPool().returnObject(ftpId, client);
        }
        return result;
    }

    public static boolean moveFileBatch(String ftpId, String oldPath, String newPath) {
        boolean success = false;

        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            client.changeWorkingDirectory(convertCharset(oldPath));
            //获取文件数组
            FTPFile[] files = client.listFiles();
            //新文件夹不存在则创建
            if (!client.changeWorkingDirectory(convertCharset(newPath))) {
                mkdirs(client, newPath);
            }
            //回到原有工作目录
            client.changeWorkingDirectory(convertCharset(oldPath));
            for (FTPFile file : files) {
                //转存目录
                success = client.rename(convertCharset(file.getName()),
                        newPath + File.separator + convertCharset(file.getName()));
                if (success) {
                    logger.info(file.getName() + "移动成功");
                } else {
                    logger.warn(file.getName() + "移动失败");
                }
            }
        } catch (Exception e) {
            logger.warn("移动文件失败:{}", e.getMessage());
            e.printStackTrace();
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return success;
    }

    public static boolean renameFile(String ftpId, String path, String oldFileName, String newFileName) throws Exception {
        boolean success;

        FTPClient client = null;
        try {
            if (path.charAt(path.length() - 1) != '/') {
                path += "/";
            }
            client = getFtpClientPool().borrowObject(ftpId);

            //回到原有工作目录
            client.changeWorkingDirectory(convertCharset(path));
            //转存目录
            success = client.rename(convertCharset(oldFileName),
                    convertCharset(newFileName));
            if (success) {
                logger.debug("FTP文件[{}]重命名成功,新文件名称：[{}]", oldFileName, newFileName);
            } else {
                logger.warn("FTP文件[{}]重命名失败", oldFileName);
                throw new RuntimeErrorException("FTP文件[" + oldFileName + "]重命名失败");
            }
        } catch (Exception e) {
            logger.warn("移动文件失败:{}", e.getMessage());
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return success;
    }

    public static boolean moveFile(String ftpId, String oldPath, String newPath, String fileName) throws Exception {
        boolean success;

        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);

            if (newPath.charAt(newPath.length() - 1) != '/') {
                newPath += "/";
            }
            //新文件夹不存在则创建
            if (!client.changeWorkingDirectory(convertCharset(newPath))) {
                mkdirs(client, newPath);
            }
            //回到原有工作目录
            client.changeWorkingDirectory(convertCharset(oldPath));
            //转存目录
            success = client.rename(convertCharset(fileName),
                    convertCharset(newPath + fileName));
            if (success) {
                logger.info("FTP文件[{}]移动成功", fileName);
            } else {
                logger.warn("FTP文件[{}]移动失败", fileName);
            }
        } catch (Exception e) {
            logger.warn("FTP文件[{}]移动异常:{}", fileName, e.getMessage());
            throw e;
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return success;
    }

    /**
     * 检查FTP是否可连接
     */
    public static boolean checkFtp(String ftpId) {
        FTPClient client = null;
        boolean result;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            result = client.isConnected();
        } catch (Exception e) {
            throw new RuntimeWarnException(e.getMessage());
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }
        return result;
    }

    public static boolean copyFile(String ftpId, String olePath, String newPath, String fileName) {
        boolean success = false;

        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            // 跳转到文件目录
            client.changeWorkingDirectory(convertCharset(olePath));
            //设置连接模式，不设置会获取为空
            client.enterLocalPassiveMode();
            // 获取目录下文件集合
            FTPFile[] files = client.listFiles();
            ByteArrayInputStream in;
            ByteArrayOutputStream out;
            String fileNameEncode = convertCharset(fileName);
            for (FTPFile file : files) {
                // 取得指定文件并下载
                if (file.getName().equals(fileNameEncode)) {
                    //读取文件，使用下载文件的方法把文件写入内存,绑定到out流上
                    out = new ByteArrayOutputStream();
                    client.retrieveFile(convertCharset(file.getName()), out);
                    in = new ByteArrayInputStream(out.toByteArray());
                    //创建新目录
                    mkdirs(client, newPath);
                    //文件复制，先读，再写
                    //二进制
                    client.setFileType(FTPClient.BINARY_FILE_TYPE);
                    success = client.storeFile(convertCharset(newPath) + File.separator + file.getName(), in);
                    out.flush();
                    out.close();
                    in.close();
                    if (success) {
                        logger.info("转存成功");
                    } else {
                        logger.warn("复制失败");
                    }
                }
            }
        } catch (Exception e) {
            logger.warn("移动文件失败:{}", e.getMessage());
            e.printStackTrace();
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return success;
    }

    private static void checkPath(String path) {
        if (path.contains("\\")) {
            throw new RuntimeException("'\\' is not allowed in the path,please use '/'");
        }
        if (!path.startsWith("/")) {
            throw new RuntimeException("Please start with '/'");
        }
    }

    public static String getModificationTime(String ftpId, String remotePath) throws Exception {
        String result;

        FTPClient client = null;
        try {
            client = getFtpClientPool().borrowObject(ftpId);
            result = client.getModificationTime(convertCharset(remotePath));
        } finally {
            getFtpClientPool().returnObject(ftpId, client);
        }

        return result;
    }

    public static boolean isFileExist(String ftpId, String remotePath) throws Exception {
        String timestamp = FtpUtils.getModificationTime(ftpId, remotePath);
        return timestamp != null;
    }

    public static boolean isFileExist(FTPClient client, String remotePath) throws Exception {
        String timestamp = client.getModificationTime(convertCharset(remotePath));

        return timestamp != null;
    }

    public static FTPClientPool getFtpClientPool() {
        if (ftpClientPool == null) {
            synchronized (FtpUtils.class) {
                if (ftpClientPool == null) {
                    ftpClientPool = SpringContextHolder.getBean(FTPClientPool.class);
                }
            }
        }
        return ftpClientPool;
    }

    public static FtpProperties getFtpProperties() {
        if (ftpProperties == null) {
            synchronized (FtpUtils.class) {
                if (ftpProperties == null) {
                    ftpProperties = SpringContextHolder.getBean(FtpProperties.class);
                }
            }
        }
        return ftpProperties;
    }

    /**
     * 获取FTP控制字符集，默认UTF-8
     */
    private static String getFtpControlCharset() {
        return getFtpProperties().getControlCharset();
    }

    /**
     * 字符集转换，将本地字符集转换为FTP服务器字符集
     */
    private static String convertCharset(String path) throws UnsupportedEncodingException {
        return new String(path.getBytes(getFtpControlCharset()), FTP_SERVER_CHARSET);
    }

}
