package com.c503.ca.cadv.enhance.ftp.pool;

import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import com.c503.ca.cadv.enhance.ftp.config.FtpProperties;
import com.c503.ca.cadv.enhance.ftp.domain.FtpConfig;
import com.c503.ca.cadv.enhance.ftp.service.FtpConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.pool2.impl.AbandonedConfig;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.apache.commons.pool2.impl.GenericKeyedObjectPoolConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.List;

/**
 * Ftp连接池
 *
 * @author liaow
 */
@Component
@Slf4j
public class FTPClientPool {

    /**
     * ftp客户端连接池
     */
    private final GenericKeyedObjectPool<String, FTPClient> pool;

    /**
     * ftp客户端工厂
     */
    private final FTPClientPooledFactory clientFactory;

    @Autowired
    private FtpConfigService ftpConfigService;

    public static String defaultSplit = "@";
    /**
     * 构造函数中 注入bean
     */
    public FTPClientPool(FTPClientPooledFactory clientFactory, FtpProperties ftpProperties) {
        this.clientFactory = clientFactory;
        pool = new GenericKeyedObjectPool<>(clientFactory, ftpProperties);
        GenericKeyedObjectPoolConfig<FTPClient> poolConfig = generatePoolConfig(ftpProperties);
        pool.setConfig(poolConfig);
    }

    private GenericKeyedObjectPoolConfig<FTPClient> generatePoolConfig(FtpProperties ftpProperties) {
        GenericKeyedObjectPoolConfig<FTPClient> poolConfig = new GenericKeyedObjectPoolConfig<>();
        poolConfig.setMaxTotal(ftpProperties.getMaxTotal());
        poolConfig.setMaxIdlePerKey(ftpProperties.getMaxIdlePerKey());
        poolConfig.setMinIdlePerKey(ftpProperties.getMinIdlePerKey());
        poolConfig.setMaxTotalPerKey(ftpProperties.getMaxTotalPerKey());
        poolConfig.setBlockWhenExhausted(true);
        poolConfig.setTestOnBorrow(true);
        poolConfig.setTestOnReturn(true);
        return poolConfig;
    }

    private AbandonedConfig generateAbandonedConfig() {
        AbandonedConfig abandonedConfig = new AbandonedConfig();
        abandonedConfig.setRemoveAbandonedOnBorrow(true);
        abandonedConfig.setRemoveAbandonedOnMaintenance(true);
        return abandonedConfig;
    }

    public FTPClientPooledFactory getClientFactory() {
        return clientFactory;
    }

    public GenericKeyedObjectPool<String, FTPClient> getPool() {
        return pool;
    }

    /**
     * 从池子中借一个连接对象
     *
     * @return (借来的FTPClient对象)
     * @throws Exception (异常)
     */
    public FTPClient borrowObject(String ftpId) throws Exception {
        FtpConfig ftpConfig = ftpConfigService.findOne(ftpId);
        if (ftpConfig == null) {
            throw new RuntimeWarnException("FTP配置查询失败，请检查是否录入此配置。FTP ID: " + ftpId);
        }
        String key = ftpConfig.getId() + defaultSplit + ftpConfig.getUsername() + defaultSplit + ftpConfig.getHost();
        FTPClient client = pool.borrowObject(key, 20 * 1000);
        client.changeWorkingDirectory("/");
        return client;
    }

    /**
     * 从池子中获取一个较不繁忙的连接对象的key
     */
    public String getLessBusyObjectKey(List<String> keys) {
        if (keys == null || keys.size() <= 0) {
            return null;
        }
        String min = keys.get(0);
        for (String key : keys) {
            if (pool.getNumActive(key) < pool.getNumActive(min)) {
                min = key;
            }
        }
        return min;
    }

    /**
     * 归还一个连接对象到池子中
     *
     * @param ftpClient (归还的连接)
     */
    public void returnObject(String ftpId, FTPClient ftpClient) {
        FtpConfig ftpConfig = ftpConfigService.findOne(ftpId);
        if (ftpConfig == null) {
            throw new RuntimeWarnException("FTP配置查询失败，请检查是否录入此配置。FTP ID: " + ftpId);
        }
        String key = ftpConfig.getId() + defaultSplit + ftpConfig.getUsername() + defaultSplit + ftpConfig.getHost();
        if (ftpClient != null) {
            try {
                ftpClient.changeWorkingDirectory("/");
            } catch (IOException e) {
                log.debug("归还对象池时抛异常:" + e.toString());
            }
            pool.returnObject(key, ftpClient);
        }
    }

    /**
     * 主动抛弃一个无效连接对象
     *
     * @param ftpClient
     */
    public void invalidateObject(String key, FTPClient ftpClient) throws Exception {
        if (ftpClient != null) {
            pool.invalidateObject(key, ftpClient);
        }
    }
}
