package com.c503.ca.cadv.enhance.ftp.pool;

import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.exception.RuntimeInfoException;
import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import com.c503.ca.cadv.enhance.ftp.domain.FtpConfig;
import com.c503.ca.cadv.enhance.ftp.service.FtpConfigService;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * Ftp连接创建工厂
 *
 * @author liaow
 */
@Component
public class FTPClientPooledFactory extends BaseKeyedPooledObjectFactory<String, FTPClient> {

    private static final Logger logger = LoggerFactory.getLogger(FTPClientPooledFactory.class);

    @Autowired
    private FtpConfigService ftpConfigService;

    public static FTPClient doConnect(FtpConfig ftpConfig) throws Exception {
        FTPClient ftpClient = new FTPClient();
        ftpClient.setConnectTimeout(2 * 1000);
        try {
            logger.debug("开始连接FTP服务器:" + ftpConfig.getHost() + ":" + ftpConfig.getPort());
            ftpClient.connect(ftpConfig.getHost(), ftpConfig.getPort());

            int reply = ftpClient.getReplyCode();
            if (!FTPReply.isPositiveCompletion(reply)) {
                ftpClient.disconnect();
                logger.error("FTP Server 拒绝连接。 FTP名称: " + ftpConfig.getName() + ", IP:" + ftpConfig.getHost() + ", userName:" + ftpConfig.getUsername());
                return null;
            }
            boolean result = ftpClient.login(ftpConfig.getUsername(), ftpConfig.getPassword());
            if (!result) {
                logger.debug("FTP 登录失败!请检查密码" + ftpConfig.getName() + ", IP:" + ftpConfig.getHost() + " , userName:" + ftpConfig.getUsername() + ", password:"
                    + ftpConfig.getPassword());
                throw new RuntimeWarnException("FTP 登录失败，请检查密码。 FTP名称: " + ftpConfig.getName() + ", IP:" + ftpConfig.getHost() + ", userName:" + ftpConfig.getUsername() + ", password:"
                    + ftpConfig.getPassword());
            }

            ftpClient.setControlEncoding(getServerCharset(ftpClient));
//            ftpClient.setBufferSize(ftpConfig.getBufferSize());
            ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
//            ftpClient.setDataTimeout(ftpConfig.getDataTimeout());
//            ftpClient.setUseEPSVwithIPv4(ftpConfig.isUseEPSVwithIPv4());
            if (ftpConfig.isPassive()) {
                ftpClient.enterLocalPassiveMode();//进入被动模式
                //ftpClient.setRemoteVerificationEnabled(false);
            }
            logger.debug("创建FTP连接成功: FTP名称: " + ftpConfig.getName() + ", IP: " + ftpConfig.getHost() + ", 模式: " + (ftpConfig.isPassive() ? "被动" : "主动"));
        } catch (IOException e) {
            throw new RuntimeInfoException("FTP连接失败。FTP名称: " + ftpConfig.getName() + " , FTP地址 " + ftpConfig.getHost());
        }
        return ftpClient;
    }

    private static String getServerCharset(FTPClient client) throws IOException {
        // 开启服务器对UTF-8的支持，如果服务器支持就用UTF-8编码，否则就使用本地编码（GBK）.
        if (FTPReply.isPositiveCompletion(client.sendCommand(
            "OPTS UTF8", "ON"))) {
            return "UTF-8";
        } else {
            return "GBK";
        }
    }

    @Override
    public FTPClient create(String key) throws Exception {

        if(StringUtils.isEmpty(key)){
            throw new RuntimeWarnException("FTP配置失败，请检查是否录入此配置。FTP ID: NULL" );
        }
        String ftpId = key.split(FTPClientPool.defaultSplit)[0];
        FtpConfig ftpConfig = ftpConfigService.findOne(ftpId);
        if (ftpConfig == null) {
            throw new RuntimeWarnException("FTP配置查询失败，请检查是否录入此配置。FTP ID: " + key);
        }

        return doConnect(ftpConfig);
    }

    @Override
    public PooledObject<FTPClient> wrap(FTPClient ftpClient) {
        return new DefaultPooledObject<FTPClient>(ftpClient);
    }

    @Override
    public void destroyObject(String key, PooledObject<FTPClient> client) throws Exception {
        FTPClient ftpClient = client.getObject();
        ftpClient.disconnect();
        super.destroyObject(key, client);
    }

    /**
     * 验证对象
     */
    @Override
    public boolean validateObject(String key, PooledObject<FTPClient> p) {
        FTPClient ftpClient = p.getObject();
        if (!ftpClient.isConnected()) {
            return false;
        }
        boolean connect = false;
        try {
            connect = ftpClient.sendNoOp();
        } catch (IOException e) {
            logger.debug("FTP连接对象已失效");
        }
        return connect;
    }

}
