/**
 * Copyright &copy; 2018-2019 c503-boss All rights reserved.
 */
package com.c503.ca.cadv.enhance.quartz.service;

import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.base.Reflections;
import com.c503.ca.cadv.common.constant.SystemConstants;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.enhance.quartz.domain.TaskScheduleJob;
import com.c503.ca.cadv.enhance.quartz.domain.vo.TaskScheduleJobVO;
import com.c503.ca.cadv.enhance.quartz.excutor.QuartzJobFactory;
import com.c503.ca.cadv.enhance.quartz.excutor.QuartzJobFactoryDisallowConcurrentExecution;
import com.c503.ca.cadv.enhance.quartz.repository.TaskScheduleJobRepository;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TimeZone;

/**
 * 任务调度管理Service 任务调度
 */
@Service
@Transactional
@SuppressWarnings({"unused"})
public class TaskScheduleJobService extends DataVOService<TaskScheduleJobRepository, TaskScheduleJob, TaskScheduleJobVO> {

    @Autowired
    private Scheduler scheduler;

    /**
     *
     */
    public void afterPropertiesSet() {
        // 这里获取任务信息数据
        List<TaskScheduleJob> jobList = findByJobStatus(SystemConstants.STR_YES);

        for (TaskScheduleJob job : jobList) {
            try {
                addJob(job);
            } catch (Exception e) {
                log.warn(e.getMessage());
            }
        }
        log.info("init database job over...");
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public TaskScheduleJob findOneBySourceId(String sourceId) {
        return getOne(createQuery().lambda()
            .eq(TaskScheduleJob::getSourceId, sourceId));
    }

    public void saveTaskScheduleJob(TaskScheduleJobVO scheduleJobVO) {
        TaskScheduleJob taskScheduleJob = copyVOToBean(scheduleJobVO);
        taskScheduleJob.setTimeZone(scheduleJobVO.getTimeZone());
        if (StringUtils.isNotEmpty(taskScheduleJob.getId())) {
            TaskScheduleJob dbTaskJob = findOne(taskScheduleJob.getId());
            if (dbTaskJob != null) {
                taskScheduleJob.setVersion(dbTaskJob.getVersion());
            }
        }
        save(taskScheduleJob, true);
    }

    public TaskScheduleJob save(TaskScheduleJob scheduleJob, boolean isAddJob) {
        try {
            CronScheduleBuilder.cronSchedule(scheduleJob.getCronExpression());
        } catch (Exception e) {
            log.warn("{}", e.getMessage());
            throw new RuntimeErrorException("cron表达式有误，不能被解析！");
        }
        Object obj = null;
        try {
            if (StringUtils.isNotBlank(scheduleJob.getSpringId())) {
                obj = SpringContextHolder.getBean(scheduleJob.getSpringId());
            } else {
                Class<?> clazz = Class.forName(scheduleJob.getBeanClass());
                obj = clazz.newInstance();
            }
        } catch (Exception ignored) {
            log.error("任务保存时抛异常(ignored):" + ignored.toString());
        }
        if (obj == null) {
            throw new RuntimeErrorException("未找到目标类！");
        }
        Method method = null;
        try {
            method = Reflections.getAccessibleMethodByName(obj, scheduleJob.getMethodName());
        } catch (Exception ignored) {
            log.error("通过映射获取任务名时抛异常(ignored):" + ignored.toString());
        }
        if (method == null) {
            throw new RuntimeErrorException("未找到目标方法！");
        }
        try {
            saveOrUpdate(scheduleJob);
        } catch (Exception e) {
            log.error("msg {}", e.getMessage());
            throw new RuntimeErrorException("保存失败，检查 name group 组合是否有重复！");
        }
        if (isAddJob) {
            if (SystemConstants.STR_NO.equals(scheduleJob.getJobStatus())) {
                deleteJob(scheduleJob);
            } else {
                addJob(scheduleJob);
            }
        }
        return scheduleJob;
    }

    public TaskScheduleJob getTaskById(String jobId) {
        return findOne(jobId);
    }


    public void changeStatus(String jobId, String cmd) {
        TaskScheduleJob job = getTaskById(jobId);
        if (job == null) {
            return;
        }
        if ("stop".equals(cmd)) {
            deleteJob(job);
            job.setJobStatus(SystemConstants.STR_NO);
        } else if ("start".equals(cmd)) {
            job.setJobStatus(SystemConstants.STR_YES);
            addJob(job);
        }
        saveOrUpdate(job);
    }

    public void updateCron(String jobId, String cron) {
        try {
            CronScheduleBuilder.cronSchedule(cron);
        } catch (Exception e) {
            throw new RuntimeErrorException("cron表达式有误，不能被解析！");
        }

        TaskScheduleJob job = getTaskById(jobId);
        if (job == null) {
            return;
        }
        job.setCronExpression(cron);
        if (SystemConstants.STR_YES.equals(job.getJobStatus())) {
            updateJobCron(job);
        }
        saveOrUpdate(job);
    }

    public void addJob(TaskScheduleJob job) {
        if (job == null || !SystemConstants.STR_YES.equals(job.getJobStatus())) {
            return;
        }

        TriggerKey triggerKey = TriggerKey.triggerKey(job.getName(), job.getGroup());

        CronTrigger trigger;
        try {
            trigger = (CronTrigger) scheduler.getTrigger(triggerKey);

            CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(job.getCronExpression());
            if (job.getTimeZone() != null) {
                scheduleBuilder.inTimeZone(job.getTimeZone());
            } else {
                scheduleBuilder.inTimeZone(TimeZone.getDefault());
            }

            // 不存在，创建一个
            if (null == trigger) {
                Class clazz = SystemConstants.YES.equals(job.getIsConcurrent()) ? QuartzJobFactory.class
                    : QuartzJobFactoryDisallowConcurrentExecution.class;

                JobDetail jobDetail = JobBuilder.newJob(clazz).withIdentity(job.getName(), job.getGroup()).build();
                jobDetail.getJobDataMap().put("taskScheduleJob", job);

                trigger = TriggerBuilder.newTrigger().withIdentity(job.getName(), job.getGroup())
                    .withSchedule(scheduleBuilder).build();
                scheduler.scheduleJob(jobDetail, trigger);

            } else {
                // Trigger已存在，那么更新相应的定时设置
                // 按新的cronExpression表达式重新构建trigger
                trigger = trigger.getTriggerBuilder().withIdentity(triggerKey).withSchedule(scheduleBuilder).build();
                // 按新的trigger重新设置job执行
                scheduler.rescheduleJob(triggerKey, trigger);
            }
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
        }
    }

    public List<TaskScheduleJob> getAllJob() {
        GroupMatcher<JobKey> matcher = GroupMatcher.anyJobGroup();
        Set<JobKey> jobKeys;
        try {
            jobKeys = scheduler.getJobKeys(matcher);

            List<TaskScheduleJob> jobList = new ArrayList<>();
            for (JobKey jobKey : jobKeys) {
                List<? extends Trigger> triggers = scheduler.getTriggersOfJob(jobKey);
                for (Trigger trigger : triggers) {
                    TaskScheduleJob job = new TaskScheduleJob();
                    job.setName(jobKey.getName());
                    job.setGroup(jobKey.getGroup());
                    job.setDescription("触发器:" + trigger.getKey());
                    Trigger.TriggerState triggerState = scheduler.getTriggerState(trigger.getKey());
                    job.setJobStatus(triggerState.name());
                    if (trigger instanceof CronTrigger) {
                        CronTrigger cronTrigger = (CronTrigger) trigger;
                        String cronExpression = cronTrigger.getCronExpression();
                        job.setCronExpression(cronExpression);
                    }
                    jobList.add(job);
                }
            }
            return jobList;
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("更新任务失败");
        }
    }

    public List<TaskScheduleJob> getRunningJob() {
        List<JobExecutionContext> executingJobs;
        try {
            executingJobs = scheduler.getCurrentlyExecutingJobs();
            List<TaskScheduleJob> jobList = new ArrayList<>(executingJobs.size());
            for (JobExecutionContext executingJob : executingJobs) {
                TaskScheduleJob job = new TaskScheduleJob();
                JobDetail jobDetail = executingJob.getJobDetail();
                JobKey jobKey = jobDetail.getKey();
                Trigger trigger = executingJob.getTrigger();
                job.setName(jobKey.getName());
                job.setGroup(jobKey.getGroup());
                job.setDescription("触发器:" + trigger.getKey());
                Trigger.TriggerState triggerState = scheduler.getTriggerState(trigger.getKey());
                job.setJobStatus(triggerState.name());
                if (trigger instanceof CronTrigger) {
                    CronTrigger cronTrigger = (CronTrigger) trigger;
                    String cronExpression = cronTrigger.getCronExpression();
                    job.setCronExpression(cronExpression);
                }
                jobList.add(job);
            }
            return jobList;
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("更新任务失败");
        }
    }

    public void pauseJob(TaskScheduleJob scheduleJob) {
        JobKey jobKey = JobKey.jobKey(scheduleJob.getName(), scheduleJob.getGroup());
        log.info("pauseJob {}", jobKey);
        try {
            scheduler.pauseJob(jobKey);
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("更新任务失败");
        }
    }


    public void resumeJob(TaskScheduleJob scheduleJob) {
        JobKey jobKey = JobKey.jobKey(scheduleJob.getName(), scheduleJob.getGroup());
        log.info("resumeJob {}", jobKey);
        try {
            scheduler.resumeJob(jobKey);
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("更新任务失败");
        }
    }

    public void removeBySourceId(String sourceId) {
        List<TaskScheduleJob> itemList = findAllBySourceId(sourceId);
        if (itemList != null) {
            for (TaskScheduleJob taskScheduleJob : itemList) {
                deleteJob(taskScheduleJob);
                removeById(taskScheduleJob.getId());
            }
        }
    }

    public void removeByName(String jobName) {
        List<TaskScheduleJob> itemList = findByName(jobName);
        if (itemList != null) {
            for (TaskScheduleJob taskScheduleJob : itemList) {
                deleteJob(taskScheduleJob);
                removeById(taskScheduleJob.getId());
            }
        }
    }

    public void deleteJob(TaskScheduleJob scheduleJob) {
        deleteJob(scheduleJob.getName(), scheduleJob.getGroup());
    }

    public void deleteJob(String jobName, String group) {
        JobKey jobKey = JobKey.jobKey(jobName, group);
        log.info("deleteJob {}", jobKey);
        try {
            scheduler.deleteJob(jobKey);
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("删除任务失败");
        }
    }

    /**
     * 通过任务id删除定时任务
     *
     * @param id 任务ID
     */
    public void deleteJobById(String id) {
        TaskScheduleJob taskScheduleJob = findOne(id);
        if (taskScheduleJob != null) {
            deleteJob(taskScheduleJob);
        }
    }

    public void runJobNow(TaskScheduleJob scheduleJob) {
        JobKey jobKey = JobKey.jobKey(scheduleJob.getName(), scheduleJob.getGroup());
        log.info("runAJobNow {}", jobKey);
        try {
            scheduler.triggerJob(jobKey);
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
        }
    }

    public void updateJobCron(TaskScheduleJob taskScheduleJob) {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(taskScheduleJob.getName(), taskScheduleJob.getGroup());

            CronTrigger trigger = (CronTrigger) scheduler.getTrigger(triggerKey);

            CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(taskScheduleJob.getCronExpression());

            trigger = trigger.getTriggerBuilder().withIdentity(triggerKey).withSchedule(scheduleBuilder).build();
            log.info("updateJobCron {}", triggerKey);
            scheduler.rescheduleJob(triggerKey, trigger);
        } catch (SchedulerException e) {
            log.warn("msg {}", e.getMessage());
            throw new RuntimeErrorException("更新任务失败");
        }
    }

    public List<TaskScheduleJob> findByJobStatus(String jobStatus) {
        return findAll(createQuery().lambda()
            .eq(TaskScheduleJob::getJobStatus, jobStatus));
    }

    public List<TaskScheduleJob> findAllBySourceId(String sourceId) {
        return findAll(createQuery().lambda()
            .eq(TaskScheduleJob::getSourceId, sourceId));
    }

    public List<TaskScheduleJob> findByName(String jobName) {
        return findAll(createQuery().lambda()
            .eq(TaskScheduleJob::getName, jobName));
    }

}

