package com.c503.ca.cadv.enhance.websocket.config;

import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.simp.config.MessageBrokerRegistry;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketTransportRegistration;

/**
 * websocket 配置
 *
 * @author liaow
 */
@Configuration
@EnableWebSocketMessageBroker
public class WebSocketConfig implements WebSocketMessageBrokerConfigurer {

    public static String MODE_SINGLE = "single";
    public static String MODE_CLUSTER = "cluster";

    @Override
    public void registerStompEndpoints(StompEndpointRegistry registry) {
        registry.addEndpoint("/ws")  //端点名称
            //.setHandshakeHandler() //握手处理
            //.addInterceptors() //拦截处理
            .setAllowedOrigins("*") //跨域
            .withSockJS(); //使用sockJS
    }

    /**
     * 注册相关服务
     */
    @Override
    public void configureMessageBroker(MessageBrokerRegistry registry) {
        //集群模式，采用Rabbit做消息代理
        Environment environment = SpringContextHolder.getBean(Environment.class);
        String mode = environment.getProperty("spring.websocket.mode");
        if (StringUtils.isEmpty(mode)) {
            mode = MODE_SINGLE; //默认单机模式
        }
        if (mode.equalsIgnoreCase(MODE_SINGLE)) {
            //单机模式，采用本地内存方式代理
            registry.enableSimpleBroker("/queue", "/topic", "/user");
        } else if (mode.equalsIgnoreCase(MODE_CLUSTER)) {
            String host = environment.getProperty("spring.rabbitmq.host");
            String username = environment.getProperty("spring.rabbitmq.username");
            String password = environment.getProperty("spring.rabbitmq.password");
            String stompPort = environment.getProperty("spring.rabbitmq.stomp-port");
            if (host == null || username == null || password == null) {
                throw new RuntimeErrorException("rabbit信息配置缺失，请检查相关配置");
            }
            stompPort = stompPort == null ? "61613" : stompPort;
            registry.enableStompBrokerRelay("/topic", "/queue", "/exchange")
                .setRelayHost(host)
                .setRelayPort(Integer.parseInt(stompPort))
                .setClientLogin(username)
                .setClientPasscode(password)
                .setSystemLogin(username)
                .setSystemPasscode(password);
        } else {
            throw new RuntimeErrorException("Websocket模式未知，请检查配置（single/cluster）");
        }

        //客户端发送前缀
        registry.setApplicationDestinationPrefixes("/app");
        //用户名称前缀
        registry.setUserDestinationPrefix("/user");
    }

    @Override
    public void configureWebSocketTransport(WebSocketTransportRegistration registry) {
        //连接性能配置在这里修改
        registry.setSendTimeLimit(10 * 1000)
            .setSendBufferSizeLimit(512 * 1024)
            .setMessageSizeLimit(1024 * 1024);
    }

    /**
     * 输入通道配置
     */
    @Override
    public void configureClientInboundChannel(ChannelRegistration registration) {
        registration.interceptors(new MyChannelInterceptor());
    }
}

