package com.c503.ca.cadv.enhance.websocket.util;

import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.enhance.websocket.config.WebSocketConstants;
import com.c503.ca.cadv.enhance.websocket.domain.CommonMessageType;
import com.c503.ca.cadv.enhance.websocket.domain.WebSocketMessage;
import org.springframework.data.redis.core.RedisTemplate;


/**
 * WebSocket 工具类
 *
 * @author liaow
 */
public class WebSocketUtils {

    private static volatile RedisTemplate<String, String> redisTemplate;

    private static RedisTemplate getRedisTemplate() {
        if (redisTemplate == null) {
            synchronized (WebSocketUtils.class) {
                if (redisTemplate == null) {
                    redisTemplate = SpringContextHolder.getBean(RedisTemplate.class);
                }
            }
        }
        return redisTemplate;
    }

    /**
     * 给指定用户推送消息
     * <p>用户订阅通道：/topic/user/{userId}</p>
     * <p>分布式情况下，convertAndSendToUser存在节点消息无法共享问题，所以依然用topic方式实现
     *
     * @param userId 用户标识
     * @param message  消息内容
     */
    public static void sendToUser(String userId, WebSocketMessage message){
        message.setUserId(userId);
        getRedisTemplate().convertAndSend("/user", JacksonUtils.toJsonString(message));
    }

    /**
     * 给指定主题订阅者推送消息
     * <p>订阅通道：/topic/主题名称 </p>
     *
     * @param topic   主题
     * @param message 消息内容
     */
    public static void sendTopic(String topic, WebSocketMessage message) {
        message.setTopicType(topic);
        getRedisTemplate().convertAndSend("/topic", JacksonUtils.toJsonString(message));
    }

    /**
     * 发送广播消息
     * <p>订阅通道：/topic/broadcast</p>
     *
     * @param message 消息对象
     */
    public static void sendBroadcast(WebSocketMessage message) {
        message.setTopicType(WebSocketConstants.TOPIC_BROADCAST);
        getRedisTemplate().convertAndSend("/topic", JacksonUtils.toJsonString(message));
    }

    /**
     * 发送广播消息
     * <p>订阅通道：</p>
     *
     * @param message 消息内容
     */
    public static void sendBroadcast(String message) {
        WebSocketMessage messageObj = WebSocketMessage.of(CommonMessageType.BROADCAST_MSG.getValue(), message);
        sendBroadcast(messageObj);
    }

}
