package com.c503.ca.cadv.module.cluster.core.monitor;

import com.c503.ca.cadv.common.util.domain.CacheConstants;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.enhance.redis.util.RedisUtils;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.c503.ca.cadv.module.cluster.core.model.Cluster;
import com.c503.ca.cadv.module.cluster.core.model.Constants;
import com.c503.ca.cadv.module.cluster.core.model.Node;



import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * 核心集群监视控制模块
 * 集群健康监控：实现分布式架构中的心跳检测，基于TCP实现长连接，利用Netty高性能网络通信插件实现。
 *
 * 1. 一个心跳进程，包含三个线程，send、recieve、check。
 * 2. 每台机器两个线程，一个是客户端线程负责发心跳消息，一个是服务端线程负责收心跳消息。
 *    N台机器，1号机器的服务端线程，负责接收其余N-1台机器客户端线程的心跳消息。1号机器的客户端线程，负责向其余N-1台机器的服务端线程发送心跳消息。
 * 3. 设定send的发送周期。
 * 4. 在check中检测节点是否故障，检测原理是当若干个发送周期内没有收到心跳消息，则认为发送节点故障。
 *
 * @author Mr.Lee
 * @date 2021/06/30 08:53
 **/
@Component
@EnableScheduling
public class ClusterMonitor {

	private static final Logger logger = LoggerFactory.getLogger(ClusterMonitor.class);

	// 本地缓存，存储当前在线节点,作为维护心跳客户端线程的依据
	// public static LinkedHashSet<Node> localNodeList = new LinkedHashSet<Node>();

	// 本地缓存，存储当前在线节点及对应的心跳客户端，这里需要采用线程安全的ConcurrentHashMap
	public static Map<Node,HeartBeatClient> localNodeList = new ConcurrentHashMap<>();

	// 本地组播（电文）业务负载均衡处理器
	/*private IBusinessProcessor businessProcessor;*/

	// 集群信息
	public static Cluster clusterInfo;

	// 本地节点
	public static Node localNode;

	public static void init() {
		clusterInfo = SpringContextHolder.getBean(Cluster.class);
		localNode = clusterInfo.getLocal();
		startHeartBeatReciever(localNode.getHeartBeatPort());
		register(localNode);
	}

	/**
	 * @Description:  Master节点定期检测在线节点信息，并与其他slave建立连接
	 * @Author: Mr.Lee
	 * @Date: 2021-6-30 13:59
	 * @Param:
	 * @Return:
	 */
	@Scheduled(fixedRate=3000)
	private void checkLiveNodes(){
		// TODO 只需要Master（Leader）节点维护在线节点情况？
		LinkedHashSet<Node> liveNodeList = getAllLiveNodes();
		if(liveNodeList == null) {
			return ;
		}
		for (Node liveNode: liveNodeList) {
			Set<Node> nodeList = localNodeList.keySet();
			// 发现新节点，则为其创建心跳客户端
			if(!ClusterUtils.isContainsNode(nodeList,liveNode) && !liveNode.equals(localNode)){
				startHeartBeatClient(liveNode);
			}else if(!ClusterUtils.isContainsNode(nodeList,liveNode) && liveNode.equals(localNode)){
				localNodeList.put(liveNode,new HeartBeatClient(localNode.getIp(),localNode.getHeartBeatPort()));// 这里不需要为节点自身创建心跳客户端
			}
		}
	}

	/**
	 * @Description: 定期检测Master状态
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 11:29
	 * @Param:
	 * @Return:
	 */
	@Scheduled(fixedRate=3000)
	private void checkMaster(){
		Long incr = RedisUtils.incr(CacheConstants.LOCK_CHECK_MASTER, 1l, TimeUnit.SECONDS);
		try {
			if (incr != null && incr == 1) {
				LinkedHashSet<Node> liveNodeList = getAllLiveNodes();
				if (liveNodeList == null || liveNodeList.isEmpty()) return;
				List<Node> masters = new ArrayList<>();
				for (Node liveNode : liveNodeList) {
					if (liveNode.getRole().equals(Constants.MASTER)) {
						masters.add(liveNode);
					}
				}
				// 如果没有Leader，则选举一个Leader
				if (masters.size() == 0) {
					logger.warn("[集群监控服务] 集群异常，无Leader");
					voteLeader();
				}
				// 如果出现多个Leader，则保留一个Leader
				else if (masters.size() > 1) {
					logger.warn("[集群监控服务] 集群异常，多个Leader");
					for (int i = 1; i < masters.size(); i++) {
						removeNode(masters.get(i));
						logger.info("[集群监控服务] 节点下线" + masters.get(i).toString());
					}
				}
				RedisUtils.del(CacheConstants.LOCK_CHECK_MASTER);
			}
		} catch (Exception e) {
			RedisUtils.del(CacheConstants.LOCK_CHECK_MASTER);
			logger.debug("checkMaster异常:" + e.toString());
		}
	}

	@Scheduled(fixedRate=3000)
	public void reportLocalNodeToCluster(){
		register(localNode);
	}

	/**
	 * @Description: 节点注册到集群中。
	 * @Author: Mr.Lee
	 * @Date: 2021-6-29 16:28
	 * @Param:
	 * @Return:
	 */
	public static void register(Node node){
		if(node == null) {
			return ;
		}
		LinkedHashSet<Node> liveNodeList = getAllLiveNodes();
		if (CollectionUtils.isEmpty(liveNodeList) || !ClusterUtils.isContainsNode(liveNodeList,node)) {
			logger.info("[集群监控服务] 节点上线" + node.toString());
			addNode(node);
		} else {
			LinkedHashSet<Node> nodes = getAllLiveNodes();
			if (CollectionUtils.isNotEmpty(nodes)) {
				RedisUtils.set(clusterInfo.getNodeName(), nodes, RedisUtils.EXRP_MONTH);
			}
		}
	}

	/**
	 * @Description: 启动心跳检测服务的服务端线程,负责recieve心跳信息
	 * @Author: Mr.Lee
	 * @Date: 2021-6-30 13:55
	 * @Param: port 心跳监听端口
	 * @Return:
	 */
	private static void startHeartBeatReciever(int port){
		logger.info("[集群监控服务] 启动心跳监听...(port:"+ port +")");
		new HeartBeatServer(port).start();
	}

	/**
	 * @Description: 为集群中的某个节点创建心跳客户端长链接
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 11:20
	 * @Param: node 集群中的某个节点
	 * @Return:
	 */
	private void startHeartBeatClient(Node node){
		logger.info("[集群监控服务] 正在与节点"+node.toString()+"建立心跳......");
		HeartBeatClient client = new HeartBeatClient(node.getIp() ,node.getHeartBeatPort());
		localNodeList.put(node,client);
		client.start();
	}

	/**
	 * @Description: 从redis上获取所有在线节点
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 11:04
	 * @Param:
	 * @Return:
	 */
	@SuppressWarnings("unchecked")
	public static LinkedHashSet<Node> getAllLiveNodes(){
		if(clusterInfo == null) {
			return null;
		}
		Object object = RedisUtils.get(clusterInfo.getNodeName());
		if( object != null) {
			return (LinkedHashSet<Node>)object;
		}
		return null;
	}


	/**
	 * @Description: 从集群中删除节点
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 8:57
	 * @Param: node 需要删除的掉线节点
	 * @Return:
	 */
	public static void removeNode(Node node){
		LinkedHashSet<Node> nodes = getAllLiveNodes();
		if(nodes != null) {
			nodes.remove(node);
		}
		RedisUtils.set(clusterInfo.getNodeName(), nodes, RedisUtils.EXRP_MONTH);
	}

	/**
	 * @Description: 往集群中增加一个节点
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 11:07
	 * @Param:
	 * @Return:
	 */
	public static void addNode(Node node){
		LinkedHashSet<Node> nodes = getAllLiveNodes();
		if(nodes == null) {
			nodes = new LinkedHashSet<Node> ();
		}
		nodes.add(node);
		//默认不过期
		RedisUtils.set(clusterInfo.getNodeName(), nodes, RedisUtils.EXRP_MONTH);
	}


	/**
	 * @Description: 当集群中没有Leader时，则选举一个新的Leader.
	 * @Author: Mr.Lee
	 * @Date: 2021-7-2 9:17
	 * @Param:
	 * @Return:
	 */
	public static void voteLeader(){
		LinkedHashSet<Node> liveNodeList = getAllLiveNodes();
		// 遍历集群，确定没有Leader后重新选择一个Leader
		boolean hasLeader = false;// 集群是否存在Leader
		for (Node node:liveNodeList) {
			if(node.getRole() .equals(Constants.MASTER)){
				hasLeader = true;
			}
		}
		if(!hasLeader){
			// 默认将集群中第一个节点作为候补Leader
			Node subLeader = liveNodeList.iterator().next();
			removeNode(subLeader);
			subLeader.setRole(Constants.MASTER);
			addNode(subLeader);
			logger.info("[集群监控服务] 选举新的Leader " + subLeader.toString());
		}
	}


	/**
	 * @Description: 从Redis中获取Leader节点信息
	 * @Author: Mr.Lee
	 * @Date: 2021-7-5 11:34
	 * @Param:
	 * @Return:
	 */
	public static Node getLeader(){
		LinkedHashSet<Node> liveNodeList = getAllLiveNodes();
		if(liveNodeList != null) {
			for (Node liveNode:liveNodeList) {
				if(liveNode.getRole().equals(Constants.MASTER) ){
					return liveNode;//直接返回Master节点
				}
			}
		}

		logger.error("[集群监控服务] 错误！！！当前集群中无Leader! ");
		return null;
	}

	/**
	 * @Description: 判断当前节点是否Leader
	 * @Author: Mr.Lee
	 * @Date: 2021-7-5 11:27
	 * @Param:
	 * @Return:
	 */
	public static boolean isLocalLeader(){
		Node leader = getLeader();
		if(leader != null) {
			return leader.getIp().equals(localNode.getIp());
		}
		return false;
	}

	public static void handleLocalDeadNode(Node node){
		logger.info("[集群监控服务] 节点下线" + node.toString());
		removeNode(node);// Redis中维护的集群状态
		localNodeList.remove(node); // 本地维护的集群状态
		if(node.getRole().equals(Constants.MASTER)){
			logger.info("[集群监控服务] 检测到Leader离线：" + node.toString());
			ClusterMonitor.voteLeader();
		}
	}
	/** 
	 * @Description: 处理离线节点 
	 * @Author: Mr.Lee
	 * @Date: 2021-7-8 14:28
	 * @Param: ip 离线节点IP
	 * @Return: 
	 */
	public static void handleDeadNode(Node node){
		logger.info("[集群监控服务] 节点下线" + node.toString());
		removeNode(node);// Redis中维护的集群状态
		localNodeList.remove(node); // 本地维护的集群状态
		if(node.getRole().equals(Constants.MASTER)){
			logger.info("[集群监控服务] 检测到Leader离线：" + node.toString());
			ClusterMonitor.voteLeader();
		}
	}

	/** 
	 * @Description: 处理离线节点，根据IP和心跳端口信息来查找对应的节点并删除
	 * @Author: Mr.Lee
	 * @Date: 2021-7-8 14:34
	 * @Param: 
	 * @Return: 
	 */
	public static void handleDeadNode(String ip,int heartbeatPort){
		for(Node node : ClusterMonitor.localNodeList.keySet()){
			if(node.getIp().equals(ip) && node.getHeartBeatPort() == heartbeatPort){
				handleDeadNode(node);
			}
		}
	}

}
