package com.c503.ca.cadv.module.cluster.core.monitor;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.ReferenceCountUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.c503.ca.cadv.module.cluster.core.model.Constants;
import com.c503.ca.cadv.module.cluster.core.model.NodeStatus;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;


/**
 * 客户端心跳信息处理器
 *
 * @author Mr.Lee
 * @date 2021/06/30 09:04
 **/
public class HeartBeatHandler4Client extends ChannelInboundHandlerAdapter  {

	private static final Logger LOG = LoggerFactory.getLogger(HeartBeatHandler4Client.class);

	private ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);



	private ScheduledFuture<?> heartBeat;

	/*channel激活所执行的方法*/
	@Override
	public void channelActive(ChannelHandlerContext ctx) {
		// 授权认证码
		String authCode = Constants.AUTH_KEY;
		//在实际的项目中可能需要根据具体需求使用特定的认证信息，并加密处理认证信息，保证网络传输认证信息的安全
		ctx.writeAndFlush(authCode);
	}

	@Override
	public void channelRead(ChannelHandlerContext ctx, Object msg) {
		try {
			if(msg instanceof String){
				String ret = (String)msg;
				if(Constants.SUCCESS_KEY.equals(ret)){
					LOG.info("[集群监控服务] 鉴权成功！");
					// 握手成功，主动发送心跳消息。初始化时等待0秒，每隔2秒执行一次发送心跳信息的任务
					this.heartBeat = this.scheduler.scheduleWithFixedDelay(new HeartBeatTask(ctx), 0, 2, TimeUnit.SECONDS);
				}
				else {
					LOG.debug((String) msg);
				}
			}
		} finally {
			ReferenceCountUtil.release(msg);
		}
	}
	//内部类，心跳信息的类
	private class HeartBeatTask implements Runnable {

		private final ChannelHandlerContext ctx;

		public HeartBeatTask(final ChannelHandlerContext ctx) {
			this.ctx = ctx;
		}

		@Override
		public void run() {
			NodeStatus status = new NodeStatus(ClusterMonitor.localNode,Constants.NODE_OK);
			ctx.writeAndFlush(status);
		}

	}

	/**
	 * @Description: 如果远程服务端强制关闭，则会进入到这里
	 * @Author: Mr.Lee
	 * @Date: 2021-7-8 15:06
	 * @Param:
	 * @Return:
	 */
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
		InetSocketAddress address = (InetSocketAddress) ctx.channel().remoteAddress();
		LOG.error("[{}:{}]节点下线",address.getHostString(),address.getPort());
		ClusterMonitor.handleDeadNode(address.getHostString(),address.getPort());
		if (heartBeat != null) {
			heartBeat.cancel(true);
			heartBeat = null;
		}
		ctx.close();
	}
	/**
	 * @Description: 如果远程服务端强制关闭，则会进入到这里
	 * @Author: Mr.Guo
	 * @Date: 2021-10-8 15:06
	 * @Param:
	 * @Return:
	 */
	@Override
	public void channelInactive(ChannelHandlerContext ctx) throws Exception {
		InetSocketAddress address = (InetSocketAddress) ctx.channel().remoteAddress();
		LOG.error("[{}:{}]节点下线",address.getHostString(),address.getPort());

		ClusterMonitor.handleDeadNode(address.getHostString(),address.getPort());

		if (heartBeat != null) {
			heartBeat.cancel(true);
			heartBeat = null;
		}
		ctx.close();
	}
}
