package com.c503.ca.cadv.module.config.sharding.db;

import org.apache.commons.lang3.StringUtils;
import org.apache.shardingsphere.driver.jdbc.core.datasource.ShardingSphereDataSource;
import org.apache.shardingsphere.sharding.api.sharding.standard.PreciseShardingValue;
import org.apache.shardingsphere.sharding.api.sharding.standard.RangeShardingValue;
import org.apache.shardingsphere.sharding.api.sharding.standard.StandardShardingAlgorithm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

/**
 * @author guoxb
 * @desc 根据目标码进行分库分片策略
 * @date 2023-07-26
 */
public class DbObjectCodeShardingAlgorithm implements StandardShardingAlgorithm<String> {

    public static Logger logger = LoggerFactory.getLogger(DbObjectCodeShardingAlgorithm.class);

    @Override
    public Collection<String> doSharding(Collection<String> tableNames, RangeShardingValue<String> rangeShardingValue) {
        return tableNames;
    }

    @Override
    public String doSharding(Collection<String> databases, PreciseShardingValue<String> preciseShardingValue) {
        String targetCode = preciseShardingValue.getValue();

        return databases.stream()
                .filter(db -> db.endsWith(targetCode)).findFirst().orElse(targetCode);
    }

    @Override
    public Properties getProps() {
        return null;
    }

    @Override
    public void init(Properties properties) {

    }

    /**
     * 构建可用表的nodes
     *
     * @param logicTableName 源表名
     * @param targetCodeList 目标码
     * @return 物理表的集合，用，号拼接
     */
    public String buildNodes(String logicTableName, List<String> targetCodeList, String dataSourceName) {
        List<String> tableNameList = new ArrayList<>();
        for (String targetCode : targetCodeList) {
            tableNameList.add(dataSourceName + "_" + targetCode + "." + logicTableName);
        }
        tableNameList.add(dataSourceName+ "." + logicTableName);
        return StringUtils.join(tableNameList, ",");
    }

    /**
     * 新建数据库及表结构
     *
     * @param dataSource     数据源
     * @param logicTableName 源表名
     * @param targetCodeList 目标码
     */
    public void createDataBases(ShardingSphereDataSource dataSource, String databaseName, String logicTableName, List<String> targetCodeList) {
        Connection connection = null;
        Statement statement = null;
        try {
            connection = dataSource.getConnection();
            statement = connection.createStatement();
            for (String targetCode : targetCodeList) {
                String actualDbName = databaseName + "_" + targetCode;
                String dbSql = "CREATE DATABASE IF NOT EXISTS " + actualDbName;
                try {
                    //1、新建数据库
                    statement.execute(dbSql);
                    //2、切换数据库
                    statement.execute("use " + actualDbName);
                    //3、新建表结构
                    statement.execute("create table IF NOT EXISTS " + logicTableName + " like " + databaseName + "." + logicTableName);
                } catch (SQLException e) {
                    logger.error("建库表失败[{}]:{}",  databaseName + "_" + targetCode, e);
                }
            }
            //3、切换至calp数据库
            statement.execute("use " + databaseName);
        } catch (Exception e) {
            logger.error("建库表失败:" + e);
        } finally {
            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
    }


}
