package com.c503.ca.cadv.module.plan.service;

import com.c503.ca.cadv.common.config.ExecutorConfig;
import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.DataEntity;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.service.TargetService;
import com.c503.ca.cadv.module.plan.domain.FileData;
import com.c503.ca.cadv.module.plan.domain.vo.CmdInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.FileDataVO;
import com.c503.ca.cadv.module.plan.repository.FileDataRepository;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.stream.Collectors;

/**
 * @author huyouxiang
 * @date 2023/12/13 14:01
 */
@Service
@Slf4j
public class FileDataService extends DataVOService<FileDataRepository, FileData, FileDataVO> {

    @Resource
    private CmdInstService cmdInstService;

    @Resource
    private TargetService targetService;

    private static final int MAX_CORE_COUNT = 20;

    public List<FileDataVO> read(String relationId) {
        List<FileDataVO> fileDataVOList = findByConfigFileId(relationId);
        if (fileDataVOList != null && fileDataVOList.size() > 0) {
            execRead(fileDataVOList);
        } else {
            fileDataVOList = Lists.newArrayList();
        }
        return fileDataVOList;
    }

    public List<FileDataVO> findByConfigFileId(String configFileId) {
        return findAllVO(createQuery().lambda().eq(FileData::getConfigFileId, configFileId));
    }

    public void execRead(List<FileDataVO> fileDataVOList) {
        ConcurrentLinkedQueue<FileDataVO> queue = new ConcurrentLinkedQueue<>(fileDataVOList);
        ThreadPoolTaskExecutor ex = new ExecutorConfig().nodeAsyncServiceExcutor("-read-filedata", MAX_CORE_COUNT);
        CompletableFuture[] completableFutures = new CompletableFuture[MAX_CORE_COUNT];
        for (int i = 0; i < MAX_CORE_COUNT; i++) {
            completableFutures[i] = CompletableFuture.runAsync(new ReadRunnable(queue), ex);
        }
        CompletableFuture<Void> voidCompletableFuture = CompletableFuture.allOf(completableFutures);
        voidCompletableFuture.join();
        ex.destroy();
    }

    public boolean physicalDel(List<String> idList) {
        boolean success = physicalRemoveByIds(idList);
        if (success) {
            Thread thread = new Thread(() -> cmdInstService.physicalDelCmdInst(idList));
            thread.setName(SystemProperties.getInstance().getSystemName() + "-del-fileData");
            thread.start();
        }
        return success;
    }

    private static final Integer DEL_SUB_SIZE = 1000;

    public void physicalDelFileData(List<String> configFileList) {
        List<String> fileDataIdList = queryIdList(configFileList);
        if (CollectionsUtils.isNotEmpty(fileDataIdList)) {
            int size = fileDataIdList.size();
            List<String> subFileDataIdList;
            for (int i = 0; i < size; i = i + DEL_SUB_SIZE) {
                subFileDataIdList = fileDataIdList.subList(i, Math.min(i + DEL_SUB_SIZE, size));
                cmdInstService.physicalDelCmdInst(subFileDataIdList);
                physicalRemoveByIds(subFileDataIdList);
            }
        }
    }

    private List<String> queryIdList(List<String> configFileList) {
        QueryDetail<FileData> query = createQuery();
        query.lambda().in(FileData::getConfigFileId, configFileList);
        List<FileData> fileDataList = findAll(query);
        if (CollectionsUtils.isEmpty(fileDataList)) {
            return null;
        }
        return fileDataList.stream().map(IdAutoEntity::getId).collect(Collectors.toList());
    }

    public FileDataVO findLastCreatedFileRecord(String filename, String targetId) {
        Target target = targetService.findOne(targetId);
        if (target == null) {
            return null;
        }
        QueryDetail<FileData> query = createQuery();
        query.lambda().eq(FileData::getFile, filename).eq(FileData::getObjectCode, target.getTargetCode()).orderByDesc(DataEntity::getCreatedDate);
        List<FileData> fileDataList = findAll(query);
        if (CollectionsUtils.isNotEmpty(fileDataList)) {
            FileDataVO fileDataVO = copyBeanToVO(fileDataList.get(0));
            fileDataVO.setCmdInstList(cmdInstService.readByFileDataId(fileDataVO.getId()));
            return fileDataVO;
        }
        return null;
    }

    public void setCmdParamFileValue(FileDataVO fileDataVO, String targetId) {
        if (null != fileDataVO) {
            cmdInstService.setCmdParamFileValue(fileDataVO.getCmdInstList(), targetId);
        }
    }


    public class ReadRunnable implements Runnable {

        private ConcurrentLinkedQueue<FileDataVO> queue;

        public ReadRunnable(ConcurrentLinkedQueue<FileDataVO> queue) {
            this.queue = queue;
        }

        @Override
        public void run() {
            while (queue.size() > 0) {
                try {
                    FileDataVO fileDataVO = queue.poll();
                    List<CmdInstVO> cmdInstVOList = cmdInstService.readByFileDataId(fileDataVO.getId());
                    fileDataVO.setCmdInstList(cmdInstVOList);
                } catch (Exception e) {
                    log.error("ReadRunnable error:{}", e.toString());
                }
            }
        }
    }

    public FileDataVO detail(String fileDataId) {
        FileDataVO result = findOneVO(fileDataId);
        result.setCmdInstList(cmdInstService.readByFileDataId(fileDataId));
        return result;
    }


}
