package com.c503.ca.cadv.module.sys.util;

import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.LocaleMessageUtils;
import com.c503.ca.cadv.common.constant.SystemConstants;
import com.c503.ca.cadv.common.util.domain.CacheConstants;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.module.sys.domain.query.DictQuery;
import com.c503.ca.cadv.module.sys.domain.vo.DictVO;
import com.c503.ca.cadv.module.sys.service.DictService;
import com.c503.ca.cadv.enhance.redis.util.RedisUtils;
import com.fasterxml.jackson.databind.JavaType;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Optional;

/**
 * 数据字典工具类
 */
public class DictUtils {

    private static Map<String, List<DictVO>> dictLists = Maps.newHashMap();
    private static Map<String, Map<String, List<DictVO>>> dictMap = Maps.newHashMap();

    private volatile static DictService dictService;

    public static DictService getDictService() {
        if (dictService == null) {
            synchronized (DictUtils.class) {
                if (dictService == null) {
                    dictService = SpringContextHolder.getBean(DictService.class);
                }
            }
        }
        return dictService;
    }

    /**
     * 清空cache中所有字典对象
     */
    public static void clearCache() {
        dictLists = Maps.newHashMap();
        dictMap = Maps.newHashMap();
        RedisUtils.del(CacheConstants.CACHE_DICT_LIST);
        RedisUtils.del(CacheConstants.CACHE_DICT_MAP);
    }

    /**
     * 获取cache中所有字典对象
     */
    public static List<DictVO> getDictList() {
        Locale locale = LocaleMessageUtils.getCurrentLocale();
        String localeName = locale.toString();
        String mapKey = CacheConstants.CACHE_DICT_LIST + ":" + locale.toString();

        String dictListJson;
        List<DictVO> dictList = dictLists.get(mapKey);

        if (dictList == null) {
            //从redis取数据
            dictListJson = (String) RedisUtils.hGet(CacheConstants.CACHE_DICT_LIST, localeName);
            if (StringUtils.isEmpty(dictListJson)) {
                dictList = getDictService().findAllVOIsShowOrderBySortAsc(SystemConstants.YES);
                List<DictVO> localeDictList = Lists.newArrayList();
                //按照语言转换字典内容
                for (DictVO dict : dictList) {
                    String localeMessageKey = String.join(".", "dict", dict.getParentCode(), dict.getCode());
                    String localeMessageName = LocaleMessageUtils.get(localeMessageKey);

                    dict.setName(localeMessageKey.equals(localeMessageName) ? dict.getName() : localeMessageName);
                    localeDictList.add(dict);
                }

                dictListJson = JacksonUtils.toJsonString(localeDictList);

                if (StringUtils.isNotEmpty(dictListJson)) {
                    RedisUtils.hSet(CacheConstants.CACHE_DICT_LIST, localeName, dictListJson);
                    dictLists.put(mapKey, localeDictList);
                }
            } else {
                dictList = JacksonUtils.deserializeList(dictListJson, DictVO.class);
                dictLists.put(mapKey, dictList);
            }
        }

        return dictList;
    }

    /**
     * 获取cache中所有字典对象
     */
    public static Map<String, List<DictVO>> getDictMap() {
        Locale locale = LocaleMessageUtils.getCurrentLocale();
        String localeName = locale.toString();
        String mapKey = CacheConstants.CACHE_DICT_MAP + ":" + localeName;

        Map<String, List<DictVO>> dictMap = DictUtils.dictMap.get(mapKey);
        if (dictMap == null) {
            dictMap = Maps.newHashMap();
            String dictMapJson = (String) RedisUtils.hGet(CacheConstants.CACHE_DICT_MAP, localeName);
            if (StringUtils.isEmpty(dictMapJson) || "{}".equals(dictMapJson)) {
                String parentCode;
                List<DictVO> list = DictUtils.getDictList();
                for (DictVO dict : list) {
                    if (StringUtils.isNotEmpty(dict.getParentCode())) {
                        parentCode = dict.getParentCode();
                        List<DictVO> tempList = dictMap.get(parentCode);
                        if (tempList != null) {
                            tempList.add(dict);
                        } else {
                            dictMap.put(parentCode, Lists.newArrayList(dict));
                        }
                    }
                }
                dictMapJson = JacksonUtils.toJsonString(dictMap);
                RedisUtils.hSet(CacheConstants.CACHE_DICT_MAP, localeName, dictMapJson);
            } else {
                JavaType keyType = JacksonUtils.constructType(String.class);
                JavaType valueType = JacksonUtils.constructCollectionType(List.class, DictVO.class);
                dictMap = JacksonUtils.deserializeMap(dictMapJson, keyType, valueType);
            }
            DictUtils.dictMap.put(mapKey, dictMap);
        }

        return dictMap;
    }

    /**
     * 根据code 和 原始值 获取数据字典name
     */
    public static String getVal(String code, String val) {
        DictVO dict = getDictByCodeAndVal(code, val);
        return dict == null ? null : dict.getName();
    }

    /**
     * 根据code 和 原始值 获取数据字典name
     */
    public static String getNamesByValues(String code, String values) {
        String[] split = values.split(",");
        List<String> names = Lists.newArrayList();
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO dict : dictList) {
            for (String s : split) {
                if (StringUtils.isNotEmpty(dict.getId()) && dict.getVal().equals(s)) {
                    names.add(dict.getName());
                }
            }
        }
        return StringUtils.join(names, ",");
    }

    /**
     * 根据code 和 name 获取数据字典原始值 下级
     */
    public static String getValByName(String code, String name) {
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO dict : dictList) {
            if (StringUtils.isNotEmpty(dict.getId()) && dict.getName().equals(name)) {
                return dict.getVal();
            }
        }
        return null;
    }

    public static String getNameByVal(String code, Object val) {
        String valStr = val == null ? "" : String.valueOf(val);
        return getNameByVal(code, valStr);
    }

    /**
     * 根据code 和 name 获取数据字典原始值 下级
     */
    public static String getNameByVal(String code, String val) {
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO dict : dictList) {
            if (StringUtils.isNotEmpty(dict.getId()) && dict.getVal().equals(val)) {
                return dict.getName();
            }
        }
        return null;
    }

    public static String getLocaleEnumVal(String code, String val) {
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO dict : dictList) {
            if (StringUtils.isNotEmpty(dict.getId()) && dict.getVal().equals(val)) {
                return dict.getName();
            }
        }
        return null;
    }

    /**
     * 根据val 和 原始值 获取数据字典name
     */
    public static String getVal(String code, String val, String defaultStr) {
        DictVO dict = getDictByCodeAndVal(code, val);
        return dict == null ? defaultStr : dict.getName();
    }

    /**
     * 根据code 和 编码 获取数据字典name
     */
    public static String getCode(String code, String codeVal) {
        DictVO dict = null;
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO item : dictList) {
            if (StringUtils.isNotEmpty(item.getId()) && item.getCode().equals(codeVal)) {
                dict = item;
                break;
            }
        }
        return dict == null ? null : dict.getName();
    }

    /**
     * 根据code 和 原始值 获取数据字典对象
     */
    public static DictVO getValItem(String code, String val) {
        DictVO dict = getDictByCodeAndVal(code, val);
        return dict;
    }

    /**
     * 根据code 和 编码 获取数据字典对象
     */
    public static DictVO getCodeItem(String code, String codeVal) {
        DictVO dict = null;
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO item : dictList) {
            if (StringUtils.isNotEmpty(item.getId()) && item.getCode().equals(codeVal)) {
                dict = item;
                break;
            }
        }
        return dict;
    }

    /**
     * 根据code 和 编码 获取数据字典对象
     */
    public static DictVO getCodeItemByVal(String code, String val) {
        DictVO dict = null;
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO item : dictList) {
            if (StringUtils.isNotEmpty(item.getId()) && item.getVal().equals(val)) {
                dict = item;
                break;
            }
        }
        return dict;
    }

    /**
     * 根据code 和 编码 获取数据字典对象
     */
    public static String getCodeValByVal(String code, String val) {
        DictVO dict = null;
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO item : dictList) {
            if (StringUtils.isNotEmpty(item.getId()) && item.getVal().equals(val)) {
                dict = item;
                break;
            }
        }
        return dict == null ? null : dict.getVal();
    }

    /**
     * 根据code 和 编码 获取数据字典对象
     */
    public static String getCodeNameByVal(String code, String val) {
        return getCodeNameByVal(code, val, null);
    }

    public static String getCodeNameByVal(String code, String val, String defaultValue) {
        DictVO dict = null;
        List<DictVO> dictList = getDictListBycode(code);
        for (DictVO item : dictList) {
            if (StringUtils.isNotEmpty(item.getId()) && item.getVal().equals(val)) {
                dict = item;
                break;
            }
        }
        return dict == null ? defaultValue : dict.getName();
    }

    /**
     * 根据code 和 编码 获取数据字典对象
     */
    public static DictVO getCodeItem(String code) {
        DictVO dict = null;
        List<DictVO> list = DictUtils.getDictList();
        for (DictVO item : list) {
            if (item.getCode().equals(code)) {
                dict = item;
            }
        }
        return dict;
    }

    /**
     * 根据code 和 编码 获取数据字典对象 本级
     */
    public static String getCodeItemVal(String code) {
        DictVO dict = getCodeItem(code);
        return dict == null ? null : dict.getVal();
    }

    public static List<DictVO> getDictList(DictQuery dictQuery) {
        if (StringUtils.isNotEmpty(dictQuery.getFilter())) {
            return getDictListFilterVal(dictQuery.getCode(), dictQuery.getFilter());
        }
        return getDictList(dictQuery.getCode());
    }

    public static List<DictVO> getDictList(String code) {
        List<DictVO> itemList = Lists.newArrayList();
        itemList.addAll(getDictListBycode(code));
        return itemList;
    }

    public static List<DictVO> getDictListFilterVal(String code, String filters) {
        List<DictVO> itemList = Lists.newArrayList();
        List<String> filterList = StringUtils.isEmpty(filters) ? null : Lists.newArrayList(filters.split(","));
        for (DictVO item : getDictListBycode(code)) {
            if ((filterList == null || filterList.isEmpty()) || !filterList.contains(item.getVal())) {
                itemList.add(item);
            }
        }
        return itemList;
    }

    public static List<DictVO> getAllDictList(String code) {
        return getDictListBycode(code);
    }

    private static DictVO getDictByCodeAndVal(String code, String val) {
        List<DictVO> dictList = getDictListBycode(code);
        return dictList.stream()
            .filter(dict -> StringUtils.isNotEmpty(dict.getId()) && dict.getVal().equals(val))
            .findFirst().orElse(null);
    }

    private static List<DictVO> getDictListBycode(String code) {
        List<DictVO> dictList = getDictMap().get(code);
        return Optional.ofNullable(dictList).orElse(Lists.newArrayList());
    }

}
