package com.c503.ca.cadv.module.sys.web;

import com.c503.ca.cadv.common.mvc.domain.BaseEntity;
import com.c503.ca.cadv.common.mvc.domain.vo.SelectResult;
import com.c503.ca.cadv.common.rest.base.TreeVOResource;
import com.c503.ca.cadv.common.util.EnumUtils;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.module.config.annotation.RepeatSubmit;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.c503.ca.cadv.module.sys.domain.Dict;
import com.c503.ca.cadv.module.sys.domain.query.*;
import com.c503.ca.cadv.module.sys.domain.vo.DictVO;
import com.c503.ca.cadv.module.sys.service.DictService;
import com.c503.ca.cadv.module.sys.util.DictUtils;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * REST controller for managing Station.
 *
 * @author liaow
 */
@Controller
@RequestMapping("/dict")
@Api(tags = "sys_dict")
public class DictResource extends TreeVOResource<DictService, DictVO> {

    public DictResource(DictService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取字典分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        pm.setSortDefaultName(Direction.DESC, Dict.F_SORT, Dict.F_LASTMODIFIEDDATE);
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存字典")
    @MyLog(value = "保存字典", type = "id", model = "字典管理")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<String>> save(@Valid @RequestBody DictVO dictVO) {
        if (StringUtils.isNotEmpty(dictVO.getId())) {
            if (service.count(service.createQuery().lambda().ne(Dict::getId, dictVO.getId()).eq(Dict::getCode, dictVO.getCode())) > 0) {
                throw new RuntimeErrorException("编码已存在");
            }
        } else {
            if (service.count(service.createQuery().lambda().eq(Dict::getCode, dictVO.getCode())) > 0) {
                throw new RuntimeErrorException("编码已存在");
            }
        }

        if (StringUtils.isNotEmpty(dictVO.getParentId())) {
            Dict parent = service.findOne(dictVO.getParentId());
            if (parent != null) {
                dictVO.setParentCode(parent.getCode());
            }
        }

        service.saveOrUpdate(dictVO);
        DictUtils.clearCache();
        return ResultBuilder.buildOk("保存", dictVO.getName(), "成功");
    }

    @DeleteMapping(value = "/{ids:" + BaseEntity.ENTITY_ID_REGEX + "}")
    @ApiOperation(value = "删除字典")
    @MyLog(value = "删除字典", type = OperateType.DELETE, model = "字典管理")
    public ResponseEntity<CustomMessage<String>> delete(@PathVariable String ids) {
        service.deleteByParentIds(Arrays.asList(ids.split(StringUtils.SPLIT_DEFAULT)), "admin");
        DictUtils.clearCache();
        return ResultBuilder.buildOk("api.common.deleteSuccess");
    }

    @GetMapping(value = "/formData")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "字典ID", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "parentId", value = "父节点ID", paramType = "query", dataTypeClass = String.class),
    })
    @ApiOperation(value = "获取表单详情")
    public ResponseEntity<CustomMessage<DictVO>> formData(@ApiIgnore DictVO dictVO) {
        if (dictVO == null) {
            throw new RuntimeErrorException(StringUtils.toAppendStr("查询模块失败，原因：无法查找到编号区域"));
        }
        if (StringUtils.isEmpty(dictVO.getId()) && StringUtils.isNotEmpty(dictVO.getParentId())) {
            Dict parent = service.getById(dictVO.getParentId());
            if (parent != null) {
                dictVO.setParentName(parent.getName());
                dictVO.setCode(parent.getCode() + "_");
            }
            service.findOptionalTopByParentId(dictVO.getParentId())
                .ifPresent(item -> dictVO.setSort(item.getSort() + 10));
        }
        if (StringUtils.isNotEmpty(dictVO.getId())) {
            Dict entity = service.findOne(dictVO.getId());
            Dict parent = service.getById(dictVO.getParentId());
            if (parent != null) {
                entity.setParentName(parent.getName());
            }
            DictVO vo = service.copyBeanToVO(entity);
            return ResultBuilder.buildOk(vo);
        }
        if (dictVO.getSort() == null) {
            dictVO.setSort(10);
        }
        return ResultBuilder.buildOk(dictVO);
    }

    @GetMapping(value = "/getEnumMap")
    @ApiOperation(value = "获取枚举内容")
    public ResponseEntity<CustomMessage<Map<String, String>>> getEnumMap(String enumName) {
        Map<String, String> resultMap = EnumUtils.getIEnumMap().get(enumName);

        return ResultBuilder.buildOk(resultMap);
    }

    @GetMapping(value = "/codes")
    @ApiOperation(value = "获取字典数据")
    public ResponseEntity<CustomMessage<List<DictComboData>>> codes(@ModelAttribute DictQuery dictQuery) {
        List<DictComboData> dataList = Lists.newArrayList();
        if (dictQuery != null && StringUtils.isNotEmpty(dictQuery.getCode())) {
            List<DictVO> dictList = DictUtils.getDictListFilterVal(dictQuery.getCode(), dictQuery.getFilter());
            if (!dictList.isEmpty()) {
                for (DictVO dict : dictList) {
                    DictComboData comboData = new DictComboData(dict.getId(), dict.getVal(), dict.getName(), dict.getCode(), dict.getColor());
                    dataList.add(comboData);
                }
            }
        }
        return ResultBuilder.buildOk(dataList);
    }

    @GetMapping(value = "/codes-batch")
    @ApiOperation(value = "批量获取字典数据,逗号分隔")
    public ResponseEntity<CustomMessage<List<DictComboList>>> codes(String codes) {
        String[] codeArray = StringUtils.splitDefault(codes);
        List<DictComboList> dictComboLists = getDictComboLists(Lists.newArrayList(codeArray));

        return ResultBuilder.buildOk(dictComboLists);
    }

    public List<DictComboList> getDictComboLists(List<String> codeArray) {
        List<DictComboList> dictComboLists = Lists.newArrayList();
        Map<String, List<DictVO>> dictAllMap = DictUtils.getDictMap();
        for (String code : codeArray) {
            List<DictComboData> dataList = Lists.newArrayList();
            if (StringUtils.isNotEmpty(code)) {
                List<DictVO> dictList = dictAllMap.get(code);
                if (dictList != null && !dictList.isEmpty()) {
                    for (DictVO dict : dictList) {
                        DictComboData comboData = new DictComboData(dict.getId(), dict.getVal(), dict.getName(), dict.getCode(), dict.getColor());
                        dataList.add(comboData);
                    }
                    DictComboList dictComboList = new DictComboList();
                    dictComboList.setParentCode(code);
                    dictComboList.setLeafs(dataList);

                    dictComboLists.add(dictComboList);
                }
            }
        }
        return dictComboLists;
    }

    @GetMapping(value = "/findTreeData")
    @ApiOperation(value = "获取树结构列表")
    public ResponseEntity<CustomMessage<List<DictTreeResult>>> findTreeData() {
        List<DictTreeResult> rs = service.assembleTreeData(DictUtils.getDictList());
        return ResultBuilder.buildOk(rs);
    }

    @GetMapping(value = "/findSelectData")
    public ResponseEntity<CustomMessage<Map<String, Object>>> findSelectData(@ModelAttribute DictQuerySearch dictQuerySearch) {
        Map<String, Object> map = Maps.newHashMap();
        if (StringUtils.isNotEmpty(dictQuerySearch.getDictQueries())) {
            List<DictQuery> dictQueries = JacksonUtils.deserializeList(dictQuerySearch.getDictQueries(), DictQuery.class);
            assert dictQueries != null;
            dictQueries.forEach(dictQuery -> map.put(StringUtils.toCamelCase(dictQuery.getCode()),
                DictUtils.getDictList(dictQuery).
                    stream().map(item -> new SelectResult(item.getVal(), item.getName())).collect(Collectors.toList())));
        }
        return ResultBuilder.buildOk(map);
    }

    @GetMapping(value = "/reGenerateI18n")
    @ApiOperation(value = "重新生成中文语言国际化字典")
    public ResponseEntity<CustomMessage<String>> reGenerateI18n() {
        List<Dict> modules = service.findAll();
        Map<String, String> localeMessageMap = Maps.newHashMap();
        StringBuilder builder = new StringBuilder();
        for (Dict dict : modules) {
            if (StringUtils.isNotEmpty(dict.getCode())) {
                String localeMessageKey = String.join(".", "module", dict.getParentCode(), dict.getCode());
                if (!localeMessageMap.containsKey(localeMessageKey)) {
                    builder.append(localeMessageKey);
                    builder.append("=");
                    builder.append(dict.getName());
                    builder.append("\n");
                    localeMessageMap.put(localeMessageKey, dict.getName());
                }
            }
        }
        log.debug(builder.toString());
        return ResultBuilder.buildObject(builder.toString());
    }

}
